import * as React from 'react';
import { collectSections } from './dialog-utils';
import { useSelector, useDispatch } from 'react-redux';
import * as xtremRedux from '../../../redux';
import { XtremTabs } from '../../ui/tabs/xtrem-tabs';
import { resolveByValue } from '../../../utils/resolve-value-utils';
import { StepSequence, StepSequenceItem } from 'carbon-react/esm/components/step-sequence';
import { getPageDefinitionFromState } from '../../../utils/state-utils';
import { getSectionValidationMessage } from '../../../service/validation-service';
const getWizardStepStatus = (index, currentSectionIndex) => {
    if (index === currentSectionIndex) {
        return 'current';
    }
    return index < currentSectionIndex ? 'complete' : 'incomplete';
};
export function DialogHeader({ dialog, setSelectedSection, selectedSection, }) {
    const dispatch = useDispatch();
    const screenDefinitions = useSelector(state => state.screenDefinitions);
    const pageSelectedSection = useSelector(state => dialog.screenId && dialog.type === 'page'
        ? state.screenDefinitions[dialog.screenId]?.activeSection || null
        : null);
    const sectionValidationErrors = useSelector(state => {
        if (dialog.screenId && dialog.type === 'page') {
            const pageDefinition = getPageDefinitionFromState(dialog.screenId, state);
            const sectionIds = pageDefinition.metadata.layout.$items
                .filter(section => !!section.$containerId)
                .map(section => section.$containerId);
            return sectionIds.reduce((result, sectionId) => {
                if (dialog.screenId) {
                    return {
                        ...result,
                        [sectionId]: getSectionValidationMessage(dialog.screenId, sectionId, state),
                    };
                }
                return result;
            }, {});
        }
        return {};
    });
    const onTabChange = React.useCallback((activeSection) => {
        if (dialog.type === 'page' && dialog.screenId) {
            dispatch(xtremRedux.actions.setActiveSection(dialog.screenId, activeSection));
        }
        else {
            setSelectedSection(activeSection);
        }
    }, [dispatch, dialog, setSelectedSection]);
    // For custom dialogs, the selected section is tracked in the local state of the dialog component
    // For page dialogs, the selected section is tracked in the application state.
    const sections = collectSections(dialog, screenDefinitions);
    const selectedTabId = pageSelectedSection || selectedSection || sections[0]?.id || null;
    const useTabs = sections.length > 1 &&
        // The tabs should only be displayed if the content is not page type OR if its a page which is set to tabs mode
        (dialog.type !== 'page' || dialog.content.mode === 'tabs');
    const useWizardSteps = sections.length > 1 &&
        // The tabs should only be displayed if the content is a page that is set to wizard mode
        dialog.type === 'page' &&
        dialog.content.mode === 'wizard';
    if (useTabs) {
        return (React.createElement(XtremTabs, { inSidebar: true, onTabChange: onTabChange, selectedTabId: selectedTabId || '', tabs: sections
                .map(section => ({
                id: section.id,
                title: resolveByValue({
                    propertyValue: section.title,
                    screenId: dialog.screenId || '',
                    fieldValue: null,
                    rowValue: null,
                    skipHexFormat: true,
                }),
                validationMessage: sectionValidationErrors[section.id],
            }))
                .filter(t => !!t.title) }));
    }
    if (useWizardSteps) {
        const currentSectionIndex = sections.findIndex(s => s.id === selectedSection);
        const steps = sections.map((section, index) => ({
            key: section.id,
            id: section.id,
            indicator: String(index + 1),
            status: getWizardStepStatus(index, currentSectionIndex),
            children: resolveByValue({
                propertyValue: section.title,
                screenId: dialog.screenId || '',
                fieldValue: null,
                rowValue: null,
                skipHexFormat: true,
            }),
        }));
        return (React.createElement("div", { className: "e-dialog-header-wizard-steps", "data-testid": "e-dialog-header-wizard-steps" },
            React.createElement(StepSequence, { orientation: "horizontal" }, steps.map(s => (React.createElement(StepSequenceItem, { key: s.key, ...s }))))));
    }
    return null;
}
//# sourceMappingURL=dialog-header.js.map