import { formatDateToCurrentLocale } from '@sage/xtrem-date-time';
import Date from 'carbon-react/esm/components/date';
import I18nProvider from 'carbon-react/esm/components/i18n-provider';
import { carbonLocale } from '../../../utils/carbon-locale';
import uniqueId from 'lodash/uniqueId';
import React from 'react';
import { resolveByValue } from '../../../utils/resolve-value-utils';
import { splitValueToMergedValue } from '../../../utils/transformers';
import { AgGridCellEditorComponent } from '../table/table-component-types';
import { setDefaultAgGridInputStyles, isLeftOrRight, isFinishedEditingPressed, isKeyPressedDateComponent, } from '../../../utils/ag-grid/ag-grid-cell-editor-utils';
import { isDeleteOrBackspace, KEY_BACKSPACE, KEY_DELETE, KEY_F2 } from '../../../utils/keyboard-event-utils';
import { getStore } from '../../../redux';
export default class DateCellEditor extends AgGridCellEditorComponent {
    constructor(props) {
        super(props);
        this.getFormattedDate = () => this.state.value ? formatDateToCurrentLocale(this.state.rawValue, this.state.locale, 'FullDate') : '';
        this.getCarbonProps = () => {
            return {
                autoComplete: 'off',
                name: this.props.column?.getColDef().field,
                className: !this.state.isValid ? ' e-field-error-message-carbon' : '',
                size: 'small',
                id: uniqueId(),
            };
        };
        this.state = this.createInitialState(props);
        this.onKeyDown = this.onKeyDown.bind(this);
        this.handleChange = this.handleChange.bind(this);
    }
    componentDidMount() {
        const input = this.props.eGridCell.querySelector('input');
        if (!input) {
            return;
        }
        // try to match the styles of ag-grid
        setDefaultAgGridInputStyles(input);
        const inputDiv = input.closest('div');
        if (inputDiv) {
            inputDiv.style.width = '100%';
            inputDiv.style.height = '100%';
            inputDiv.style.border = 'none';
        }
        input.focus();
        if (this.state.highlightAllOnFocus) {
            input.select();
            this.setState({
                highlightAllOnFocus: false,
            });
        }
        else {
            // when we started editing, we want the caret at the end, not the start.
            // this comes into play in two scenarios: a) when user hits F2 and b)
            // when user hits a printable character, then on IE (and only IE) the caret
            // was placed after the first character, thus 'apply' would end up as 'pplea'
            const length = input.value ? input.value.length : 0;
            if (length > 0) {
                input.setSelectionRange(length, length);
            }
        }
    }
    handleChange(event) {
        if (event.target?.value) {
            this.setState({ value: event.target.value.formattedValue, rawValue: event.target.value.rawValue });
        }
    }
    onKeyDown(event) {
        if (isLeftOrRight(event) || isDeleteOrBackspace(event)) {
            event.stopPropagation();
            return;
        }
        if (!isFinishedEditingPressed(event) && !isKeyPressedDateComponent(event) && event.preventDefault) {
            event.preventDefault();
        }
    }
    getValue() {
        const date = this.getFormattedDate();
        return date === '' ? this.state.startValue : this.state.rawValue;
    }
    isCancelBeforeStart() {
        return this.cancelBeforeStart;
    }
    isCancelAfterEnd() {
        return !this.state.isValid;
    }
    createInitialState(props) {
        let startValue;
        let highlightAllOnFocus = true;
        if (props.keyPress === KEY_BACKSPACE || props.keyPress === KEY_DELETE) {
            // if backspace or delete pressed, we clear the cell
            startValue = '';
        }
        else if (props.charPress) {
            // if a letter was pressed, we start with the letter
            startValue = props.charPress;
            highlightAllOnFocus = false;
        }
        else {
            // otherwise we start with the current value
            startValue = props.value;
            if (props.keyPress === KEY_F2) {
                highlightAllOnFocus = false;
            }
        }
        return {
            startValue,
            isValid: true,
            value: startValue,
            rawValue: startValue,
            highlightAllOnFocus,
            locale: (getStore().getState().applicationContext?.locale || 'en-US'),
        };
    }
    render() {
        const width = this.props.eGridCell?.style?.width || '200px';
        const { fieldProperties, tableElementId, screenId, data } = this.props;
        const rowValue = splitValueToMergedValue(data);
        const maxDate = resolveByValue({
            screenId,
            propertyValue: fieldProperties.maxDate,
            rowValue,
            skipHexFormat: true,
            fieldValue: null,
        });
        const minDate = resolveByValue({
            screenId,
            propertyValue: fieldProperties.minDate,
            rowValue,
            skipHexFormat: true,
            fieldValue: null,
        });
        const allColumns = this.props.api.getColumns() || [];
        return (React.createElement("div", { className: "ag-cell-edit-wrapper e-nested-cell-field-date", "data-testid": `${tableElementId}-${this.props.rowIndex}-${allColumns.indexOf(this.props.column) + 1}`, style: { width } },
            React.createElement(I18nProvider, { locale: carbonLocale(this.state.locale || 'en-US') },
                React.createElement(Date, { allowEmptyValue: true, onChange: this.handleChange, value: this.state.value || '', maxDate: maxDate, minDate: minDate, ...this.getCarbonProps(), onKeyDown: this.onKeyDown, hasError: !this.state.isValid }))));
    }
}
//# sourceMappingURL=date-cell-editor.js.map