import Button from 'carbon-react/esm/components/button';
import * as React from 'react';
import { connect } from 'react-redux';
import * as tokens from '@sage/design-tokens/js/base/common';
import { localize } from '../../../service/i18n-service';
import { handleChange } from '../../../utils/abstract-fields-utils';
import { getComponentClass, getDataTestIdAttribute } from '../../../utils/dom';
import { triggerFieldEvent } from '../../../utils/events';
import { useFocus } from '../../../utils/hooks/effects/use-focus';
import { Icon } from '../../ui/icon/icon-component';
import { getLabelTitle, isFieldDisabled, isFieldReadOnly } from '../carbon-helpers';
import { mapDispatchToProps, mapStateToProps } from '../field-base-utils';
import { FieldLabel, HelperText } from '../carbon-utility-components';
import { getScreenElement } from '../../../service/screen-base-definition';
import { getPageDefinitionFromState } from '../../../utils/state-utils';
import axios from 'axios';
import { errorDialog } from '../../../service/dialog-service';
import { resolveByValue } from '../../../utils/resolve-value-utils';
import ProgressTracker from 'carbon-react/esm/components/progress-tracker';
const STATUS_CREATED = 'created';
const STATUS_UPLOADED = 'uploaded';
const STATUS_UPLOAD_FAILED = 'uploadFailed';
export function FileDepositComponent(props) {
    const hasValue = props.value?.downloadUrl !== undefined && props.value?.downloadUrl !== null;
    const inputFileRef = React.useRef(null);
    const fileComponentRef = React.useRef(null);
    const [uploadProgress, setUploadProgress] = React.useState(0);
    const [isUploadInProgress, setUploadInProgress] = React.useState(false);
    useFocus(fileComponentRef, props.isInFocus, 'button');
    const isReadOnly = isFieldReadOnly(props.screenId, props.fieldProperties, props.value, null); // Not available as a nested fields
    const isDisabled = isFieldDisabled(props.screenId, props.fieldProperties, props.value, null); // Not available as a nested fields
    const changeEventHandler = React.useCallback(() => triggerFieldEvent(props.screenId, props.elementId, 'onChange'), [props.screenId, props.elementId]);
    const changeClickHandler = React.useCallback(() => triggerFieldEvent(props.screenId, props.elementId, 'onClick'), [props.screenId, props.elementId]);
    const onUploadProgress = (progressEvent) => setUploadProgress(Math.round((progressEvent.loaded / progressEvent.total) * 100));
    const onInputTypeFileChanged = async (event) => {
        if (event.target.files && event.target.files.length > 0) {
            const file = event.target.files[0];
            const targetNode = props.graphApi.node(props.fieldProperties.node);
            const fieldValue = {
                filename: file.name,
                mimeType: file.type,
                lastModified: new Date(file.lastModified).toISOString(),
                contentLength: file.size,
                status: STATUS_CREATED,
            };
            try {
                const createResponse = await targetNode.mutations
                    .create({ _id: true, uploadUrl: true }, {
                    data: fieldValue,
                })
                    .execute();
                fieldValue._id = createResponse._id;
                handleChange(props.elementId, fieldValue, props.setFieldValue, props.validate, changeEventHandler);
                setUploadProgress(0);
                setUploadInProgress(true);
                try {
                    await axios.put(createResponse.uploadUrl, file, { onUploadProgress });
                    fieldValue.status = STATUS_UPLOADED;
                }
                catch (err) {
                    fieldValue.status = STATUS_UPLOAD_FAILED;
                }
                const updateResponse = await targetNode.mutations
                    .update({ _id: true, downloadUrl: true }, { data: fieldValue })
                    .execute();
                fieldValue.downloadUrl = updateResponse.downloadUrl;
                handleChange(props.elementId, fieldValue, props.setFieldValue, props.validate, changeEventHandler);
                setUploadInProgress(false);
            }
            catch (err) {
                errorDialog(props.screenId, localize('@sage/xtrem-ui/file-upload-failed', 'Failed to upload file.'), err);
                setUploadProgress(0);
                setUploadInProgress(false);
            }
        }
    };
    const onFileUploadAreaClick = () => {
        if (!isReadOnly && !isDisabled) {
            inputFileRef.current?.click();
        }
    };
    const onDeleteFile = () => handleChange(props.elementId, null, props.setFieldValue, props.validate, changeEventHandler);
    const renderFileUploadedView = () => {
        const deleteLabel = localize('@sage/xtrem-ui/crud-delete', 'Delete');
        return (React.createElement("div", { className: `e-file-deposit-field-file-uploaded${isDisabled ? ' e-file-deposit-field-file-uploaded-disabled' : ''}`, "data-testid": "e-file-deposit-field-file-uploaded" },
            React.createElement("div", { className: "e-file-deposit-field-details", "data-testid": "e-file-deposit-field-details" },
                React.createElement(Icon, { type: "pdf", my: 1, color: tokens.colorsUtilityMajor300 }),
                React.createElement("a", { className: "e-file-deposit-field-name", "data-testid": "e-file-deposit-field-name", href: isDisabled ? '#' : props.value?.downloadUrl, target: "_blank", download: props.value?.filename, rel: "noreferrer", title: props.value?.filename },
                    React.createElement("span", null, props.value?.filename)),
                !isReadOnly && !isDisabled && (React.createElement(Button, { iconType: "delete", "aria-label": deleteLabel, iconTooltipMessage: deleteLabel, disabled: isDisabled, onClick: isDisabled || isReadOnly ? undefined : onDeleteFile })))));
    };
    const renderUploadArea = () => (React.createElement("div", { "data-testid": "e-file-deposit-field-upload-area", onClick: onFileUploadAreaClick },
        React.createElement("input", { accept: props.fieldProperties.fileTypes, className: "e-hidden", "data-testid": "e-file-deposit-field-file-input", onChange: onInputTypeFileChanged, ref: inputFileRef, type: "file", disabled: isDisabled || isReadOnly }),
        React.createElement(Button, { buttonType: "secondary", iconType: "upload", disabled: isDisabled || isReadOnly, onClick: changeClickHandler }, localize('@sage/xtrem-ui/file-component-browse-files', 'Browse Files'))));
    const renderBody = () => {
        if (isUploadInProgress) {
            return React.createElement(ProgressTracker, { length: "100%", progress: uploadProgress, showDefaultLabels: true });
        }
        if (hasValue) {
            return renderFileUploadedView();
        }
        return renderUploadArea();
    };
    const title = getLabelTitle(props.screenId, props.fieldProperties, null); // Not available as a nested fields
    const { isTitleHidden } = props.fieldProperties;
    const infoMessage = resolveByValue({
        screenId: props.screenId,
        fieldValue: props.value,
        propertyValue: props.fieldProperties.infoMessage,
        rowValue: null,
        skipHexFormat: true,
    });
    const warningMessage = resolveByValue({
        screenId: props.screenId,
        fieldValue: props.value,
        propertyValue: props.fieldProperties.warningMessage,
        rowValue: null,
        skipHexFormat: true,
    });
    const helperText = isUploadInProgress
        ? localize('@sage/xtrem-ui/upload-in-progress', 'Uploading file...')
        : props.fieldProperties.helperText;
    return (React.createElement("div", { "data-testid": getDataTestIdAttribute('file', title, props.elementId), className: getComponentClass(props, 'e-file-deposit-field'), ref: fileComponentRef },
        !isTitleHidden && (React.createElement(FieldLabel, { label: title, errorMessage: props.validationErrors?.[0].message, infoMessage: infoMessage, warningMessage: warningMessage })),
        React.createElement("div", { className: "e-file-deposit-field-content-wrapper", "data-testid": "e-file-deposit-field-content-wrapper" }, renderBody()),
        React.createElement(HelperText, { helperText: helperText })));
}
export const ConnectedFileDepositComponent = connect((state, props) => {
    const screenDefinition = getPageDefinitionFromState(props.screenId, state);
    const screenElement = getScreenElement(screenDefinition);
    const basicProps = mapStateToProps()(state, props);
    return {
        ...props,
        ...basicProps,
        graphApi: screenElement.$.graph,
    };
}, mapDispatchToProps())(FileDepositComponent);
export default ConnectedFileDepositComponent;
//# sourceMappingURL=file-deposit-component.js.map