import React from 'react';
import { localize } from '../../../service/i18n-service';
import { getNumberComponentsFromLocalizedNumberString, parseLocalizedNumberStringToNumber, } from '../../../utils/formatters';
import { resolveByValue } from '../../../utils/resolve-value-utils';
import { splitValueToMergedValue } from '../../../utils/transformers';
import { AgGridCellEditorComponent } from '../table/table-component-types';
import { isCharNumeric, isLeftOrRight, isFinishedEditingPressed, isKeyPressedNumeric, defaultCellEditorStyles, } from '../../../utils/ag-grid/ag-grid-cell-editor-utils';
import { isEsc, isDeleteOrBackspace, KEY_BACKSPACE, KEY_DELETE, KEY_F2 } from '../../../utils/keyboard-event-utils';
export default class NumericEditor extends AgGridCellEditorComponent {
    constructor(props) {
        super(props);
        this.cancelBeforeStart = this.props.charPress && !isCharNumeric(this.props.charPress);
        this.state = this.createInitialState(props);
        this.input = React.createRef();
        this.onKeyDown = this.onKeyDown.bind(this);
        this.handleChange = this.handleChange.bind(this);
        this.isInputLimitedByScale = this.isInputLimitedByScale.bind(this);
    }
    componentDidMount() {
        const input = this.input.current;
        if (!input) {
            return;
        }
        input.focus();
        if (this.state.highlightAllOnFocus) {
            input.select();
            this.setState({
                highlightAllOnFocus: false,
            });
        }
        else {
            // when we started editing, we want the caret at the end, not the start.
            // this comes into play in two scenarios: a) when user hits F2 and b)
            // when user hits a printable character, then on IE (and only IE) the caret
            // was placed after the first character, thus 'apply' would end up as 'pplea'
            const length = input.value ? input.value.length : 0;
            if (length > 0) {
                // The selection cannot be set on number input types, so we temporarily switch the type to text.
                input.setSelectionRange(length, length);
            }
        }
        input.addEventListener('keydown', this.onKeyDown);
    }
    componentWillUnmount() {
        this.input.current?.removeEventListener('keydown', this.onKeyDown);
    }
    handleChange(event) {
        this.setState({ value: event.target.value });
    }
    onKeyDown(event) {
        if (isEsc(event)) {
            return;
        }
        if (isLeftOrRight(event) || isDeleteOrBackspace(event)) {
            event.stopPropagation();
            return;
        }
        if (!isFinishedEditingPressed(event) && !isKeyPressedNumeric(event) && event.preventDefault) {
            event.preventDefault();
        }
        if (isKeyPressedNumeric(event) && this.isInputLimitedByScale(event)) {
            event.preventDefault();
        }
    }
    getValue() {
        return parseLocalizedNumberStringToNumber(this.state.value, localize('@sage/xtrem-ui/number-format-separator', '.'));
    }
    createInitialState(props) {
        let startValue;
        let highlightAllOnFocus = true;
        if (props.keyPress === KEY_BACKSPACE || props.keyPress === KEY_DELETE) {
            // if backspace or delete pressed, we clear the cell
            startValue = '';
        }
        else if (props.charPress) {
            // if a letter was pressed, we start with the letter
            startValue = props.charPress;
            highlightAllOnFocus = false;
        }
        else {
            // otherwise we start with the current value
            startValue = props.value;
            if (props.keyPress === KEY_F2) {
                highlightAllOnFocus = false;
            }
        }
        return {
            value: startValue,
            highlightAllOnFocus,
        };
    }
    isCancelBeforeStart() {
        return this.cancelBeforeStart;
    }
    // will reject the number if it greater than Number.MAX_SAFE_INTEGER
    isCancelAfterEnd() {
        return Number(this.state.value) > Number.MAX_SAFE_INTEGER;
    }
    isInputLimitedByScale(event) {
        const components = getNumberComponentsFromLocalizedNumberString(event.currentTarget.value, localize('@sage/xtrem-ui/number-format-separator', '.'));
        const inputPosition = this.input.current?.selectionStart || 0;
        const scale = resolveByValue({
            fieldValue: this.props.value,
            propertyValue: this.props.colDef?.cellEditorParams?.fieldProperties?.scale,
            rowValue: splitValueToMergedValue(this.props.data),
            screenId: this.props.colDef.screenId,
            skipHexFormat: true,
        });
        // Allow unlimited decimals if scale is undefined.
        if (typeof scale !== 'number') {
            return false;
        }
        // Ignore integer values if value does not contain decimal seperator.
        if (!event.currentTarget.value.includes(localize('@sage/xtrem-ui/number-format-separator', '.'))) {
            return false;
        }
        // Limit decimals if scale is exceeded.
        if (scale <= components.decimal.length) {
            // The line below is added to allow the user to insert numbers before the decimal separator
            // and replace selections with numeric inputs.
            return inputPosition > event.currentTarget.value.length - components.decimal.length - 1;
        }
        return false;
    }
    render() {
        return (React.createElement("div", { className: "ag-cell-edit-wrapper" },
            React.createElement("input", { className: "e-numeric-cell-editor", "data-testid": `${this.props.tableElementId}-${this.props.rowIndex}-${this.props.api.getColumns().indexOf(this.props.column) + 1}`, ref: this.input, type: "text", value: this.state.value, onChange: this.handleChange, style: defaultCellEditorStyles })));
    }
}
//# sourceMappingURL=numeric-cell-editor.js.map