import { debounce } from 'lodash';
import * as React from 'react';
import { connect } from 'react-redux';
import { localize } from '../../../service/i18n-service';
import { handleChange } from '../../../utils/abstract-fields-utils';
import { xtremConsole } from '../../../utils/console';
import { triggerFieldEvent } from '../../../utils/events';
import { formatNumericValue } from '../../../utils/formatters';
import { isDevMode } from '../../../utils/window';
import { Decimal } from '../../ui/decimal/decimal-component';
import { getCommonCarbonComponentProperties } from '../carbon-helpers';
import { CarbonWrapper } from '../carbon-wrapper';
import { EditableFieldBaseComponent, mapDispatchToProps, mapStateToProps } from '../field-base-component';
import { resolveByValue } from '../../../utils/resolve-value-utils';
import { isKeyPressedNumeric } from '../../../utils/ag-grid/ag-grid-cell-editor-utils';
export class NumericComponent extends EditableFieldBaseComponent {
    constructor(props) {
        super(props);
        this.decimalRef = React.createRef();
        this.onInputValueChanged = debounce(async (searchText) => {
            await triggerFieldEvent(this.props.screenId, this.props.elementId, 'onInputValueChange', searchText);
        }, 150);
        this.onChange = async (event) => {
            const eventValue = event.target.value;
            const rawValue = eventValue.rawValue;
            this.setState({ value: rawValue });
            await this.onInputValueChanged(eventValue.rawValue || eventValue.formattedValue);
        };
        this.onBlur = () => {
            const carbonProps = getCommonCarbonComponentProperties(this.props);
            carbonProps.onBlur();
            if (this.state.value === '-') {
                handleChange(this.props.elementId, 0, this.props.setFieldValue, this.props.validate, this.triggerChangeListener);
            }
            else if (this.state.value !== '') {
                if (isDevMode() && this.state.value.replace(/^\D+/g, '').replace('.', '').length > 16) {
                    xtremConsole.warn(`The number supplied in the input is greater than what JavaScript can safely handle: ${this.state.value}`);
                }
                const sep = localize('@sage/xtrem-ui/number-format-separator', '.');
                const parts = this.state.value.split(sep);
                const stateNumberValue = !Number.isNaN(parseFloat(this.state.value))
                    ? parseFloat(this.state.value)
                    : parts.length > 0
                        ? Number(`${parts[0] || 0}.${parts[1] || 0}`)
                        : 0;
                const propNumberValue = parseFloat(String(this.props.value));
                if (stateNumberValue !== propNumberValue) {
                    handleChange(this.props.elementId, stateNumberValue, this.props.setFieldValue, this.props.validate, this.triggerChangeListener);
                }
            }
            else if (this.props.value !== (this.state.value || null)) {
                handleChange(this.props.elementId, this.state.value || null, this.props.setFieldValue, this.props.validate, this.triggerChangeListener);
            }
        };
        this.onKeyDown = (event) => {
            const { key } = event;
            if (key === 'Enter') {
                this.onBlur();
                return;
            }
            if (key === 'Escape') {
                this.setState({
                    value: this.props.value
                        ? formatNumericValue({
                            screenId: this.props.screenId,
                            value: this.props.value,
                            ignoreLocale: true,
                            scale: this.props.fieldProperties.scale,
                            rowValue: this.props.handlersArguments?.rowValue,
                        })
                        : '',
                });
                return;
            }
            if ([
                'ArrowLeft',
                'ArrowRight',
                'Tab',
                'Control',
                'Meta',
                'Shift',
                'Delete',
                'Backspace',
                'End',
                'Home',
                'PageUp',
                'PageDown',
            ].indexOf(key) !== -1 ||
                event.altKey ||
                event.metaKey ||
                event.ctrlKey) {
                return;
            }
            // Discard start event if user starts with the decimal separator
            const target = event.currentTarget;
            const targetValue = target.value;
            const currentValueLength = targetValue.length;
            const sep = localize('@sage/xtrem-ui/number-format-separator', '.');
            const currentSelectionLength = (target.selectionEnd || 0) - (target.selectionStart || 0);
            if ((event.key === sep && currentValueLength - currentSelectionLength === 0) || !isKeyPressedNumeric(event)) {
                event.preventDefault();
                event.stopPropagation();
            }
            // If any part of value is selected then the key would overwrite the particular key so we don't need to check the scale
            if (target.selectionStart === target.selectionEnd) {
                const scale = resolveByValue({
                    propertyValue: this.props.fieldProperties.scale,
                    screenId: this.props.screenId,
                    rowValue: this.props.handlersArguments?.rowValue,
                    skipHexFormat: true,
                    fieldValue: null,
                });
                const parts = targetValue.split(sep);
                // If the user tries to use the separator character in an integer field.
                if (scale === 0 && (parts[1] === '' || event.key === sep)) {
                    event.preventDefault();
                    event.stopPropagation();
                }
                /**
                 * When the user is about to type in more decimal characters than allowed. We need to check the position of the cursor, we only want to stop the event if the cursor is in the decimal part
                 *  */
                if (scale && parts[1] && target.selectionStart > parts[0].length && parts[1].length + 1 > scale) {
                    event.preventDefault();
                    event.stopPropagation();
                }
            }
        };
        this.onKeyUp = (event) => {
            const sep = localize('@sage/xtrem-ui/number-format-separator', '.');
            const scale = resolveByValue({
                propertyValue: this.props.fieldProperties.scale,
                screenId: this.props.screenId,
                rowValue: this.props.handlersArguments?.rowValue,
                skipHexFormat: true,
                fieldValue: null,
            });
            if (event.key === sep && !event.currentTarget.value.endsWith(sep) && scale !== 0) {
                this.setState({ value: sep });
            }
        };
        this.state = {
            value: formatNumericValue({
                screenId: props.screenId,
                value: props.value,
                ignoreLocale: true,
                scale: props.fieldProperties.scale,
                rowValue: props.handlersArguments?.rowValue,
            }),
        };
    }
    UNSAFE_componentWillReceiveProps(nextProps) {
        const value = formatNumericValue({
            screenId: nextProps.screenId,
            value: nextProps.value,
            ignoreLocale: true,
            scale: nextProps.fieldProperties.scale,
            rowValue: nextProps.handlersArguments?.rowValue,
        });
        if (this.state.value !== value) {
            this.setState({ value });
        }
    }
    render() {
        const resolvedScale = this.getResolvedProperty('scale');
        const decimalProperties = {
            ...getCommonCarbonComponentProperties(this.props),
            onBlur: this.onBlur,
            onChange: this.onChange,
            onClick: this.getClickHandler(),
            onKeyDown: this.onKeyDown,
            onKeyUp: this.onKeyUp,
            precision: resolvedScale === null || resolvedScale === undefined || Number.isNaN(Number(resolvedScale))
                ? 0
                : resolvedScale,
            styleOverride: {
                input: {
                    textAlign: 'right',
                },
            },
            value: this.state.value,
            inputIcon: this.props.fieldProperties.icon,
        };
        const readOnlyValue = Number.isNaN(Number(this.state.value))
            ? this.state.value
            : formatNumericValue({
                screenId: this.props.screenId,
                value: this.state.value,
                scale: this.props.fieldProperties.scale,
                rowValue: this.props.handlersArguments?.rowValue,
            });
        return (React.createElement(CarbonWrapper, { ...this.props, className: `e-numeric-field${resolvedScale ? ' e-decimal-field' : ''}`, componentName: "numeric", componentRef: this.componentRef, handlersArguments: this.props.handlersArguments, noReadOnlySupport: true, value: readOnlyValue },
            React.createElement(Decimal, { ref: this.decimalRef, ...decimalProperties, onKeyDown: this.onKeyDown, onKeyUp: this.onKeyUp })));
    }
}
export const ConnectedNumericComponent = connect(mapStateToProps(), mapDispatchToProps())(NumericComponent);
export default ConnectedNumericComponent;
//# sourceMappingURL=numeric-component.js.map