import React from 'react';
import * as xtremRedux from '../../../redux';
import { fetchReferenceFieldSuggestions } from '../../../service/graphql-service';
import { triggerFieldEvent } from '../../../utils/events';
import { cleanMetadataAndNonPersistentIdFromRecord, splitValueToMergedValue } from '../../../utils/transformers';
import LookupDialog from '../../ui/lookup-dialog/lookup-dialog-component';
import { Select } from '../../ui/select/select-component';
import { getReferenceSelectedRecord, hasLookupIcon, nodesToSelectItems, nodeToSelectItem } from './reference-utils';
import uid from 'uid';
import { get } from 'lodash';
import { shouldRenderDropdownAbove } from '../../../utils/table-component-utils';
import { LOOKUP_DIALOG_INTERACTION, manageOnBlurFocus } from '../../ui/table-shared/cell/cell-editor-utils';
import { AgGridCellEditorComponent } from '../table/table-component-types';
import { setDefaultAgGridInputStyles } from '../../../utils/ag-grid/ag-grid-cell-editor-utils';
import { TAB, SHIFT_TAB, KEY_BACKSPACE, KEY_DELETE, KEY_F2 } from '../../../utils/keyboard-event-utils';
export default class ReferenceCellEditor extends AgGridCellEditorComponent {
    constructor(props) {
        super(props);
        this.isSelecting = false;
        this.getItems = async (filterValue) => {
            const level = this.props.node?.data?.__level;
            const nodes = await fetchReferenceFieldSuggestions({
                fieldProperties: this.props.fieldProperties,
                screenId: this.props.screenId,
                fieldId: this.props.elementId.split('.').slice(0, 2).join('.'),
                filterValue,
                parentElementId: this.props.tableElementId,
                recordContext: splitValueToMergedValue(cleanMetadataAndNonPersistentIdFromRecord(this.props.data)),
                level,
                contextNode: String(this.props.contextNode),
            });
            return nodesToSelectItems({ nodes, fieldProperties: this.props.fieldProperties });
        };
        this.onKeyDown = (event) => {
            if (event.key === TAB) {
                if (event.shiftKey) {
                    this.setState({ endAction: SHIFT_TAB });
                }
                else {
                    this.setState({ endAction: TAB });
                }
            }
        };
        this.onBlur = (endAction) => () => {
            manageOnBlurFocus(this.isSelecting, this.props.api, this.props.column, this.props.rowIndex, this.props.node.isRowPinned() ? 'top' : null, this.props.stopEditing, endAction ?? this.state.endAction);
        };
        this.onSelected = (endAction) => (selectedItem) => {
            this.isSelecting = true;
            this.setState({ selectedRecord: selectedItem ?? undefined, currentValue: selectedItem?.value || '' }, () => {
                this.isSelecting = false;
                this.onBlur(endAction)();
            });
        };
        this.isLookupDialogOpen = () => this.state.isLookupPanelOpen && !!this.props.fieldProperties.columns;
        this.onLookupDialogSelectionFinished = ([collectionItem] = []) => {
            if (collectionItem) {
                this.onSelected(LOOKUP_DIALOG_INTERACTION)(nodeToSelectItem({ collectionItem, fieldProperties: this.props.fieldProperties }));
            }
            else {
                this.onSelected(LOOKUP_DIALOG_INTERACTION)();
            }
            this.closeLookupDialog();
        };
        this.closeLookupDialog = () => {
            triggerFieldEvent(this.props.screenId, this.props.elementId, 'onCloseReferenceDialog');
            xtremRedux.getStore().dispatch(xtremRedux.actions.setFieldProperties(this.props.screenId, this.props.elementId, {
                ...this.props.fieldProperties,
                isReferenceDialogOpen: false,
            }));
            this.setState({ isLookupPanelOpen: false });
        };
        this.openLookupDialog = (event) => {
            event.stopPropagation();
            event.preventDefault();
            triggerFieldEvent(this.props.screenId, this.props.elementId, 'onOpenReferenceDialog');
            const referenceValue = get(this.props.data, this.props.colDef.field);
            const dispatch = xtremRedux.getStore().dispatch;
            dispatch(xtremRedux.actions.setFieldProperties(this.props.screenId, this.props.elementId, referenceValue));
            dispatch(xtremRedux.actions.setFieldProperties(this.props.screenId, this.props.elementId, {
                ...this.props.fieldProperties,
                isReferenceDialogOpen: true,
            }));
            dispatch(xtremRedux.actions.removeNonNestedErrors(this.props.screenId, this.props.elementId));
            this.setState({ isLookupPanelOpen: true });
        };
        this.onInputChange = (value) => {
            this.setState({ currentValue: value });
        };
        this.getCurrentValue = () => {
            return this.state.currentValue ?? '';
        };
        this.onOpenTunnel = async (isNew = false) => {
            const dispatch = xtremRedux.getStore().dispatch;
            const newValue = await dispatch(xtremRedux.actions.openTunnel({
                elementId: this.props.elementId,
                parentElementId: this.props.tableElementId,
                screenId: this.props.screenId,
                recordContext: this.props.data,
                contextNode: this.props.contextNode,
                fieldProperties: this.props.fieldProperties,
                value: isNew ? null : this.props.value,
            }));
            // Only update the value if the tunnel results some changes. If null returned, no changes were done.
            if (newValue) {
                this.onLookupDialogSelectionFinished([newValue]);
            }
        };
        this.handleOnCreateNewItemLinkClick = () => {
            this.onOpenTunnel(true);
        };
        this.selectRef = React.createRef();
        this.state = this.createInitialState(props);
    }
    componentDidMount() {
        const input = this.selectRef.current;
        if (!input) {
            return;
        }
        // try to match the styles of ag-grid
        setDefaultAgGridInputStyles(input);
        const inputDiv = input.parentNode;
        inputDiv.style.width = '100%';
        inputDiv.style.height = '100%';
        inputDiv.style.border = 'none';
        input.focus();
        if (this.state.highlightAllOnFocus) {
            input.select();
            this.setState({
                highlightAllOnFocus: false,
            });
        }
        else {
            // when we started editing, we want the caret at the end, not the start.
            // this comes into play in two scenarios: a) when user hits F2 and b)
            // when user hits a printable character, then on IE (and only IE) the caret
            // was placed after the first character, thus 'apply' would end up as 'pplea'
            const length = input.value ? input.value.length : 0;
            if (length > 0) {
                input.setSelectionRange(length, length);
            }
        }
    }
    async componentWillUnmount() {
        /**
         * The standard auto selection implementation doesn't work here because the component gets unmounted by the time it would be triggered,
         * so a simple, custom implementation is needed */
        if (!this.state.currentValue ||
            !this.props.node ||
            !this.props.column ||
            !this.props.fieldProperties.isAutoSelectEnabled) {
            return;
        }
        const result = await this.getItems(this.state.currentValue);
        if (result.length !== 1 || !result[0].__collectionItem) {
            return;
        }
        this.props.node.setDataValue(this.props.column, result[0].__collectionItem);
    }
    getValue() {
        /*
        This function is called by ag-grid when the cell is being saved.
        If the user removes the input the selected record is not updated and the value is not saved.
        So in order to avoid this problem we need to manually update the selected record to set it to null when Ag-grid calls this function before save.
        */
        if (!this.state.currentValue && !this.isLookupDialogOpen()) {
            this.setState({ selectedRecord: undefined });
            return null;
        }
        return this.state.selectedRecord?.__collectionItem ?? null;
    }
    getWidth() {
        return this.props.eGridCell?.style?.width ?? '200px';
    }
    createInitialState(props) {
        let startValue;
        let highlightAllOnFocus = true;
        if (props.keyPress === KEY_BACKSPACE || props.keyPress === KEY_DELETE) {
            // if backspace or delete pressed, we clear the cell
            startValue = '';
        }
        else if (props.charPress) {
            // if a letter was pressed, we start with the letter
            startValue = props.charPress;
            highlightAllOnFocus = false;
        }
        else {
            // otherwise we start with the current value
            startValue = props.value;
            if (props.keyPress === KEY_F2) {
                highlightAllOnFocus = false;
            }
        }
        return {
            id: uid(16),
            isLookupPanelOpen: false,
            highlightAllOnFocus,
            selectedRecord: getReferenceSelectedRecord({
                value: get(this.props.data, this.props.colDef.field),
                fieldProperties: this.props.fieldProperties,
            }),
            startValue,
            currentValue: startValue,
        };
    }
    render() {
        const dataTestId = `${this.props.tableElementId}-${this.props.rowIndex}-${this.props.api.getColumns().indexOf(this.props.column) + 1}`;
        const hasLookup = hasLookupIcon({
            fieldProperties: this.props.fieldProperties,
            screenId: this.props.screenId,
            value: splitValueToMergedValue(this.props.data[this.props.colDef.field]),
            rowValue: splitValueToMergedValue(this.props.data),
        });
        const width = this.getWidth();
        let inputWidth = `${width} - 32px`;
        if (this.props.fieldProperties.imageField) {
            inputWidth += ' - 34px';
        }
        return (React.createElement("div", { "data-testid": dataTestId, style: { width }, className: "e-reference-cell-editor" },
            React.createElement(Select, { allowClearOnTab: this.props.fieldProperties.isAutoSelectEnabled, allowSelectOnTab: this.props.fieldProperties.isAutoSelectEnabled, autoSelect: this.props.fieldProperties.isAutoSelectEnabled, defaultIsOpen: !hasLookup, elementId: this.props.elementId, escapeBehavior: "revert", getItems: this.getItems, hasLookupIcon: hasLookup, initialInputValue: this.state.startValue, inputWidth: `calc(${inputWidth})`, isDropdownDisabled: this.props.fieldProperties.isDropdownDisabled, isSortedAlphabetically: true, isInTable: true, lookupIconId: `e-reference-cell-editor-lookup-icon-${this.state.id}`, minLookupCharacters: this.props.fieldProperties.minLookupCharacters, onBlur: this.onBlur(), onKeyDown: this.onKeyDown, onLookupIconClick: this.openLookupDialog, onSelected: this.onSelected(), placeholder: this.props.fieldProperties.placeholder, preventSelectionOnBlur: this.isLookupDialogOpen(), ref: this.selectRef, screenId: this.props.screenId, selectedItem: this.state.selectedRecord, shouldFilterItems: false, shouldRenderOptionsAbove: shouldRenderDropdownAbove({
                    isPhantomRow: !!this.props.data?.__phantom,
                    pageSize: this.props.api.paginationGetPageSize(),
                    rowIndex: this.props.rowIndex,
                }), testId: `${dataTestId}-input`, variant: "plain", onInputChange: this.onInputChange, createTunnelLinkText: this.props.fieldProperties.createTunnelLinkText, onCreateNewItemLinkClick: this.handleOnCreateNewItemLinkClick, isLinkCreateNewText: !!this.props.fieldProperties.tunnelPage, node: this.props.fieldProperties.node }),
            React.createElement(LookupDialog, { isOpen: this.isLookupDialogOpen(), fieldProperties: this.props.fieldProperties, screenId: this.props.screenId, onSelectionFinished: this.onLookupDialogSelectionFinished, fieldId: this.props.elementId, closeLookupDialog: () => {
                    this.closeLookupDialog();
                    this.onBlur(LOOKUP_DIALOG_INTERACTION)();
                }, parentElementId: this.props.tableElementId, contextNode: String(this.props.contextNode), recordContext: splitValueToMergedValue(this.props.data), level: this.props.node?.data?.__level, createTunnelLinkText: this.props.fieldProperties.createTunnelLinkText, onCreateNewItemLinkClick: this.handleOnCreateNewItemLinkClick, isLinkCreateNewText: !!this.props.fieldProperties.tunnelPage })));
    }
}
//# sourceMappingURL=reference-cell-editor.js.map