import { useCombobox } from 'downshift';
import { debounce, noop } from 'lodash';
import * as React from 'react';
import { connect } from 'react-redux';
import uid from 'uid';
import { fetchReferenceFieldSuggestions } from '../../../service/graphql-service';
import { SoundApi } from '../../../service/sound-api';
import { ContextType } from '../../../types';
import { handleChange } from '../../../utils/abstract-fields-utils';
import { triggerFieldEvent } from '../../../utils/events';
import { splitValueToMergedValue } from '../../../utils/transformers';
import { text } from '../../nested-fields';
import LookupDialog from '../../ui/lookup-dialog/lookup-dialog-component';
import { Portrait } from '../../ui/portrait-component';
import { Select } from '../../ui/select/select-component';
import { getCommonCarbonComponentProperties, getLabelTitle } from '../carbon-helpers';
import { CarbonWrapper } from '../carbon-wrapper';
import { EditableFieldBaseComponent, mapDispatchToProps, mapStateToProps } from '../field-base-component';
import { getReferenceHelperText, getReferenceSearchText, getReferenceSelectedRecord, hasLookupIcon, nodesToSelectItems, } from './reference-utils';
import { resolveByValue } from '../../../utils/resolve-value-utils';
import * as xtremRedux from '../../../redux';
export class ReferenceComponent extends EditableFieldBaseComponent {
    constructor(props) {
        super(props);
        this.lookupButtonRef = React.createRef();
        this.onInputValueChanged = debounce(async (searchText) => {
            if (!this.props.parentElementId) {
                await triggerFieldEvent(this.props.screenId, this.props.elementId, 'onInputValueChange', searchText);
            }
        }, 150);
        this.onChange = (collectionItem, isOrganicChange) => {
            if (isOrganicChange) {
                this.closeLookupDialog();
                handleChange(this.props.elementId, collectionItem ?? null, this.props.setFieldValue, this.props.validate, this.triggerChangeListener);
            }
        };
        this.onMobileButtonClick = (event) => {
            event.stopPropagation();
            event.preventDefault();
            if (this.selectInputRef.current) {
                this.searchText = this.selectInputRef.current.value ?? '';
                this.openLookupDialog();
            }
        };
        this.mobileAutoSelect = debounce(async (type) => {
            const isDifferentSearchText = getReferenceSearchText({
                value: this.props.value,
                fieldProperties: this.props.fieldProperties,
            }) !== this.selectInputRef.current?.value;
            const isOrganicChange = this.selectInputRef.current?.value !== '' && type !== useCombobox.stateChangeTypes.FunctionSelectItem;
            if (!this.selectInputRef.current?.value) {
                this.onChange(undefined, isOrganicChange);
                return;
            }
            if (this.props.fieldProperties.isAutoSelectEnabled) {
                await this.handleAutoSelection(isOrganicChange, isDifferentSearchText);
            }
            else if (isOrganicChange) {
                this.searchText = this.selectInputRef.current.value;
                this.openLookupDialog();
            }
        }, 200);
        this.onButtonClick = (event) => {
            event.stopPropagation();
            event.preventDefault();
            this.openLookupDialog();
        };
        this.openLookupDialog = () => {
            if (this.props.setFieldProperties) {
                this.props.setFieldProperties(this.props.elementId, {
                    ...this.props.fieldProperties,
                    isReferenceDialogOpen: true,
                });
            }
            const triggerDialogOpen = async () => {
                if (!this.props.parentElementId) {
                    await triggerFieldEvent(this.props.screenId, this.props.elementId, 'onOpenReferenceDialog');
                }
            };
            if (this.props.parentElementId) {
                this.setState({ isNestedLookupDialogOpen: true }, () => triggerDialogOpen);
            }
            else {
                triggerDialogOpen();
            }
        };
        this.closeLookupDialog = async (cb = noop) => {
            if (!this.props.parentElementId) {
                await triggerFieldEvent(this.props.screenId, this.props.elementId, 'onCloseReferenceDialog');
            }
            if (this.props.setFieldProperties) {
                this.props.setFieldProperties(this.props.elementId, {
                    ...this.props.fieldProperties,
                    isReferenceDialogOpen: false,
                });
            }
            const stateUpdate = {};
            if (this.props.parentElementId && this.state.isNestedLookupDialogOpen) {
                stateUpdate.isNestedLookupDialogOpen = false;
            }
            this.setState(stateUpdate, () => {
                cb();
                this.lookupButtonRef.current?.focus();
            });
            if (this.componentRef.current) {
                const input = this.getFocusableElement(this.componentRef.current);
                if (input &&
                    document.activeElement !== input &&
                    (this.lookupButtonRef.current === null || document.activeElement !== this.lookupButtonRef.current)) {
                    input.focus();
                }
            }
        };
        this.onLookupDialogSelectionFinished = ([selectedRecord]) => {
            this.onChange(selectedRecord, true);
            this.lookupButtonRef.current?.focus();
        };
        this.hasLookupIcon = () => hasLookupIcon({
            screenId: this.props.screenId,
            fieldProperties: this.props.fieldProperties,
            value: this.props.value,
            rowValue: this.props.handlersArguments?.rowValue
                ? splitValueToMergedValue(this.props.handlersArguments?.rowValue)
                : this.props.handlersArguments?.rowValue,
        });
        this.isLookupDialogOpen = () => (!!this.props.fieldProperties.isReferenceDialogOpen || !!this.state.isNestedLookupDialogOpen) &&
            !!this.props.fieldProperties.columns;
        this.isLookupDialogOpenMobile = () => !!this.props.fieldProperties.isReferenceDialogOpen && !this.isReadOnly() && !this.isDisabled();
        this.isReadOnlyAndHasPicture = () => !!this.props.fieldProperties.imageField && (this.isReadOnly() || this.isDisabled());
        this.handleOnCreateNewItemLinkClick = () => {
            this.onOpenTunnel(true);
        };
        this.renderMobile = (className) => {
            const carbonProps = getCommonCarbonComponentProperties(this.props);
            const { value, fieldProperties } = this.props;
            const helperText = getReferenceHelperText({ value, fieldProperties });
            const wrapperStyle = helperText && !this.props.fieldProperties.isHelperTextHidden
                ? { alignItems: 'center' }
                : { alignItems: 'flex-end' };
            const searchText = getReferenceSearchText({
                value,
                fieldProperties,
            });
            const initialSelectedItem = getReferenceSelectedRecord({
                value,
                fieldProperties,
            });
            return (React.createElement(CarbonWrapper, { ...this.props, className: className, componentName: "reference", componentRef: this.componentRef, handlersArguments: this.props.handlersArguments, helperText: helperText, noReadOnlySupport: true, readOnlyDisplayValue: searchText },
                React.createElement("div", { className: "e-reference-field", style: wrapperStyle },
                    React.createElement(Select, { ...carbonProps, autoSelect: false, disabled: this.isDisabled(), error: this.props.validationErrors?.[0]?.message, fullWidth: this.props.fieldProperties.isFullWidth, getItems: this.getItems, hasLookupIcon: true, hasHighlightMatchText: true, helperText: helperText, helperTextLink: this.getHelperTextLink(), isLinkHelperText: !!this.props.fieldProperties.tunnelPage, icon: this.props.fieldProperties.icon, onHelperTextLinkClick: this.onOpenTunnel, initialInputValue: searchText, inputId: carbonProps.id, hasClearFieldButton: true, isSortedAlphabetically: true, isDropdownDisabled: true, isHelperTextHidden: this.props.fieldProperties.isHelperTextHidden, label: getLabelTitle(this.props.screenId, this.props.fieldProperties, this.props.handlersArguments?.rowValue), lookupButtonRef: this.lookupButtonRef, lookupIconId: `e-reference-field-lookup-icon-${this.state.id}`, closeIconClassName: "e-ui-select-close-icon", minLookupCharacters: this.getMinLookupCharacters(), onChange: this.onChange, onInputChange: this.onMobileInputChange, onInputFocus: this.props.onFocus, onLookupIconClick: this.onMobileButtonClick, placeholder: this.props.fieldProperties.placeholder, preventSelectionOnBlur: Boolean(this.props.fieldProperties.isReferenceDialogOpen) ||
                            Boolean(this.state.isNestedLookupDialogOpen), readOnly: this.isReadOnly(), ref: this.selectInputRef, selectedItem: initialSelectedItem, testId: `e-reference-field-lookup-search-text-${this.state.id}`, screenId: this.props.screenId, elementId: this.props.elementId, shouldFilterItems: false, isSoundDisabled: resolveByValue({
                            screenId: this.props.screenId,
                            propertyValue: this.props.fieldProperties.isSoundDisabled,
                            skipHexFormat: true,
                            fieldValue: null,
                            rowValue: null,
                        }), createTunnelLinkText: this.props.fieldProperties.createTunnelLinkText, onCreateNewItemLinkClick: this.handleOnCreateNewItemLinkClick, node: this.props.fieldProperties.node, isLinkCreateNewText: !!this.props.fieldProperties.tunnelPage })),
                React.createElement(LookupDialog, { isOpen: this.isLookupDialogOpenMobile(), fieldProperties: this.props.fieldProperties, screenId: this.props.screenId, onSelectionFinished: this.onLookupDialogSelectionFinished, fieldId: this.props.elementId, searchText: this.searchText, closeLookupDialog: this.closeLookupDialog, createTunnelLinkText: this.props.fieldProperties.createTunnelLinkText, onCreateNewItemLinkClick: this.handleOnCreateNewItemLinkClick, isLinkCreateNewText: !!this.props.fieldProperties.tunnelPage })));
        };
        this.onOpenTunnel = async (isNew = false) => {
            const newValue = await this.props.openTunnel(this.props.fieldProperties, isNew ? null : this.props.value);
            // Only update the value if the tunnel results some changes. If null returned, no changes were done.
            if (newValue) {
                this.onChange(newValue, true);
            }
        };
        this.getHelperTextLink = () => {
            if (this.props.fieldProperties.tunnelPage && this.props.value?._id) {
                return `${this.props.fieldProperties.tunnelPage}/${btoa(JSON.stringify({ _id: this.props.value._id }))}`;
            }
            return undefined;
        };
        this.onMobileInputChange = async (searchText, type) => {
            this.mobileAutoSelect(type);
            await this.onInputChange(searchText, type);
        };
        this.onInputChange = async (searchText, type) => {
            if (type !== useCombobox.stateChangeTypes.FunctionSelectItem) {
                await this.onInputValueChanged(searchText);
            }
        };
        this.getPortraitSize = () => {
            if (this.props.contextType === ContextType.table) {
                return 'M';
            }
            if (this.props.fieldProperties.helperTextField || this.props.fieldProperties.helperText) {
                return 'XL';
            }
            return 'L';
        };
        this.getItems = async (filterValue) => {
            const nodes = await fetchReferenceFieldSuggestions({
                fieldProperties: this.props.fieldProperties,
                screenId: this.props.screenId,
                fieldId: this.props.elementId,
                filterValue,
                parentElementId: this.props.parentElementId,
                recordContext: this.props.handlersArguments?.rowValue,
                contextNode: this.props.contextNode,
            });
            return nodesToSelectItems({
                nodes,
                fieldProperties: this.props.fieldProperties,
            });
        };
        this.getMinLookupCharacters = () => this.props.fieldProperties.minLookupCharacters ?? 3;
        this.renderDesktop = (className) => {
            const { value, fieldProperties } = this.props;
            const searchText = getReferenceSearchText({
                value,
                fieldProperties,
            });
            const selectedRecord = getReferenceSelectedRecord({
                value,
                fieldProperties,
            });
            const helperText = getReferenceHelperText({ value, fieldProperties });
            const carbonProps = getCommonCarbonComponentProperties(this.props);
            return (React.createElement(React.Fragment, null,
                React.createElement(CarbonWrapper, { ...this.props, className: className, componentName: "reference", componentRef: this.componentRef, helperText: this.props.fieldProperties.helperText, noReadOnlySupport: true, readOnlyDisplayValue: searchText },
                    this.props.fieldProperties.imageField &&
                        selectedRecord &&
                        (this.isReadOnly() || this.isDisabled()) && (React.createElement(Portrait, { image: selectedRecord.image, placeholderValue: String(selectedRecord.value), size: this.getPortraitSize() })),
                    React.createElement("div", { onClick: this.getClickHandler(), className: "e-reference-field-body" },
                        React.createElement(Select, { autoSelect: this.props.fieldProperties.isAutoSelectEnabled, disabled: this.isDisabled(), error: this.props.validationErrors?.[0]?.message, warning: carbonProps.warning, info: carbonProps.info, fullWidth: this.props.fieldProperties.isFullWidth, getItems: this.getItems, hasLookupIcon: this.hasLookupIcon(), hasHighlightMatchText: true, helperText: helperText, helperTextLink: this.getHelperTextLink(), icon: this.props.fieldProperties.icon, initialInputValue: searchText, inputId: carbonProps.id, onInputChange: this.onInputChange, onHelperTextLinkClick: this.onOpenTunnel, isSortedAlphabetically: true, isDropdownDisabled: this.props.fieldProperties.isDropdownDisabled, isHelperTextHidden: this.props.fieldProperties.isHelperTextHidden, isLinkHelperText: !!this.props.fieldProperties.tunnelPage, hasClearFieldButton: true, label: !this.props.fieldProperties.isTitleHidden
                                ? getLabelTitle(this.props.screenId, this.props.fieldProperties, this.props.handlersArguments?.rowValue)
                                : undefined, lookupIconId: `e-reference-field-lookup-icon-${this.state.id}`, closeIconClassName: "e-ui-select-close-icon", lookupButtonRef: this.lookupButtonRef, minLookupCharacters: this.getMinLookupCharacters(), onChange: this.onChange, onInputFocus: this.props.onFocus, onLookupIconClick: this.onButtonClick, placeholder: this.props.fieldProperties.placeholder, readOnly: !this.isDisabled() && this.isReadOnly(), selectedItem: selectedRecord, size: this.props.fieldProperties.size, testId: `e-reference-field-lookup-input-${this.state.id}`, screenId: this.props.screenId, elementId: this.props.elementId, shouldFilterItems: false, isSoundDisabled: resolveByValue({
                                screenId: this.props.screenId,
                                propertyValue: this.props.fieldProperties.isSoundDisabled,
                                skipHexFormat: true,
                                fieldValue: null,
                                rowValue: null,
                            }), createTunnelLinkText: this.props.fieldProperties.createTunnelLinkText, onCreateNewItemLinkClick: this.handleOnCreateNewItemLinkClick, isLinkCreateNewText: !!this.props.fieldProperties.tunnelPage, node: this.props.fieldProperties.node }))),
                React.createElement(LookupDialog, { isOpen: this.isLookupDialogOpen(), fieldProperties: this.props.fieldProperties, screenId: this.props.screenId, onSelectionFinished: this.onLookupDialogSelectionFinished, fieldId: this.props.elementId, closeLookupDialog: this.closeLookupDialog, parentElementId: this.props.parentElementId, selectedRecordId: selectedRecord?.id, recordContext: this.props.recordContext, contextNode: this.props.contextNode, createTunnelLinkText: this.props.fieldProperties.createTunnelLinkText, onCreateNewItemLinkClick: this.handleOnCreateNewItemLinkClick, isLinkCreateNewText: !!this.props.fieldProperties.tunnelPage })));
        };
        this.isRelatedTargetEqualTo = (e, input) => e.relatedTarget?.id === `${input}-${this.state.id}`;
        const { value, fieldProperties } = props;
        const searchText = getReferenceSearchText({ value, fieldProperties });
        this.state = {
            id: uid(16),
        };
        this.searchText = searchText;
        this.selectInputRef = React.createRef();
    }
    componentDidMount() {
        // On mobile if there are no columns by default we add "valueField" & "helperTextField"
        if (!this.props.fieldProperties.columns &&
            !this.isReadOnly() &&
            !(this.props.browser?.greaterThan.s ?? true) &&
            this.props.setFieldProperties) {
            this.props.setFieldProperties(this.props.elementId, {
                ...this.props.fieldProperties,
                columns: [this.props.fieldProperties.valueField, this.props.fieldProperties.helperTextField]
                    .filter(c => !!c)
                    .map(c => {
                    // Hacky as query generation works with object as well as strings
                    const bind = typeof c === 'object' ? c : String(c);
                    return text({ bind: bind });
                }),
            });
        }
    }
    async handleAutoSelection(isOrganicChange, isDifferentSearchText) {
        if (!this.selectInputRef.current) {
            return;
        }
        const soundApi = new SoundApi(resolveByValue({
            screenId: this.props.screenId,
            propertyValue: this.props.fieldProperties.isSoundDisabled,
            skipHexFormat: true,
            fieldValue: null,
            rowValue: null,
        }));
        const result = await this.getItems(this.selectInputRef.current.value);
        if (result.length === 1 && result[0].__collectionItem) {
            this.onChange(result[0].__collectionItem, isOrganicChange);
            if (isOrganicChange) {
                await soundApi.success();
            }
            return;
        }
        if (result.length === 0 && isOrganicChange) {
            await soundApi.error();
        }
        if (isOrganicChange) {
            this.searchText = this.selectInputRef.current.value;
            this.openLookupDialog();
        }
        else if (isDifferentSearchText) {
            this.onChange(undefined, isOrganicChange);
        }
    }
    render() {
        let className = 'e-reference-field';
        if (this.hasLookupIcon()) {
            className += ' e-reference-field-lookup';
        }
        if (this.isReadOnlyAndHasPicture()) {
            className += ' e-reference-inline-picture';
        }
        if (!this.isReadOnly() && !(this.props.browser?.greaterThan.s ?? true)) {
            return this.renderMobile(className);
        }
        return this.renderDesktop(className);
    }
}
const extendedMapDispatchToProps = (dispatch, props) => {
    const defaultMapDispatchToProps = mapDispatchToProps()(dispatch, props);
    return {
        ...defaultMapDispatchToProps,
        openTunnel: (fieldProperties, value) => dispatch(xtremRedux.actions.openTunnel({
            elementId: props.elementId,
            parentElementId: props.parentElementId,
            screenId: props.screenId,
            recordContext: props.recordContext,
            contextNode: props.contextNode,
            fieldProperties,
            value,
        })),
    };
};
export const ConnectedReferenceComponent = connect(mapStateToProps, extendedMapDispatchToProps)(ReferenceComponent);
export default ConnectedReferenceComponent;
//# sourceMappingURL=reference-component.js.map