import React from 'react';
import { splitValueToMergedValue } from '../../../utils/transformers';
import { Select } from '../../ui/select/select-component';
import { shouldRenderDropdownAbove } from '../../../utils/table-component-utils';
import { createSelectItemFromOption, getItemsFromProps } from './select-utils';
import { manageOnBlurFocus } from '../../ui/table-shared/cell/cell-editor-utils';
import { AgGridCellEditorComponent } from '../table/table-component-types';
import { setDefaultAgGridInputStyles } from '../../../utils/ag-grid/ag-grid-cell-editor-utils';
import { TAB, SHIFT_TAB, KEY_BACKSPACE, KEY_DELETE, KEY_F2 } from '../../../utils/keyboard-event-utils';
export class SelectCellEditor extends AgGridCellEditorComponent {
    constructor(props) {
        super(props);
        this.isSelecting = false;
        this.onKeyDown = (event) => {
            if (event.key === TAB) {
                if (event.shiftKey) {
                    this.setState({ endAction: SHIFT_TAB });
                }
                else {
                    this.setState({ endAction: TAB });
                }
            }
        };
        this.getItems = (searchText) => {
            const items = getItemsFromProps(this.props, searchText, splitValueToMergedValue(this.props.data));
            return Promise.resolve(items);
        };
        this.onSelected = (selectedItem) => {
            this.isSelecting = true;
            this.setState({ selectedRecord: selectedItem ?? undefined, highlightedItem: null }, () => {
                this.isSelecting = false;
                this.onBlur();
            });
        };
        this.onHighlighted = (highlightedItem) => {
            this.setState({ highlightedItem });
        };
        this.onBlur = () => manageOnBlurFocus(this.isSelecting, this.props.api, this.props.column, this.props.rowIndex, this.props.node.isRowPinned() ? 'top' : null, this.props.stopEditing, this.state.endAction);
        this.selectInputRef = React.createRef();
        this.state = this.createInitialState(props);
    }
    componentDidMount() {
        const input = this.selectInputRef.current;
        if (!input) {
            return;
        }
        // try to match the styles of ag-grid
        setDefaultAgGridInputStyles(input);
        const inputDiv = input.parentNode;
        inputDiv.style.width = '100%';
        inputDiv.style.height = '100%';
        inputDiv.style.border = 'none';
        input.focus();
        if (this.state.highlightAllOnFocus) {
            input.select();
            this.setState({
                highlightAllOnFocus: false,
            });
        }
        else {
            // when we started editing, we want the caret at the end, not the start.
            // this comes into play in two scenarios: a) when user hits F2 and b)
            // when user hits a printable character, then on IE (and only IE) the caret
            // was placed after the first character, thus 'apply' would end up as 'pplea'
            const length = input.value ? input.value.length : 0;
            if (length > 0) {
                input.setSelectionRange(length, length);
            }
        }
    }
    getValue() {
        return (this.state.highlightedItem?.id || this.state.selectedRecord?.id) ?? null;
    }
    getInitialSelectedItem(startValue = '') {
        return startValue === '' ? undefined : createSelectItemFromOption(startValue, this.props);
    }
    createInitialState(props) {
        let startValue;
        let highlightAllOnFocus = true;
        if (props.keyPress === KEY_BACKSPACE || props.keyPress === KEY_DELETE) {
            // if backspace or delete pressed, we clear the cell
            startValue = '';
        }
        else if (props.charPress) {
            // if a letter was pressed, we start with the letter
            startValue = props.charPress;
            highlightAllOnFocus = false;
        }
        else {
            // otherwise we start with the current value
            startValue =
                props.localizedOptions && props.localizedOptions[props.value]
                    ? props.localizedOptions[props.value]
                    : props.value;
            if (props.keyPress === KEY_F2) {
                highlightAllOnFocus = false;
            }
        }
        return {
            startValue: startValue || '',
            highlightAllOnFocus,
            highlightedItem: null,
            selectedRecord: this.getInitialSelectedItem(startValue || ''),
        };
    }
    render() {
        const dataTestId = `${this.props.tableElementId}-${this.props.rowIndex}-${this.props.api.getColumns().indexOf(this.props.column) + 1}`;
        const width = this.props.eGridCell?.style?.width || '200px';
        const initialInputValue = this.state.startValue;
        const initialSelectedItem = this.state.selectedRecord;
        return (React.createElement("div", { "data-testid": dataTestId, style: { width } },
            React.createElement(Select, { autoSelect: true, elementId: this.props.elementId, getItems: this.getItems, initialInputValue: initialInputValue, isSortedAlphabetically: this.props.fieldProperties.isSortedAlphabetically, isInTable: true, minLookupCharacters: 0, onBlur: this.onBlur, onKeyDown: this.onKeyDown, onSelected: this.onSelected, onHighlightedChange: this.onHighlighted, ref: this.selectInputRef, screenId: this.props.screenId, selectedItem: initialSelectedItem, shouldRenderOptionsAbove: shouldRenderDropdownAbove({
                    isPhantomRow: !!this.props.data?.__phantom,
                    pageSize: this.props.api.paginationGetPageSize(),
                    rowIndex: this.props.rowIndex,
                }), testId: `${dataTestId}-input`, variant: "plain" })));
    }
}
//# sourceMappingURL=select-cell-editor.js.map