import * as tokens from '@sage/design-tokens/js/base/common';
import axios from 'axios';
import Button from 'carbon-react/esm/components/button';
import Loader from 'carbon-react/esm/components/loader';
import { camelCase, get, includes, isEqual } from 'lodash';
import React from 'react';
import { connect } from 'react-redux';
import * as xtremRedux from '../../../redux';
import { RecordActionType } from '../../../service/collection-data-types';
import { localize } from '../../../service/i18n-service';
import { getScreenElement } from '../../../service/screen-base-definition';
import { SoundApi } from '../../../service/sound-api';
import { getPropertyScalarType } from '../../../utils/abstract-fields-utils';
import { takeLatest } from '../../../utils/async';
import { containsValueDeep } from '../../../utils/common-util';
import { xtremConsole } from '../../../utils/console';
import { triggerFieldEvent } from '../../../utils/events';
import { convertDeepBindToPath, convertDeepBindToPathNotNull, getImagePlaceholderValue, } from '../../../utils/nested-field-utils';
import { findDeepPropertyType } from '../../../utils/node-utils';
import { resolveByValue } from '../../../utils/resolve-value-utils';
import { schemaTypeNameFromNodeName, splitValueToMergedValue } from '../../../utils/transformers';
import { navigationPanelId } from '../../container/navigation-panel/navigation-panel-types';
import { FieldKey } from '../../types';
import { CardComponent } from '../../ui/card/card-component';
import { FiltersComponent, FiltersLabels } from '../../ui/filter/filters-component';
import { EmptyTableComponent } from '../../ui/no-rows-found/no-rows-found-component';
import { getFieldTitle, isFieldDisabled, isFieldReadOnly } from '../carbon-helpers';
import { FieldLabel } from '../carbon-utility-components';
import FieldActions from '../field-actions-component';
import { getReferenceValueField } from '../reference/reference-utils';
import { DesktopTableHeaderComponent } from '../../ui/table-shared/desktop-table-header';
import { MobileTableSkeletonCard } from './mobile-table-skeleton-card';
import { calculateHiddenColumns, getActiveOptionsMenu, getCardDefinitionFromColumns, } from '../../../utils/table-component-utils';
import { ConnectedTableOptionsMenu } from '../../ui/table-shared/table-options-menu';
import { TableSearchBox, getSearchTextboxId } from '../../ui/table-shared/table-search-box-component';
import { ContextType } from '../../../types';
import { focusPageContent } from '../../../utils/dom';
import { mapAgGridFilterToXtremFilters } from '../../../utils/ag-grid/ag-grid-table-utils';
const NAVIGATION_PANEL_ELEMENT_ID_PREFIX = 'navigation-panel-card-';
export class UnconnectedMobileTableComponent extends React.Component {
    constructor(props) {
        super(props);
        this.takeLatest = takeLatest();
        this.isNavigationPanel = () => this.props.elementId === navigationPanelId;
        this.toXtremFilters = (filter) => {
            return this.isNavigationPanel() ? Object.keys(filter).map(mapAgGridFilterToXtremFilters(filter)) : filter;
        };
        this.isDisabled = () => this.props.isParentDisabled || isFieldDisabled(this.props.screenId, this.props.fieldProperties, null, null);
        this.isReadOnly = () => isFieldReadOnly(this.props.screenId, this.props.fieldProperties, null, null);
        this.subscribeToCollectionValueEvents = (pageSize, collectionValue) => {
            if (collectionValue) {
                collectionValue.subscribeForValueChanges(async (actionType, rowValue) => {
                    // Check if the row modified is displayed on the screen.
                    const rowPosition = this.state.valuesDisplayed.findIndex(r => r._id === rowValue._id);
                    const isRowOnScreen = rowPosition !== -1;
                    if (actionType === RecordActionType.MODIFIED && isRowOnScreen) {
                        // If so, we update it's value in the component state.
                        this.setState(state => {
                            const valuesDisplayed = [...state.valuesDisplayed];
                            valuesDisplayed[rowPosition] = rowValue;
                            return { valuesDisplayed };
                        });
                    }
                    else if (actionType === RecordActionType.REMOVED && isRowOnScreen) {
                        // If the removed record was on the screen, we refetch all displayed records
                        const newValues = await collectionValue.getRecordWithCurrentQueryArguments({
                            tableFieldProperties: this.props.fieldProperties,
                            pageSize,
                            // eslint-disable-next-line react/no-access-state-in-setstate
                            pageNumber: this.state.pageNumber,
                            cursor: this.getCursor(),
                            cleanMetadata: false,
                        });
                        this.setState({ valuesDisplayed: newValues });
                    }
                    else if (actionType === RecordActionType.ADDED) {
                        // If a new record was added we need to refetch all records displayed so far.
                        /* eslint-disable react/no-access-state-in-setstate */
                        this.setState({ isLoading: true }, async () => {
                            const valuesDisplayed = await collectionValue.getPage({
                                tableFieldProperties: this.props.fieldProperties,
                                filters: this.toXtremFilters(this.props.filterModel),
                                orderBy: this.props.fieldProperties.orderBy,
                                pageSize: Math.max(this.state.valuesDisplayed.length, this.props.fieldProperties.pageSize),
                                cursor: undefined,
                                cleanMetadata: false,
                            });
                            this.setState({ isLoading: false, valuesDisplayed, pageNumber: 0 });
                        });
                        /* eslint-enable react/no-access-state-in-setstate */
                    }
                });
            }
        };
        this.shouldDisplayElement = (elementId, isNewRecord, hiddenFields) => {
            return (!!elementId &&
                !(isNewRecord && elementId === '_id') &&
                !includes(hiddenFields, convertDeepBindToPathNotNull(elementId)));
        };
        this.onRowSelected = async (row) => {
            if (this.props.setFieldProperties) {
                const elementId = this.props.elementId;
                const selectedRecords = [...(this.props.fieldProperties.selectedRecords || [])];
                const currentPosition = selectedRecords.indexOf(row._id);
                const selected = currentPosition >= 0;
                if (!selected) {
                    selectedRecords.push(row._id);
                    this.props.setFieldProperties(elementId, {
                        ...this.props.fieldProperties,
                        selectedRecords,
                    });
                    await triggerFieldEvent(this.props.screenId, this.props.elementId, 'onRowSelected', row._id, splitValueToMergedValue(row));
                }
                else {
                    selectedRecords.splice(currentPosition, 1);
                    this.props.setFieldProperties(elementId, {
                        ...this.props.fieldProperties,
                        selectedRecords,
                    });
                    await triggerFieldEvent(this.props.screenId, this.props.elementId, 'onRowUnselected', row._id, splitValueToMergedValue(row));
                }
            }
        };
        this.canCreateNewItem = (collection) => {
            return (!!this.props.fieldProperties.isNewEnabled &&
                !!this.props.fieldProperties.valueField &&
                !!this.state.searchText?.length &&
                collection.every(item => {
                    return item[`${this.props.fieldProperties.valueField}`] !== this.state.searchText;
                }));
        };
        this.getCreateNewItem = () => ({
            _id: '-1',
            _isNewRecord: true,
            [`${this.props.fieldProperties.valueField}`]: this.state.searchText,
        });
        this.onRowClick = (item) => (_item, isModifierKeyPushed = false) => {
            if (this.props.fieldProperties.isNewEnabled && item._isNewRecord) {
                if (typeof this.props.onRowClick === 'function') {
                    this.props.onRowClick(item._id, splitValueToMergedValue(item), isModifierKeyPushed)();
                }
            }
            else if (typeof this.props.onRowClick === 'function') {
                this.props.onRowClick(item._id, undefined, isModifierKeyPushed)();
            }
        };
        this.refreshTableContent = async () => {
            const pageSize = this.props.fieldProperties.pageSize || 20;
            this.setState({ isLoading: true });
            const { cancel, token } = axios.CancelToken.source();
            try {
                const params = {
                    tableFieldProperties: this.props.fieldProperties,
                    filters: this.toXtremFilters(this.props.filterModel),
                    orderBy: this.props.fieldProperties.orderBy,
                    pageSize,
                    searchText: this.state.searchText,
                    axiosCancelToken: token,
                    cursor: undefined,
                    cleanMetadata: false,
                    selectedOptionsMenuItem: this.props.selectedOptionsMenuItem,
                };
                const valuesDisplayed = await this.takeLatest(this.props.value.getPage(params));
                const newState = {
                    valuesDisplayed,
                    isLoading: false,
                    hasMorePages: valuesDisplayed.length === pageSize,
                    pageNumber: 0,
                };
                this.setState(newState);
                const soundApi = new SoundApi(resolveByValue({
                    screenId: this.props.screenId,
                    propertyValue: this.props.fieldProperties.isSoundDisabled,
                    skipHexFormat: true,
                    fieldValue: null,
                    rowValue: null,
                }));
                if (valuesDisplayed.length === 1 &&
                    (this.props.isAutoSelectEnabled || this.props.fieldProperties.isAutoSelectEnabled) &&
                    containsValueDeep(valuesDisplayed[0], this.state.searchText)) {
                    await soundApi.success();
                    this.onRowClick(valuesDisplayed[0])(valuesDisplayed[0], false);
                }
                if (!valuesDisplayed.length &&
                    (this.props.isAutoSelectEnabled || this.props.fieldProperties.isAutoSelectEnabled)) {
                    await soundApi.error();
                }
            }
            catch (err) {
                if (err.name === 'AbortError') {
                    // propagate outdated call cancellations to axios
                    cancel();
                }
                else {
                    xtremConsole.error(err);
                }
            }
        };
        this.onFiltersChanged = async (filters) => {
            if (this.isNavigationPanel() && this.props.setTableViewFilter) {
                this.props.setTableViewFilter(0, filters);
            }
        };
        this.getFilterableFields = () => {
            const node = this.props.fieldProperties.node;
            return this.props.fieldProperties.columns.map(nestedField => {
                const propertyType = findDeepPropertyType(schemaTypeNameFromNodeName(node), nestedField.properties.bind, this.props.nodeTypes, FieldKey.Text, true);
                const fieldType = nestedField.type;
                return {
                    type: fieldType,
                    properties: {
                        ...nestedField.properties,
                        bind: convertDeepBindToPathNotNull(nestedField.properties.bind),
                        title: getFieldTitle(this.props.screenId, nestedField.properties, null),
                        type: getPropertyScalarType(this.props.nodeTypes, propertyType, nestedField.type, nestedField.properties.valueField),
                    },
                };
            });
        };
        this.fetchNextPage = () => {
            const pageSize = this.props.fieldProperties.pageSize || 20;
            /* eslint-disable react/no-access-state-in-setstate */
            this.setState({
                isFetchingNextPage: true,
                pageNumber: this.state.pageNumber + 1,
            }, async () => {
                const newValues = await this.props.value.getPageWithCurrentQueryArguments({
                    tableFieldProperties: this.props.fieldProperties,
                    pageSize,
                    pageNumber: this.state.pageNumber,
                    cursor: this.getCursor(),
                    cleanMetadata: false,
                });
                this.setState({
                    valuesDisplayed: [...this.state.valuesDisplayed, ...newValues],
                    isFetchingNextPage: false,
                    hasMorePages: newValues.length === pageSize,
                });
            });
            /* eslint-enable react/no-access-state-in-setstate */
        };
        this.getNestedReferenceValue = (definition, data) => {
            const valueField = getReferenceValueField(definition);
            return valueField ? get(data, valueField) : undefined;
        };
        this.getRows = () => {
            return this.canCreateNewItem(this.state.valuesDisplayed)
                ? [this.getCreateNewItem(), ...this.state.valuesDisplayed]
                : this.state.valuesDisplayed;
        };
        this.getImagePlaceholderValue = (item, nFields) => {
            const titleElementId = convertDeepBindToPathNotNull(nFields.title?.properties?.bind);
            const value = nFields.title?.type === FieldKey.Reference
                ? this.getNestedReferenceValue(nFields.title.properties, get(item, titleElementId))
                : get(item, titleElementId);
            return item._isNewRecord ? `${value} (${localize('@sage/xtrem-ui/new', 'New')})` : value;
        };
        this.getSeparatorValue = (record) => {
            if (!this.props.groupByField || !record) {
                return null;
            }
            const nestedField = this.props.fieldProperties.columns.find(c => convertDeepBindToPathNotNull(c.properties.bind) ===
                convertDeepBindToPathNotNull(this.props.groupByField));
            if (!nestedField) {
                return null;
            }
            const result = getImagePlaceholderValue(record, nestedField);
            return result || null;
        };
        this.getCardDefinition = (nestedFields, item, hiddenFields) => {
            const titleRightElementId = nestedFields.titleRight?.properties?.bind;
            const shouldDisplayTitleRight = this.shouldDisplayElement(titleRightElementId, item._isNewRecord, hiddenFields);
            const line2ElementId = nestedFields.line2?.properties?.bind;
            const shouldDisplayLine2 = this.shouldDisplayElement(line2ElementId, item._isNewRecord, hiddenFields);
            const line2RightElementId = nestedFields.line2Right?.properties?.bind;
            const shouldDisplayLine2Right = this.shouldDisplayElement(line2RightElementId, item._isNewRecord, hiddenFields);
            const line3ElementId = nestedFields.line3?.properties?.bind;
            const shouldDisplayLine3 = this.shouldDisplayElement(line3ElementId, item._isNewRecord, hiddenFields);
            const line3RightElementId = nestedFields.line3Right?.properties?.bind;
            const shouldDisplayLine3Right = this.shouldDisplayElement(line3RightElementId, item._isNewRecord, hiddenFields);
            const cardDefinition = {
                ...nestedFields,
                title: nestedFields.title,
                titleRight: shouldDisplayTitleRight ? nestedFields.titleRight : undefined,
                line2: shouldDisplayLine2 ? nestedFields.line2 : undefined,
                line2Right: shouldDisplayLine2Right ? nestedFields.line2Right : undefined,
                line3: shouldDisplayLine3 ? nestedFields.line3 : undefined,
                line3Right: shouldDisplayLine3Right ? nestedFields.line3Right : undefined,
            };
            if (nestedFields.image &&
                nestedFields.image.type === FieldKey.Image &&
                !nestedFields.image.properties.placeholderValue) {
                cardDefinition.image = {
                    ...nestedFields.image,
                    properties: {
                        ...nestedFields.image.properties,
                        placeholderValue: this.getImagePlaceholderValue(item, nestedFields),
                    },
                };
            }
            if (cardDefinition.image && cardDefinition.image.type === FieldKey.Icon) {
                cardDefinition.image.properties = {
                    ...cardDefinition.image.properties,
                    color: tokens.colorsUtilityMajor300,
                    backgroundColor: tokens.colorsActionMinor050,
                    backgroundShape: 'circle',
                    backgroundSize: 'large',
                };
            }
            if (item._isNewRecord) {
                const nestedValueFieldKey = Object.keys(nestedFields).find(key => nestedFields[key]?.properties.bind === this.props.fieldProperties.valueField);
                if (nestedValueFieldKey) {
                    cardDefinition.title = nestedFields[nestedValueFieldKey];
                }
            }
            return cardDefinition;
        };
        this.getOptionsMenuType = () => {
            if (!this.props.fieldProperties.optionsMenu?.length) {
                return undefined;
            }
            return this.props.fieldProperties.optionsMenuType;
        };
        this.onScroll = (event) => {
            const listBody = event.target;
            // fetch next page when there are only 5 cards left (78px tall)
            const shouldLoadNextPage = listBody.scrollHeight - listBody.scrollTop <= listBody.clientHeight + 390;
            if (this.state.hasMorePages && !this.state.isFetchingNextPage && shouldLoadNextPage) {
                this.fetchNextPage();
            }
        };
        this.onKeyDown = (event) => {
            if (document.activeElement?.id.startsWith(NAVIGATION_PANEL_ELEMENT_ID_PREFIX) && this.isNavigationPanel()) {
                const currentItemIndex = Number(document.activeElement.id.replace(NAVIGATION_PANEL_ELEMENT_ID_PREFIX, ''));
                if (event.key === 'ArrowDown') {
                    const nextElement = event.currentTarget.querySelector(`#${NAVIGATION_PANEL_ELEMENT_ID_PREFIX}${currentItemIndex + 1}`);
                    nextElement?.focus();
                }
                if (event.key === 'ArrowUp') {
                    const previousElement = event.currentTarget.querySelector(`#${NAVIGATION_PANEL_ELEMENT_ID_PREFIX}${currentItemIndex - 1}`);
                    previousElement?.focus();
                }
                if (event.key === 'Tab' && !event.shiftKey) {
                    event.preventDefault();
                    focusPageContent();
                }
                if (event.key === 'Tab' && event.shiftKey) {
                    const searchTextboxId = getSearchTextboxId(this.props.screenId, this.props.elementId);
                    const inputElement = searchTextboxId && document.querySelector(`#${searchTextboxId}`);
                    if (inputElement) {
                        event.preventDefault();
                        inputElement.focus();
                    }
                }
            }
        };
        this.onOptionsMenuItemChanged = async (selectedOptionsMenuItem) => {
            this.props.setTableViewOptionsMenuItem?.(0, selectedOptionsMenuItem);
        };
        this.onSearchBoxValueChange = (searchText) => {
            this.setState({ searchText, isLoading: true }, async () => {
                if (this.props.setTableViewSearchText) {
                    this.props.setTableViewSearchText(0, searchText);
                }
                await this.refreshTableContent();
            });
        };
        this.onResetFilter = () => this.onSearchBoxValueChange('');
        const pageSize = this.props.fieldProperties.pageSize || 20;
        this.subscribeToCollectionValueEvents(pageSize, this.props.value);
        this.state = {
            hasMorePages: false,
            isFetchingNextPage: false,
            pageNumber: 0,
            searchText: this.props.searchText || '',
            valuesDisplayed: [],
            isLoading: true,
        };
    }
    async componentDidMount() {
        const pageSize = this.props.fieldProperties.pageSize || 20;
        const valuesDisplayed = this.props.value
            ? await this.props.value.getPage({
                cleanMetadata: false,
                tableFieldProperties: this.props.fieldProperties,
                filters: this.toXtremFilters(this.props.filterModel),
                pageNumber: 0,
                searchText: this.props.searchText || '',
                selectedOptionsMenuItem: this.props.selectedOptionsMenuItem,
            })
            : [];
        this.setState({
            hasMorePages: valuesDisplayed.length >= pageSize,
            valuesDisplayed: valuesDisplayed.slice(0, pageSize),
            isLoading: false,
        });
    }
    async componentDidUpdate(prevProps) {
        if (prevProps.value !== this.props.value ||
            !isEqual(prevProps.fieldProperties.filter, this.props.fieldProperties.filter) ||
            !isEqual(prevProps.filterModel, this.props.filterModel) ||
            !isEqual(prevProps.selectedOptionsMenuItem, this.props.selectedOptionsMenuItem)) {
            const pageSize = this.props.fieldProperties.pageSize || 20;
            const valuesDisplayed = this.props.value
                ? await this.props.value.getPage({
                    cleanMetadata: false,
                    filters: this.toXtremFilters(this.props.filterModel),
                    tableFieldProperties: this.props.fieldProperties,
                    pageNumber: 0,
                    searchText: this.props.searchText || '',
                    selectedOptionsMenuItem: this.props.selectedOptionsMenuItem,
                })
                : [];
            this.subscribeToCollectionValueEvents(pageSize, this.props.value);
            this.setState({
                hasMorePages: valuesDisplayed.length >= pageSize,
                pageNumber: 0,
                valuesDisplayed: valuesDisplayed.slice(0, pageSize),
                searchText: this.props.searchText || '',
            });
        }
    }
    getCursor() {
        return [...this.state.valuesDisplayed].reverse()[0]?.__cursor;
    }
    renderRows() {
        const nestedFields = getCardDefinitionFromColumns({
            columns: this.props.fieldProperties.columns,
            mobileCard: this.props.fieldProperties.mobileCard,
            screenId: this.props.screenId,
            isGreaterThanSmall: this.props.browser.greaterThan.s,
            hiddenColumns: this.props.fieldProperties.hiddenColumns,
            sortColumns: this.props.fieldProperties.sortColumns,
        });
        const currentFields = [
            convertDeepBindToPath(nestedFields.title?.properties?.bind),
            convertDeepBindToPath(nestedFields.titleRight?.properties?.bind),
            convertDeepBindToPath(nestedFields.line2?.properties?.bind),
            convertDeepBindToPath(nestedFields.line2Right?.properties?.bind),
            convertDeepBindToPath(nestedFields.line3?.properties?.bind),
            convertDeepBindToPath(nestedFields.line3Right?.properties?.bind),
        ];
        const hiddenFields = calculateHiddenColumns(this.props.fieldProperties.hiddenColumns ?? [], currentFields);
        const rows = this.getRows();
        return (React.createElement("div", { className: "e-table-field-mobile-rows", "data-testid": "e-table-field-mobile-rows", onScroll: this.props.isUsingInfiniteScroll ? this.onScroll : undefined, onKeyDown: this.onKeyDown },
            !this.state.isLoading &&
                rows.map((item, index) => {
                    const isRowSelected = this.props.fieldProperties.selectedRecords &&
                        this.props.fieldProperties.selectedRecords.indexOf(item._id) !== -1;
                    const cardDefinition = this.getCardDefinition(nestedFields, item, hiddenFields);
                    const previousSeparatorValue = this.getSeparatorValue(rows[index - 1]);
                    const currentSeparatorValue = this.getSeparatorValue(item);
                    const hasReferenceFieldLookup = this.props.fieldProperties._controlObjectType === FieldKey.FilterSelect ||
                        this.props.fieldProperties._controlObjectType === FieldKey.Reference;
                    const cardComponentProps = {
                        canDrag: !!this.props.canDragCard,
                        canSelect: this.props.fieldProperties.canSelect,
                        cardDefinition,
                        contextType: ContextType.table,
                        isClickIndicatorHidden: hasReferenceFieldLookup,
                        isDisabled: this.isDisabled(),
                        isNewItem: item._isNewRecord,
                        isSelected: isRowSelected,
                        id: this.isNavigationPanel() ? `${NAVIGATION_PANEL_ELEMENT_ID_PREFIX}${index}` : undefined,
                        onCardClick: this.props.onRowClick ? this.onRowClick(item) : undefined,
                        onSelect: this.onRowSelected,
                        parentElementId: this.props.elementId,
                        dropdownActions: this.props.fieldProperties.dropdownActions,
                        inlineActions: this.props.fieldProperties.inlineActions,
                        screenId: this.props.screenId,
                        value: item,
                    };
                    return (React.createElement(React.Fragment, null,
                        this.props.groupByField &&
                            currentSeparatorValue &&
                            previousSeparatorValue !== currentSeparatorValue && (React.createElement("div", { className: "e-mobile-table-separator", "data-testid": `e-mobile-table-separator-${camelCase(currentSeparatorValue)}` }, currentSeparatorValue)),
                        React.createElement(CardComponent, { key: `${this.props.elementId}_${item._id}`, ...cardComponentProps })));
                }),
            this.state.isLoading && (React.createElement(React.Fragment, null,
                React.createElement(MobileTableSkeletonCard, { key: "1", cardDefinition: nestedFields }),
                React.createElement(MobileTableSkeletonCard, { key: "2", cardDefinition: nestedFields }),
                React.createElement(MobileTableSkeletonCard, { key: "3", cardDefinition: nestedFields }),
                React.createElement(MobileTableSkeletonCard, { key: "4", cardDefinition: nestedFields }))),
            !this.state.isLoading && !this.state.valuesDisplayed.length && (React.createElement("div", { className: "e-mobile-table-body-empty" },
                React.createElement(EmptyTableComponent, { elementId: this.props.elementId, screenId: this.props.screenId, emptyStateText: this.props.fieldProperties?.emptyStateText, emptyStateClickableText: this.props.fieldProperties?.emptyStateClickableText, onEmptyStateLinkClick: this.props.fieldProperties?.onEmptyStateLinkClick, isFiltering: !!this.state.searchText, isCardView: true, onRemoveFiltersClick: this.onResetFilter }))),
            this.renderFooter()));
    }
    renderFooter() {
        if (this.props.isUsingInfiniteScroll && this.state.isFetchingNextPage) {
            return React.createElement(Loader, { size: "large" });
        }
        if (!this.props.isUsingInfiniteScroll && (this.state.isFetchingNextPage || this.state.hasMorePages)) {
            return (React.createElement("div", { className: "e-load-more-button-container", "data-testid": "e-load-more-button-container" },
                React.createElement(Button, { buttonType: "tertiary", "data-testid": this.state.isFetchingNextPage ? 'e-is-loading-more-button' : 'e-load-more-button', disabled: this.isDisabled() || !this.state.hasMorePages || this.state.isFetchingNextPage, onClick: this.fetchNextPage }, this.state.isFetchingNextPage ? (React.createElement(Loader, { size: "large" })) : (localize('@sage/xtrem-ui/mobile-table-load-more', 'Load more')))));
        }
        return null;
    }
    renderMobileHeaderActions() {
        const filterableField = this.getFilterableFields();
        return (React.createElement("div", { className: "e-field-actions-wrapper" },
            React.createElement("div", { className: "e-component-actions" },
                React.createElement(FieldActions, { screenId: this.props.screenId, fieldId: this.props.elementId, isDisabled: this.isDisabled() }),
                this.props.fieldProperties.canFilter && (React.createElement(FiltersComponent, { screenId: this.props.screenId, fields: filterableField, handleSave: this.onFiltersChanged, filters: this.toXtremFilters(this.props.filterModel), isDisabled: this.isDisabled() })))));
    }
    renderDesktopHeader() {
        return (React.createElement(DesktopTableHeaderComponent, { elementId: this.props.elementId, fieldProperties: this.props.fieldProperties, filterComponentFields: this.getFilterableFields(), filtersComponentFilters: this.toXtremFilters(this.props.filterModel), hasData: this.getRows().length > 0, isDisabled: this.isDisabled(), isReadOnly: this.isReadOnly(), onFiltersComponentChanged: this.onFiltersChanged, onOptionsMenuItemChange: this.onOptionsMenuItemChanged, onSearchBoxValueChange: this.onSearchBoxValueChange, screenId: this.props.screenId, selectedOptionsMenuItem: this.props.selectedOptionsMenuItem, validationErrors: this.props.validationErrors }));
    }
    renderOptionsMenu() {
        return (React.createElement(ConnectedTableOptionsMenu, { elementId: this.props.elementId, onSelectionChange: this.onOptionsMenuItemChanged, optionsMenuItems: resolveByValue({
                propertyValue: this.props.fieldProperties.optionsMenu,
                rowValue: null,
                screenId: this.props.screenId,
                fieldValue: null,
                skipHexFormat: true,
            }), optionsMenuType: this.props.fieldProperties.optionsMenuType, screenId: this.props.screenId, selectedOptionsMenuItem: this.props.selectedOptionsMenuItem }));
    }
    renderMobileSearchbox() {
        return this.props.fieldProperties.hasSearchBoxMobile ? (React.createElement(TableSearchBox, { elementId: this.props.elementId, screenId: this.props.screenId, isDisabled: this.isDisabled(), fieldProperties: this.props.fieldProperties, onSearchBoxValueChange: this.onSearchBoxValueChange, filterText: this.state.searchText })) : (React.createElement("div", { className: "e-table-field-mobile-search" }));
    }
    renderMobileHeader() {
        const shouldRenderFieldTitle = !!this.props.fieldProperties.title && !this.props.fieldProperties.isTitleHidden;
        const optionsMenu = resolveByValue({
            propertyValue: this.props.fieldProperties.optionsMenu,
            rowValue: null,
            screenId: this.props.screenId,
            fieldValue: null,
            skipHexFormat: true,
        });
        const shouldRenderFieldHeader = this.props.fieldProperties.hasSearchBoxMobile ||
            optionsMenu?.length > 0 ||
            this.props.fieldProperties.canFilter ||
            this.props.hasFieldActions;
        const shouldRenderDropdownOrToggleOptionsMenu = optionsMenu?.length > 0 && this.props.fieldProperties.optionsMenuType !== 'tabs';
        const shouldRenderTabsOptionsMenu = optionsMenu?.length > 0 && this.props.fieldProperties.optionsMenuType === 'tabs';
        const shouldRenderFilterLabels = !this.isNavigationPanel() &&
            this.props.fieldProperties.canFilter &&
            this.toXtremFilters(this.props.filterModel).length > 0;
        const shouldRenderNavigationPanelFilterLabels = this.isNavigationPanel() && this.toXtremFilters(this.props.filterModel).length > 0;
        // If none of the header functionality is used, the empty wrapper div should not be rendered
        if (!shouldRenderFieldTitle &&
            !shouldRenderFieldHeader &&
            !shouldRenderFilterLabels &&
            !shouldRenderNavigationPanelFilterLabels) {
            return null;
        }
        return (React.createElement("div", { className: "e-mobile-table-header" },
            shouldRenderFieldTitle && (React.createElement("div", { className: "e-field-title" },
                React.createElement(FieldLabel, { label: getFieldTitle(this.props.screenId, this.props.fieldProperties, null) }))),
            shouldRenderFieldHeader && (React.createElement("div", { className: "e-field-header", "data-testid": "e-field-header", "data-options-menu-type": this.getOptionsMenuType() },
                shouldRenderDropdownOrToggleOptionsMenu && this.renderOptionsMenu(),
                React.createElement("div", { className: "e-table-mobile-searchbox-with-filter-line" },
                    this.renderMobileSearchbox(),
                    this.renderMobileHeaderActions()),
                shouldRenderTabsOptionsMenu && this.renderOptionsMenu())),
            (shouldRenderFilterLabels || shouldRenderNavigationPanelFilterLabels) && (React.createElement(FiltersLabels, { fields: this.getFilterableFields(), screenId: this.props.screenId, handleSave: this.onFiltersChanged, filters: this.props.filterModel, isDisabled: this.isDisabled(), isNavigationPanel: this.isNavigationPanel() }))));
    }
    render() {
        // The mobile header gets rendered in mobile or in the split view navigation panel
        const shouldRenderMobileHeader = this.props.browser.is.xs ||
            this.props.elementId === navigationPanelId ||
            // The lookup dialog renders in mobile mode on XS and S devices, so in case the table is used on such screen sizes, we still need the mobile mode
            (this.props.browser.is.s &&
                (this.props.fieldProperties._controlObjectType === FieldKey.FilterSelect ||
                    this.props.fieldProperties._controlObjectType === FieldKey.Reference));
        return (React.createElement("div", { className: "e-table-field-mobile" },
            shouldRenderMobileHeader ? this.renderMobileHeader() : this.renderDesktopHeader(),
            this.renderRows()));
    }
}
/** @internal */
const mapStateToProps = (state, props) => {
    const screenDefinition = state.screenDefinitions[props.screenId];
    const tableViews = screenDefinition.tableViews?.[props.elementId];
    const currentViewLevel = tableViews?.$current?.level?.[0];
    const filter = currentViewLevel?.filter || {};
    const searchText = props.searchText || currentViewLevel?.searchText || '';
    const selectedOptionsMenuItem = getActiveOptionsMenu(props.screenId, tableViews, props.fieldProperties.optionsMenu);
    const screenElement = getScreenElement(screenDefinition);
    const fieldActions = typeof props.fieldProperties?.fieldActions === 'function'
        ? props.fieldProperties?.fieldActions.apply(screenElement)
        : [];
    return {
        ...props,
        browser: state.browser,
        nodeTypes: state.nodeTypes,
        hasFieldActions: fieldActions.length > 0,
        filterModel: filter,
        selectedOptionsMenuItem,
        searchText,
    };
};
const mapDispatchToProps = (dispatch, props) => ({
    setTableViewFilter: (level, filter) => {
        dispatch(xtremRedux.actions.setTableViewFilter(props.screenId, props.elementId, level, filter));
    },
    setTableViewOptionsMenuItem: (level, optionsMenuItem) => {
        dispatch(xtremRedux.actions.setTableViewOptionMenuItem(props.screenId, props.elementId, level, optionsMenuItem));
    },
    setTableViewSearchText: (level, searchText) => {
        dispatch(xtremRedux.actions.setTableViewSearchText(props.screenId, props.elementId, level, searchText));
    },
});
/** @internal */
export const MobileTableComponent = connect(mapStateToProps, mapDispatchToProps)(UnconnectedMobileTableComponent);
//# sourceMappingURL=mobile-table-component.js.map