import React, { useEffect, useRef, useState } from 'react';
import CarbonDate from 'carbon-react/esm/components/date';
import { EQUALS, GREATER_THAN, GREATER_THAN_EQUAL, LESS_THAN, LESS_THAN_EQUAL, NOT_EQUALS, RANGE, } from '@sage/xtrem-shared';
import { Select } from '../../../select/select-component';
import localeText from '../../../../../utils/ag-grid/ag-grid-strings';
import { isValidIsoDate } from '@sage/xtrem-date-time';
import { isEqual, isNil } from 'lodash';
import { usePrevious } from '@sage/xtrem-ui-components';
// Component that shows a selector with the options to filter by
function FilterBySelector(props) {
    const tableLocaleText = localeText();
    const filterOptions = [
        { id: EQUALS, value: EQUALS, displayedAs: tableLocaleText.equals },
        { id: NOT_EQUALS, value: NOT_EQUALS, displayedAs: tableLocaleText.notEqual },
        { id: LESS_THAN, value: LESS_THAN, displayedAs: tableLocaleText.lessThan },
        { id: GREATER_THAN, value: GREATER_THAN, displayedAs: tableLocaleText.greaterThan },
        { id: LESS_THAN_EQUAL, value: LESS_THAN_EQUAL, displayedAs: tableLocaleText.lessThanOrEqual },
        { id: GREATER_THAN_EQUAL, value: GREATER_THAN_EQUAL, displayedAs: tableLocaleText.greaterThanOrEqual },
        { id: RANGE, value: RANGE, displayedAs: tableLocaleText.inRange },
    ];
    return (React.createElement(Select, { disableOpenOnFocus: true, hasInputSearch: false, getItems: () => Promise.resolve(filterOptions), onChange: props.onChange, selectedItem: filterOptions.find(option => option.id === props.value) }));
}
const defaultDateState = () => ({
    rawValue: '',
    formattedValue: '',
    rawValueTo: '',
    formattedValueTo: '',
    filterBy: EQUALS,
});
const stateToModel = (dateState) => ({
    filterType: 'date',
    dateFrom: dateState.rawValue,
    dateTo: dateState.rawValueTo,
    type: dateState.filterBy,
    formattedValue: dateState.formattedValue,
    formattedValueTo: dateState.formattedValueTo,
});
const DateFilter = React.forwardRef((props, ref) => {
    const dateRef = useRef();
    const [dateState, setDateState] = useState(defaultDateState());
    const [andDateState, setAndDateState] = useState(defaultDateState());
    const prevDateState = usePrevious(dateState);
    const handleChange = (event) => {
        const newValue = event.target.value.rawValue;
        const newFormattedValue = event.target.value.formattedValue;
        setDateState({ ...dateState, rawValue: newValue, formattedValue: newFormattedValue });
    };
    const handleChangeTo = (event) => {
        const newValue = event.target.value.rawValue;
        const newFormattedValue = event.target.value.formattedValue;
        setDateState({ ...dateState, rawValueTo: newValue, formattedValueTo: newFormattedValue });
    };
    const handleChangeAnd = (event) => {
        const newValue = event.target.value.rawValue;
        const newFormattedValue = event.target.value.formattedValue;
        setAndDateState({ ...dateState, rawValue: newValue, formattedValue: newFormattedValue });
    };
    const handleChangeAndTo = (event) => {
        const newValue = event.target.value.rawValue;
        const newFormattedValue = event.target.value.formattedValue;
        setAndDateState({ ...dateState, rawValueTo: newValue, formattedValueTo: newFormattedValue });
    };
    const filterBySelecterOnChange = (item) => {
        const newValue = item.value;
        setDateState({ ...dateState, filterBy: newValue });
    };
    const filterBySelecterAndOnChange = (item) => {
        const newValue = item.value;
        setAndDateState({ ...dateState, filterBy: newValue });
    };
    useEffect(() => {
        if (prevDateState && !isEqual(prevDateState, dateState)) {
            props.filterChangedCallback();
        }
    }, [props, dateState, andDateState, prevDateState]);
    React.useImperativeHandle(ref, () => ({
        doesFilterPass: () => {
            return true;
        },
        isFilterActive: () => {
            if (dateState.filterBy === RANGE) {
                return isValidIsoDate(dateState.rawValue) && isValidIsoDate(dateState.rawValueTo);
            }
            return isValidIsoDate(dateState.rawValue);
        },
        getModel: () => {
            if (dateState.formattedValue !== '' && andDateState.formattedValue !== '') {
                return {
                    condition1: stateToModel(dateState),
                    condition2: stateToModel(andDateState),
                    filterType: 'date',
                    operator: 'AND',
                };
            }
            return dateState.formattedValue !== '' ? stateToModel(dateState) : null;
        },
        setModel: (model) => {
            if (!model) {
                const filterBy = dateState.filterBy;
                setAndDateState(defaultDateState());
                setDateState({ ...defaultDateState(), filterBy });
                props.filterChangedCallback();
                return;
            }
            if (!isNil(model.dateFrom) && model.dateFrom !== dateState.rawValue) {
                setDateState({
                    ...dateState,
                    rawValue: model.dateFrom,
                });
            }
            if (!isNil(model.dateTo) && model.dateTo && model.dateTo !== dateState.rawValueTo) {
                setDateState({
                    ...dateState,
                    rawValueTo: model.dateTo,
                });
            }
            if (!isNil(model.type) && model.type && model.type !== dateState.filterBy) {
                setDateState({
                    ...dateState,
                    filterBy: model.type,
                });
            }
        },
        afterGuiAttached: () => {
            dateRef.current?.focus();
        },
        onFloatingFilterChanged: (_changeType, newValue) => {
            setDateState({
                ...dateState,
                formattedValue: newValue.formattedValue,
                rawValue: newValue.rawValue || '',
            });
        },
    }), 
    // eslint-disable-next-line react-hooks/exhaustive-deps
    [dateState, andDateState]);
    return (React.createElement("div", { className: "e-filter-date-container" },
        React.createElement(FilterBySelector, { onChange: filterBySelecterOnChange, value: dateState.filterBy }),
        React.createElement(CarbonDate, { "data-testid": "e-date-filter", ref: dateRef, allowEmptyValue: true, onChange: handleChange, value: dateState.formattedValue, size: "small", autoComplete: "off", disablePortal: true, inputWidth: 100 }),
        dateState.filterBy === RANGE && (React.createElement(CarbonDate, { allowEmptyValue: true, onChange: handleChangeTo, value: dateState.formattedValueTo, size: "small", autoComplete: "off", disablePortal: true, inputWidth: 100 })),
        dateState.formattedValue !== '' && (React.createElement("div", { className: "e-filter-date-and-container" },
            React.createElement(FilterBySelector, { onChange: filterBySelecterAndOnChange, value: dateState.filterBy }),
            React.createElement(CarbonDate, { allowEmptyValue: true, onChange: handleChangeAnd, value: andDateState.formattedValue, size: "small", autoComplete: "off", disablePortal: true, inputWidth: 100 }),
            dateState.filterBy === RANGE && (React.createElement(CarbonDate, { allowEmptyValue: true, onChange: handleChangeAndTo, value: andDateState.formattedValueTo, size: "small", autoComplete: "off", disablePortal: true, inputWidth: 100 }))))));
});
DateFilter.displayName = 'DateFilter';
export default DateFilter;
//# sourceMappingURL=date-filter.js.map