import * as React from 'react';
import { fetchReferenceFieldSuggestions } from '../../../../../service/graphql-service';
import { nodesToSelectItems } from '../../../../field/reference/reference-utils';
import Loader from 'carbon-react/esm/components/loader';
import { debounce, camelCase, isEqual, set, uniqBy } from 'lodash';
import { localize } from '../../../../../service/i18n-service';
import { xtremConsole } from '../../../../../utils/console';
import { convertDeepBindToPathNotNull } from '../../../../../utils/nested-field-utils';
import Link from 'carbon-react/esm/components/link';
import { SET } from '../../../../types';
import CheckboxComponent from 'carbon-react/esm/components/checkbox/checkbox.component';
import Textbox from 'carbon-react/esm/components/textbox';
import { AUTO_COLUMN_ID } from '../../../../../utils/table-component-utils';
const getItems = async (filterValue, columnDefinition, after) => {
    const valueField = columnDefinition.cellRendererParams.fieldProperties.valueField;
    const isFilterLimitedToDataset = columnDefinition.cellRendererParams.fieldProperties.isFilterLimitedToDataset;
    const nodes = await fetchReferenceFieldSuggestions({
        after,
        contextNode: columnDefinition.cellRendererParams.contextNode,
        fieldProperties: {
            ...columnDefinition.cellRendererParams.fieldProperties,
            isTransient: true,
            columns: undefined,
            helperTextField: undefined,
            minLookupCharacters: 0,
            orderBy: set({}, convertDeepBindToPathNotNull(valueField), 1),
        },
        fieldId: columnDefinition.cellRendererParams.columnId,
        filterValue: filterValue || undefined,
        isFilterLimitedToDataset,
        parentElementId: columnDefinition.cellRendererParams.tableElementId,
        recordContext: {},
        screenId: columnDefinition.screenId,
    });
    return nodesToSelectItems({
        nodes,
        fieldProperties: columnDefinition.cellRendererParams.fieldProperties,
    });
};
const ReferenceFilterItem = React.memo(({ textFilter: searchCriteria, itemValue }) => {
    if (!itemValue) {
        return React.createElement("span", { className: "e-reference-custom-filter-item-value" });
    }
    if (!searchCriteria) {
        return React.createElement("span", { className: "e-reference-custom-filter-item-value" }, itemValue);
    }
    const pos = itemValue.toLowerCase().indexOf(searchCriteria.toLowerCase());
    const firstPart = itemValue.substr(0, pos);
    const highlightedPart = itemValue.substr(pos, searchCriteria.length);
    const lastPart = itemValue.substr(pos + searchCriteria.length);
    return (React.createElement("span", { className: "e-reference-custom-filter-item-value" },
        firstPart,
        React.createElement("span", { className: "e-reference-custom-filter-item-value-highlighted" }, highlightedPart),
        lastPart));
});
ReferenceFilterItem.displayName = 'ReferenceFilterItem';
/**
 * Custom filter component for AG-Grid, handling references.
 *
 * Not implemented:
 * - Support for nested grids
 * - "Select All" functionality
 */
export const ReferenceFilter = React.forwardRef(({ colDef, filterChangedCallback, api }, ref) => {
    const currentlyActiveFilter = React.useRef([]);
    const [textFilter, setTextFilter] = React.useState('');
    const [textFilterDisplayValue, setTextFilterDisplayValue] = React.useState('');
    const [hasNextPage, setHasNextPage] = React.useState(true);
    const [isFetching, setFetching] = React.useState(true);
    const [availableItems, setAvailableItems] = React.useState([]);
    const [selectedItems, setSelectedItems] = React.useState(() => {
        const currentFilter = colDef.field ? api.getFilterModel()?.[colDef.field]?.values : [];
        return currentFilter
            ? currentFilter.map((id) => {
                return {
                    _id: id,
                    id,
                    value: id,
                };
            })
            : [];
    });
    const debouncedSearch = React.useMemo(() => debounce((filter) => {
        setTextFilter(filter);
    }, 500), []);
    // Debounce the actual filter value update so we don't refetch on every key stroke
    const debouncedTextFilterUpdate = React.useCallback(debouncedSearch, [debouncedSearch]);
    React.useEffect(() => {
        getItems(textFilter, colDef)
            .then(items => {
            setAvailableItems(items);
        })
            .catch(e => {
            xtremConsole.error(e);
            setAvailableItems([]);
        })
            .finally(() => {
            setFetching(false);
        });
    }, [colDef, textFilter]);
    React.useEffect(() => {
        // If the selected items equal the selected items that were set last time, we should not rerender the table by calling the callback
        if (!isEqual(selectedItems, currentlyActiveFilter.current)) {
            filterChangedCallback();
            currentlyActiveFilter.current = selectedItems;
        }
    }, [selectedItems, filterChangedCallback]);
    // expose AG Grid Filter Lifecycle callbacks
    React.useImperativeHandle(ref, () => {
        return {
            doesFilterPass() {
                // TODO: Implement validations here when we will have external filter management, like the table view system
                return true;
            },
            // This callback is used to decide whether to display the filtered icon on the column header.
            isFilterActive() {
                // If we have any selected items, then we should consider the filter active
                return selectedItems.length > 0;
            },
            getModel() {
                if (selectedItems.length === 0) {
                    return null;
                }
                return {
                    filterType: SET,
                    values: selectedItems.map(r => r.id),
                    /**
                     * By default, the filtering applied to the value field text value, so we need to pass this flag to indicate that
                     * the records should be filtered by their ID values
                     *  */
                    filterById: true,
                    // This value is used by the ReferenceFloatingFilter component to display the selection summary in the header
                    displayValues: selectedItems.map(r => r.displayedAs || r.value),
                };
            },
            setModel(m) {
                const model = colDef.colId === AUTO_COLUMN_ID && colDef.field
                    ? api.getFilterModel()?.[AUTO_COLUMN_ID]
                    : m;
                // If the new model for the column is empty, then we should unset the list of selected items on the UI
                if (!model) {
                    setTextFilter('');
                    setSelectedItems([]);
                }
                else {
                    setTimeout(() => {
                        const newFilter = [...selectedItems];
                        model.values.forEach((value, idx) => {
                            const ele = newFilter.find(i => i.id === value);
                            if (!ele) {
                                newFilter.push({
                                    id: value,
                                    value,
                                    displayedAs: model.displayValues?.[idx] || value,
                                });
                            }
                        });
                        setSelectedItems(newFilter);
                    }, 50);
                }
                // TODO: Implement when we will have external filter management, like the table view system
            },
        };
    }, [api, colDef.colId, colDef.field, selectedItems]);
    const onTextFilterChange = (ev) => {
        setHasNextPage(true);
        setTextFilterDisplayValue(ev.target.value);
        debouncedTextFilterUpdate(ev.target.value);
    };
    const onSelectionChange = (isChecked, item) => {
        const newFilter = [...selectedItems];
        if (isChecked) {
            newFilter.push(item);
        }
        else {
            const index = newFilter.findIndex(i => i.id === item.id);
            if (index !== -1) {
                newFilter.splice(index, 1);
            }
        }
        setSelectedItems(newFilter);
    };
    const onScroll = async (event) => {
        const listBody = event.target;
        // If the scrollbar is 30 pixels away from the bottom, then we initiate the fetch of the next set of records
        const shouldLoadNextPage = listBody.scrollHeight - listBody.scrollTop <= listBody.clientHeight + 30;
        if (!isFetching && shouldLoadNextPage && hasNextPage) {
            const lastItem = availableItems[availableItems.length - 1];
            const newItems = await getItems(textFilter, colDef, lastItem.__collectionItem?.__cursor);
            setHasNextPage(newItems.length > 0);
            setAvailableItems(uniqBy([...availableItems, ...newItems], i => i.id));
            setFetching(false);
        }
    };
    const onClearSelection = (ev) => {
        ev.preventDefault();
        ev.stopPropagation();
        setSelectedItems([]);
    };
    const debouncedOnScroll = debounce(onScroll, 1000);
    return (React.createElement("div", { className: "e-reference-custom-filter" },
        React.createElement("div", { className: "e-reference-custom-filter-clear-container", "data-testid": "e-reference-custom-filter-clear-button" },
            React.createElement(Link, { onClick: onClearSelection, disabled: selectedItems.length === 0 }, localize('@sage/xtrem-ui/reference-filter-clear-selection', 'Clear selected items'))),
        React.createElement(Textbox, { "data-testid": "e-reference-custom-filter-input", placeholder: localize('@sage/xtrem-ui/filter-search-placeholder', 'Search'), value: textFilterDisplayValue, onChange: onTextFilterChange, size: "small", mx: "8px", my: "4px" }),
        React.createElement("div", { className: "e-reference-custom-filter-body", onScroll: debouncedOnScroll },
            availableItems.map(item => (React.createElement("div", { className: "e-reference-custom-filter-item", "data-testid": "e-reference-custom-filter-item", key: item.id },
                React.createElement(CheckboxComponent, { onChange: (event) => onSelectionChange(!!event.target.checked, item), key: item.id, value: item.id, checked: selectedItems.findIndex(i => i.id === item.id) !== -1, "data-testid": `e-reference-custom-filter-checkbox-label-${camelCase(String(item.value))} e-reference-custom-filter-checkbox-bind-${item.id}` }),
                React.createElement(ReferenceFilterItem, { itemValue: String(item.value), textFilter: textFilter })))),
            isFetching && (React.createElement("div", { className: "e-reference-custom-filter-loader" },
                React.createElement(Loader, null))))));
});
ReferenceFilter.displayName = 'ReferenceFilter';
export default ReferenceFilter;
//# sourceMappingURL=reference-filter.js.map