import { integration } from '@sage/xtrem-ui';
import * as React from 'react';
import * as ReactDOM from 'react-dom';
import { connect, Provider } from 'react-redux';
import { ConnectedXtremHeader } from './component/header';
import { ConnectedXtremNavigation } from './component/navigation';
import * as actions from './standalone-redux/actions';
import { getStore } from './standalone-redux/store';
import { lockRenewal, shouldRenewToken, unlockRenewal } from './service/auth-service';
import { ConnectedMobileNavigation } from './component/mobile-navigation';
import sageTheme from 'carbon-react/esm/style/themes/sage';
import Loader from 'carbon-react/esm/components/loader';
import { localize } from './service/standalone-i18n-service';
import I18nProvider from 'carbon-react/esm/components/i18n-provider';
import CarbonProvider from 'carbon-react/esm/components/carbon-provider';
import { clone, set } from 'lodash';
import { getNewRelicApi, getPendoApi } from './service/telemetry-service';
import { ErrorBoundary } from './error-boundary';
import { Chatbot } from './component/chatbot';
import './style.scss';
const LAST_USER_INTERACTION = 'LAST_USER_INTERACTION';
const IDLE_LIMIT = 60 * 60000;
const customizedTheme = set(clone(sageTheme), 'zIndex.popover', 5000);
export class XtremStandalone extends React.Component {
    constructor(props) {
        super(props);
        this.intervalRef = null;
        this.iframeRef = React.createRef();
        this.xtremUiRef = React.createRef();
        this.appContext = {
            handleNavigation: (path) => this.props.onInternalNavigationChange(path, false),
            updateMenu: this.props.setMenuItems,
            onPageTitleChange: this.props.setPageTitle,
        };
        this.checkAuthTokenState = async (locale, forceRenewToken) => new Promise((resolve, reject) => {
            const lastInteraction = window.sessionStorage.getItem(LAST_USER_INTERACTION);
            const idleTimeLimit = new Date().getTime() - IDLE_LIMIT;
            if (lastInteraction && Number(lastInteraction) < idleTimeLimit) {
                getPendoApi()?.track('userIdleTimeout');
                // redirects user to login page if LAST_USER_INTERACTION was more than 5 min ago
                this.onUserLogout();
            }
            if ((shouldRenewToken() || forceRenewToken) && this.props.loginService && this.iframeRef.current) {
                // It locks the renewal process so other tabs will not try to renew the session at the same time.
                lockRenewal();
                this.iframeRef.current.onerror = () => {
                    this.onUserLogout();
                    reject();
                };
                // Once the page is successfully loaded, it removed the lock
                this.iframeRef.current.onload = () => {
                    unlockRenewal();
                    resolve();
                };
                this.iframeRef.current.src = `${this.props.loginService}/renew?locale=${locale}`;
            }
        });
        this.onTelemetryEvent = (event, data) => {
            if (event === 'unhandledError') {
                const error = data;
                getPendoApi()?.track(event, {
                    message: error.message,
                    stack: error.stack,
                });
                getNewRelicApi()?.noticeError(error);
            }
            else {
                getPendoApi()?.track(event, data);
            }
        };
        this.onUserLogout = () => {
            window.location.replace(`${this.props.loginService}/logout?fromUrl=${document.location.href}`);
        };
        this.onUserUpdate = () => {
            window.sessionStorage.setItem(LAST_USER_INTERACTION, String(new Date().getTime()));
        };
        this.onUserChangeLocale = async (locale) => {
            try {
                await this.props.onInternalNavigationChange(this.props.path, true);
                await this.checkAuthTokenState(locale, true);
                this.setState({ removeXtremUi: true }, () => document.location.reload());
            }
            catch (e) {
                // ignore
            }
        };
        this.onUserUpdate();
        this.checkAuthTokenState(props.user?.locale, false);
        this.state = {
            removeXtremUi: false,
        };
    }
    componentDidMount() {
        // Start listening on browser navigation changes
        window.addEventListener('popstate', this.props.updateNavStateBasedOnLocation);
        this.props.init();
        // Checks every 20 seconds if the session needs to be renewed
        this.intervalRef = setInterval(this.checkAuthTokenState, 20000);
    }
    componentDidUpdate() {
        if (this.props.isNavigationOpen && this.xtremUiRef.current) {
            this.xtremUiRef.current.cleanToasts();
        }
    }
    componentWillUnmount() {
        if (this.intervalRef) {
            clearInterval(this.intervalRef);
            this.intervalRef = null;
        }
    }
    render() {
        if (this.props.user && this.props.config) {
            const locale = this.props.user.locale || 'en-US';
            this.appContext.locale = locale;
            this.appContext.login = this.props.user.id;
            this.appContext.userCode = this.props.user.userCode || this.appContext.login;
            this.appContext.cacheEncryptionKey = this.props.user.clientEncryptionKey;
            this.appContext.agGridLicenceKey = this.props.config?.agGridLicenceKey;
            this.appContext.onDirtyStatusChange = this.props.onDirtyStatusChange;
            this.appContext.onTelemetryEvent = this.onTelemetryEvent;
            this.appContext.onApiRequestError = ({ status }) => {
                if (status === 401) {
                    this.onUserLogout();
                }
            };
            const carbonLocale = {
                link: {
                    skipLinkLabel: () => localize('@sage/xtrem-standalone/skip-to-main-content', 'Skip to main content', {}, this.props.translations, locale),
                },
            };
            const hasPage = !!this.props.path;
            return (React.createElement(ErrorBoundary, null,
                React.createElement(I18nProvider, { locale: carbonLocale },
                    React.createElement(ConnectedXtremHeader, { onUserChangeLocale: this.onUserChangeLocale }),
                    React.createElement(ConnectedMobileNavigation, null),
                    React.createElement(Chatbot, null),
                    React.createElement("div", { className: "xe-body", onKeyDown: this.onUserUpdate, onMouseMove: this.onUserUpdate },
                        !this.props.isExtraSmall && React.createElement(ConnectedXtremNavigation, null),
                        React.createElement("div", { className: "xe-app-container", style: {
                                height: hasPage ? '100vh' : 0,
                                paddingLeft: this.props.isExtraSmall ? 0 : '64px',
                            } },
                            React.createElement(ErrorBoundary, null, !this.state?.removeXtremUi && (React.createElement(integration.XtremUiIndex, { path: this.props.path, applicationContext: this.appContext, ref: this.xtremUiRef })))),
                        !hasPage && React.createElement(integration.DashboardRootComponent, { applicationContext: this.appContext })),
                    React.createElement("iframe", { ref: this.iframeRef, style: { display: 'none' }, title: "auth iframe" }))));
        }
        return (React.createElement("div", { className: "initial-loader-wrapper" },
            React.createElement("div", { className: "initial-loader" },
                React.createElement(Loader, { size: "large" }))));
    }
}
const mapStateToProps = (state) => ({
    path: state.path,
    user: state.user,
    translations: state.translations,
    loginService: state.loginService,
    config: state.config,
    isExtraSmall: state.browser.is.xs,
    isNavigationOpen: state.isNavigationOpen,
    updateNavStateBasedOnLocation: actions.actionStub,
    init: actions.actionStub,
    onInternalNavigationChange: actions.actionStub,
    setPageTitle: actions.actionStub,
    setMenuItems: actions.actionStub,
    onDirtyStatusChange: actions.actionStub,
});
const mapDispatchToProps = (dispatch) => ({
    updateNavStateBasedOnLocation: () => dispatch(actions.updateNavStateBasedOnLocation()),
    onDirtyStatusChange: (isApplicationDirty, preNavigationConfirmation) => dispatch(actions.onDirtyStatusChange(isApplicationDirty, preNavigationConfirmation)),
    init: () => dispatch(actions.init()),
    onInternalNavigationChange: (path, doDirtyCheck = false) => dispatch(actions.onInternalNavigationChange(path, doDirtyCheck)),
    setPageTitle: (title) => {
        dispatch(actions.setPageTitle(title));
    },
    setMenuItems: (menuItems) => dispatch(actions.setMenuItems(menuItems)),
});
// tslint:disable-next-line:variable-name
export const ConnectedXtremStandaloneComponent = connect(mapStateToProps, mapDispatchToProps)(XtremStandalone);
window.start = () => {
    ReactDOM.render(React.createElement(Provider, { store: getStore() },
        React.createElement(CarbonProvider, { theme: customizedTheme },
            React.createElement(ConnectedXtremStandaloneComponent, null))), window.document.getElementById('root'));
};
//# sourceMappingURL=index.js.map