"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runBytecodeFile = exports.runBytecode = exports.compileFile = exports.compileCode = exports.getSystemInfo = exports.sourceMapSetup = void 0;
const fs = require("fs");
const v8 = require("node:v8");
const path = require("path");
const semver = require("semver");
const source_map_support_1 = require("source-map-support");
const vm = require('vm');
const zlib = require('zlib');
const Module = require('module');
const os = require('os');
v8.setFlagsFromString('--no-lazy');
if (Number.parseInt(process.versions.node.split('.')[0], 10) >= 12) {
    v8.setFlagsFromString('--no-flush-bytecode'); // Thanks to A-Parser (@a-parser)
}
const encoding = 'utf8';
const replacedSrcExtension = '.jsb';
const compressedReplacedSrcExt = '.jsbz';
const magicBytes = Buffer.from('#JSB#\n', encoding);
const separatorBytes = Buffer.from('#76ef331c-0587-4aad-adbb-d98eded8605c#\n', encoding);
const sourceMapSupportOptions = {
    retrieveSourceMap(source) {
        // source map exists in the supplied source path
        if (fs.existsSync(`${source}.map`)) {
            return {
                url: source,
                map: fs.readFileSync(`${source}.map`, 'utf8'),
            };
        }
        // get file URL from @sage position
        //  e.g. source = /home/vsts/work/1/s/xtrem-platform/@sage/xtrem-cli/lib/cli.js
        //  fileUrl = @sage/xtrem-cli/lib/cli.js
        const fileUrl = source.substring(source.indexOf('@sage'));
        // filePath will be sth like /app/xtrem/xtrem-services/@sage/xtrem-sales/node_modules/@sage/xtrem-cli/lib/cli.js
        const filePath = path.join(process.cwd(), 'node_modules', fileUrl);
        // check if file path exists the cwd node_modules
        if (fs.existsSync(`${filePath}.map`)) {
            return {
                url: filePath,
                map: fs.readFileSync(`${filePath}.map`, 'utf8'),
            };
        }
        // rootFilePath will be sth like /app/xtrem/xtrem-services/node_modules/@sage/xtrem-cli/lib/cli.js
        const rootFilePath = path.join(process.cwd(), '..', '..', 'node_modules', fileUrl);
        // check if file path exists the root node_modules
        if (fs.existsSync(`${rootFilePath}.map`)) {
            return {
                url: rootFilePath,
                map: fs.readFileSync(`${rootFilePath}.map`, 'utf8'),
            };
        }
        // source map is not found anywhere
        return null;
    },
};
/**
 * install source-map-support with value for retrieveSourceMap function that will be used to retrieve the source map file content
 * @param options
 */
function sourceMapSetup(options = {}) {
    (0, source_map_support_1.install)({ ...sourceMapSupportOptions, ...options });
}
exports.sourceMapSetup = sourceMapSetup;
function getSystemInfo() {
    return { nodeVersion: process.version, arch: os.arch() };
}
exports.getSystemInfo = getSystemInfo;
// TODO: this is a temporary hack to avoid node.js crash when printing stack trace in async/await mode.
const keepSource = false;
function getReplacementSource(originalSource, preservedRangeList) {
    if (keepSource)
        return originalSource;
    let newSource = '';
    for (let index = 0; index < originalSource.length; index += 1) {
        const char = originalSource.charAt(index);
        if (char === '\n') {
            newSource = `${newSource}${char}`;
        }
        else if (preservedRangeList.find((r) => r[0] === index)) {
            const range = preservedRangeList.find((r) => r[0] === index);
            const preservedString = originalSource.substring(range[0], range[1] + 1);
            newSource = `${newSource}${preservedString}`;
            index = range[1];
        }
        else {
            newSource = `${newSource}${' '}`;
        }
    }
    return newSource;
}
/**
 * Generates v8 bytecode buffer.
 * @param   {string} javascriptCode JavaScript source that will be compiled to bytecode.
 * @returns {Buffer} The generated bytecode.
 */
function compileCode(javascriptCode, options) {
    const script = new vm.Script(javascriptCode, {
        filename: options?.filename,
        produceCachedData: true,
    });
    const systemInfo = getSystemInfo();
    if (semver.major(systemInfo.nodeVersion) < 14) {
        throw new Error(`Node version must be at least 14 to use xtrem-bytenode. Current version ${systemInfo.nodeVersion}`);
    }
    let bytecodeBuffer = script.createCachedData && typeof script.createCachedData === 'function'
        ? script.createCachedData()
        : script.cachedData;
    if (options && options.replacementSource) {
        bytecodeBuffer = Buffer.concat([
            magicBytes,
            Buffer.from(JSON.stringify(systemInfo), encoding),
            separatorBytes,
            options.replacementSource,
            separatorBytes,
            bytecodeBuffer, // binary code
        ]);
        if (options.compress) {
            // bytecodeBuffer = Buffer.from(zlib.deflateRawSync(bytecodeBuffer));
            bytecodeBuffer = zlib.deflateSync(bytecodeBuffer, (err) => {
                if (err) {
                    // eslint-disable-next-line no-console
                    console.error('An error occurred:', err);
                    process.exitCode = 1;
                }
            });
        }
    }
    return bytecodeBuffer;
}
exports.compileCode = compileCode;
/**
 * Compiles JavaScript file to .jsb/.jsbz file.
 * @param   {object|string} args
 * @param   {string}          args.filename The JavaScript source file that will be compiled
 * @param   {boolean}         [args.compileAsModule=true] If true, the output will be a commonjs module
 * @param   {string}          [args.output=filename.jsb] The output filename. Defaults to the same path and name of the original file, but with `.jsb/.jsbz` extension.
 * @returns {string}        The compiled filename
 */
function compileFile(args) {
    let filename = '';
    let compileAsModule = true;
    let compiledFilename = '';
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    let getPreservedRanges = (source) => {
        return [];
    };
    let compress = false;
    if (typeof args === 'string') {
        filename = args;
        compileAsModule = true;
        compiledFilename = filename.slice(0, -3) + replacedSrcExtension;
    }
    else if (typeof args === 'object') {
        filename = args.filename;
        compileAsModule = args.compileAsModule === undefined ? compileAsModule : args.compileAsModule;
        compress = !!args.compress;
        const extension = compress ? compressedReplacedSrcExt : replacedSrcExtension;
        if (args.getPreservedRanges) {
            getPreservedRanges = args.getPreservedRanges;
        }
        compiledFilename = args.output || filename.slice(0, -3) + extension;
    }
    let scriptCode = fs.readFileSync(filename, encoding).replace(/^#!.*/, '');
    if (compileAsModule) {
        scriptCode = Module.wrap(scriptCode);
    }
    const preservedRangeList = getPreservedRanges(scriptCode);
    const options = {
        filename,
        compress,
        replacementSource: Buffer.from(getReplacementSource(scriptCode, preservedRangeList), encoding),
    };
    const bytecodeBuffer = compileCode(scriptCode, options);
    fs.writeFileSync(compiledFilename, bytecodeBuffer, { encoding });
    return compiledFilename;
}
exports.compileFile = compileFile;
function invalidFileError(filename) {
    return new Error(`Invalid binary file content: ${filename}`);
}
// TODO: rewrite this function
function fixBytecode(bytecodeBuffer) {
    const dummyBytecode = compileCode('"ಠ_ಠ"');
    if (process.version.startsWith('v8.8') || process.version.startsWith('v8.9')) {
        // Node is v8.8.x or v8.9.x
        dummyBytecode.slice(16, 20).copy(bytecodeBuffer, 16);
        dummyBytecode.slice(20, 24).copy(bytecodeBuffer, 20);
    }
    else if (process.version.startsWith('v12') ||
        process.version.startsWith('v13') ||
        process.version.startsWith('v14') ||
        process.version.startsWith('v15') ||
        process.version.startsWith('v16') ||
        process.version.startsWith('v17') ||
        process.version.startsWith('v18') ||
        process.version.startsWith('v19') ||
        process.version.startsWith('v20')) {
        dummyBytecode.slice(12, 16).copy(bytecodeBuffer, 12);
    }
    else {
        dummyBytecode.slice(12, 16).copy(bytecodeBuffer, 12);
        dummyBytecode.slice(16, 20).copy(bytecodeBuffer, 16);
    }
}
function splitContent(fileContent, filename) {
    const magicPosition = fileContent.indexOf(magicBytes);
    const sepPosition = fileContent.indexOf(separatorBytes);
    if (magicPosition < 0 || sepPosition < 0 || magicPosition > sepPosition) {
        throw invalidFileError(filename);
    }
    const systemInfo = fileContent.subarray(magicPosition + magicBytes.length - 1, sepPosition);
    const nextSepPosition = fileContent.indexOf(separatorBytes, sepPosition + separatorBytes.length + 1);
    if (nextSepPosition < 0) {
        throw invalidFileError(filename);
    }
    const dummyCode = fileContent.subarray(sepPosition + separatorBytes.length, nextSepPosition);
    const bytecodeBuffer = fileContent.subarray(nextSepPosition + separatorBytes.length);
    fixBytecode(bytecodeBuffer);
    return { dummyCode, bytecodeBuffer, systemInfo };
}
/**
 * Runs v8 bytecode buffer and returns the result.
 * @param   {Buffer} bytecodeBuffer The buffer object that was created using compileCode function.
 * @returns {any}    The result of the very last statement executed in the script.
 */
function runBytecode(bytecodeBuffer, filename) {
    const content = splitContent(bytecodeBuffer, filename);
    const script = new vm.Script(content.dummyCode.toString(), {
        cachedData: content.bytecodeBuffer,
    });
    if (script.cachedDataRejected) {
        throw new Error(`Invalid or incompatible cached data (cachedDataRejected): ${filename}`);
    }
    return script.runInThisContext();
}
exports.runBytecode = runBytecode;
/**
 * Runs .jsb/.jsbz file and returns the result.
 * @param   {string} filename
 * @returns {any}    The result of the very last statement executed in the script.
 */
function runBytecodeFile(filename) {
    if (typeof filename !== 'string') {
        throw new Error(`filename must be a string. ${typeof filename} was given.`);
    }
    const bytecodeBuffer = Buffer.from(fs.readFileSync(filename, encoding), encoding);
    return runBytecode(bytecodeBuffer, filename);
}
exports.runBytecodeFile = runBytecodeFile;
function applyCode(module, filename, script) {
    /*
  This part is based on:
  https://github.com/zertosh/v8-compile-cache/blob/7182bd0e30ab6f6421365cee0a0c4a8679e9eb7c/v8-compile-cache.js#L158-L178
  */
    function require(id) {
        return module.require(id);
    }
    require.resolve = (request, options) => {
        return Module._resolveFilename(request, module, false, options);
    };
    require.main = process.mainModule;
    require.extensions = Module._extensions;
    require.cache = Module._cache;
    const compiledWrapper = script.runInThisContext({
        filename,
        lineOffset: 0,
        columnOffset: 0,
        displayErrors: true,
    });
    const dirname = path.dirname(filename);
    const args = [module.exports, require, module, filename, dirname, process, global];
    return compiledWrapper.apply(module.exports, args);
}
function verifySource(systemInfo, filename) {
    const fileSystemInfo = JSON.parse(Buffer.from(systemInfo).toString(encoding));
    if (!fileSystemInfo.arch || !fileSystemInfo.nodeVersion) {
        throw invalidFileError(filename);
    }
    const currentSystemInfo = getSystemInfo();
    if (currentSystemInfo.arch !== fileSystemInfo.arch) {
        throw new Error(`Architecture ${currentSystemInfo.arch} invalid for file ${filename} which was created in ${fileSystemInfo.arch} architecture.`);
    }
    const buildVersion = semver.parse(fileSystemInfo.nodeVersion);
    if (buildVersion == null) {
        throw new Error(`Invalid build version '${fileSystemInfo.nodeVersion}'.`);
    }
    const expectedMajorMinorVersion = `${buildVersion.major}.${buildVersion.minor}`;
    if (!semver.satisfies(currentSystemInfo.nodeVersion, expectedMajorMinorVersion)) {
        throw new Error(`Node version must satisfy major.minor version '${expectedMajorMinorVersion}'. Current version is '${currentSystemInfo.nodeVersion}'.`);
    }
}
function getReplacedSourceScript(filename, compressed = false) {
    let fileContent = fs.readFileSync(filename);
    if (compressed) {
        // fileContent = Buffer.from(zlib.inflateRawSync(fileContent));
        fileContent = zlib.inflateSync(fileContent, (err) => {
            if (err) {
                // eslint-disable-next-line no-console
                console.error('An error occurred:', err);
                process.exitCode = 1;
            }
        });
    }
    const content = splitContent(fileContent, filename);
    verifySource(content.systemInfo, filename);
    const script = new vm.Script(content.dummyCode.toString(), {
        filename: `${filename.substring(0, filename.lastIndexOf('.'))}.js`,
        lineOffset: 0,
        columnOffset: 0,
        displayErrors: true,
        cachedData: content.bytecodeBuffer,
    });
    if (script.cachedDataRejected) {
        throw new Error(`Invalid or incompatible cached data (cachedDataRejected) in ${filename}`);
    }
    return script;
}
Module._extensions[replacedSrcExtension] = (module, filename) => {
    const script = getReplacedSourceScript(filename);
    return applyCode(module, filename, script);
};
Module._extensions[compressedReplacedSrcExt] = (module, filename) => {
    const script = getReplacedSourceScript(filename, true);
    return applyCode(module, filename, script);
};
//# sourceMappingURL=bytenode.js.map