"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DecimalDataType = exports.RoundingMode = void 0;
const xtrem_decimal_1 = require("@sage/xtrem-decimal");
const _ = require("lodash");
const data_type_1 = require("./data-type");
var RoundingMode;
(function (RoundingMode) {
    RoundingMode[RoundingMode["roundUp"] = 0] = "roundUp";
    RoundingMode[RoundingMode["roundDown"] = 1] = "roundDown";
    RoundingMode[RoundingMode["roundCeil"] = 2] = "roundCeil";
    RoundingMode[RoundingMode["roundFloor"] = 3] = "roundFloor";
    RoundingMode[RoundingMode["roundHalfUp"] = 4] = "roundHalfUp";
    RoundingMode[RoundingMode["roundHalfDown"] = 5] = "roundHalfDown";
    RoundingMode[RoundingMode["roundHalfEven"] = 6] = "roundHalfEven";
    RoundingMode[RoundingMode["roundHalfCeil"] = 7] = "roundHalfCeil";
    RoundingMode[RoundingMode["roundHalfFloor"] = 8] = "roundHalfFloor";
})(RoundingMode || (exports.RoundingMode = RoundingMode = {}));
class DecimalDataType extends data_type_1.DataType {
    static { this.defaultRoundingMode = RoundingMode.roundHalfUp; }
    static { this.defaultPrecision = 16; }
    static { this.defaultScale = 2; }
    constructor(options) {
        super('decimal', options);
        this.options = options;
    }
    get roundingMode() {
        return this.options.roundingMode ?? DecimalDataType.defaultRoundingMode;
    }
    get precision() {
        return this.options.precision ?? DecimalDataType.defaultPrecision;
    }
    get scale() {
        // if we have a precision but no scale, assume that scale is 0
        return this.options.scale ?? (this.options.precision === undefined ? DecimalDataType.defaultScale : 0);
    }
    static toImplementation(val) {
        return val instanceof xtrem_decimal_1.Decimal ? val : new xtrem_decimal_1.Decimal(val);
    }
    static fromImplementation(val) {
        return val;
    }
    adaptValue(node, val) {
        if (val == null)
            return val;
        const impl = DecimalDataType.toImplementation(val).toDecimalPlaces(this.scale, this.roundingMode);
        return DecimalDataType.fromImplementation(impl);
    }
    async controlValue(node, cx, val) {
        await super.controlValue(node, cx, val);
        const impl = val && DecimalDataType.toImplementation(val);
        if (this.precision && impl && impl.precision() > this.precision) {
            cx.addDiagnose(data_type_1.ValidationSeverity.error, `${val} exceeds the maximum precision allowed for this field: ${this.precision}.`);
        }
        if (this.scale && impl && impl.decimalPlaces() > this.scale) {
            cx.addDiagnose(data_type_1.ValidationSeverity.error, `${val} exceeds the maximum scale allowed for this field: ${this.scale}.`);
        }
    }
    static toDecimalPlaces(val, scale, roundingMode = DecimalDataType.defaultRoundingMode) {
        const impl = this.toImplementation(val).toDecimalPlaces(scale, roundingMode);
        return this.fromImplementation(impl);
    }
    getMetaData(options) {
        const commonDataTypeMetaData = super.getMetaData(options);
        return {
            ...commonDataTypeMetaData,
            type: 'decimal',
            isDefault: false,
            precision: this.precision,
            scale: this.scale,
            roundingMode: _.invert(_.pickBy(RoundingMode, _.isNumber))[this.roundingMode], // TODO: move the enum from xtrem-shared to xtrem-core to get localization
        };
    }
}
exports.DecimalDataType = DecimalDataType;
//# sourceMappingURL=decimal-data-type.js.map