"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFilterObject = exports.geFilterQuery = void 0;
const xtrem_date_time_1 = require("@sage/xtrem-date-time");
const xtrem_shared_1 = require("@sage/xtrem-shared");
const lodash_1 = require("lodash");
const geFilterQuery = ({ filterType, filterValue, parameter, }) => {
    const remappedValue = parameter ? `{{${filterValue}}}` : filterValue;
    switch (filterType) {
        case xtrem_shared_1.CONTAINS:
            return { _regex: (0, lodash_1.escapeRegExp)(remappedValue), _options: 'i' };
        case xtrem_shared_1.STARTS_WITH:
            return { _regex: `^${(0, lodash_1.escapeRegExp)(remappedValue)}`, _options: 'i' };
        case xtrem_shared_1.ENDS_WITH:
            return { _regex: `${(0, lodash_1.escapeRegExp)(remappedValue)}$`, _options: 'i' };
        case xtrem_shared_1.EQUALS:
            return { _eq: remappedValue };
        case xtrem_shared_1.MATCHES:
            return { _eq: remappedValue };
        case xtrem_shared_1.NOT_EQUALS:
            return { _ne: remappedValue };
        case xtrem_shared_1.MULTI_NOT_EQUALS:
            return { _nin: remappedValue };
        case xtrem_shared_1.GREATER_THAN:
            return { _gt: remappedValue };
        case xtrem_shared_1.GREATER_THAN_EQUAL:
            return { _gte: remappedValue };
        case xtrem_shared_1.LESS_THAN:
            return { _lt: remappedValue };
        case xtrem_shared_1.LESS_THAN_EQUAL:
            return { _lte: remappedValue };
        case xtrem_shared_1.RANGE: {
            const values = remappedValue.toString().split(xtrem_shared_1.RANGE_DIVIDER);
            return {
                _gte: values[0],
                _lte: values[1],
            };
        }
        case xtrem_shared_1.SET: {
            return {
                _in: remappedValue,
            };
        }
        default:
            return {};
    }
};
exports.geFilterQuery = geFilterQuery;
function getFilterObject(filters) {
    return filters?.reduce((acc, { filterType, parameter, filterValue: filterVal, id, data: { type } }) => {
        if (Array.isArray(filterVal) &&
            ((filterVal.length !== 2 && filterType === xtrem_shared_1.RANGE) ||
                (filterType !== xtrem_shared_1.SET && filterType !== xtrem_shared_1.MULTI_NOT_EQUALS && filterType !== xtrem_shared_1.RANGE))) {
            throw new Error(`Unsupported filter. Filter type: ${filterType}, filter value: ${JSON.stringify(filterVal)}`);
        }
        let filterValue = filterVal;
        let filter;
        if (filterType === 'timeFrame') {
            if (type !== 'Date' && type !== 'DateTime') {
                throw new Error(`Invalid filter value for filter type "timeFrame": expected one of ['Date', 'DateTime'], got '${type}'`);
            }
            const allowedFilterValues = [
                'same-month',
                'same-year',
                'previous-month',
                'previous-year',
                'last-7-days',
                'last-30-days',
            ];
            if (!allowedFilterValues.includes(filterValue)) {
                throw new Error(`Invalid filter value for type "timeFrame": expected one of ${JSON.stringify(allowedFilterValues)}, got '${filterValue}'`);
            }
            const range = xtrem_date_time_1.DateRange.getDateRange({ date: new Date().toISOString().slice(0, 10), range: filterValue });
            filter = {
                _gte: range.start,
                _lte: range.end,
            };
        }
        else {
            if (type === 'Boolean') {
                filterValue = filterValue === true || filterValue === 'true';
            }
            if ((type === 'Date' || type === 'DateTime') && !parameter) {
                filterValue = Array.isArray(filterValue)
                    ? filterValue.map(g => g.rawValue).join(xtrem_shared_1.RANGE_DIVIDER)
                    : filterValue.rawValue;
            }
            filter = (0, exports.geFilterQuery)({
                filterType,
                filterValue,
                parameter,
            });
        }
        return (0, xtrem_shared_1.deepMerge)(acc, (0, lodash_1.set)({}, id, { _and: [filter] }));
    }, {});
}
exports.getFilterObject = getFilterObject;
//# sourceMappingURL=index.js.map