"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildIsClearedByResetNotCompliantError = exports.buildGetValueNotCompliantError = exports.buildDefaultValueNotCompliantError = void 0;
/**
 * @fileoverview Check property decorators
 * @author Sage
 */
const xtrem_ts_to_sql_1 = require("@sage/xtrem-ts-to-sql");
const utils_1 = require("@typescript-eslint/utils");
const { AST_NODE_TYPES } = utils_1.TSESTree;
function buildDefaultValueNotCompliantError(propertyName) {
    return `The function ${propertyName}.defaultValue() is too complex to be parsed. You will need to create a DataUpdateAction to support upgrades (see https://confluence.sage.com/display/XTREEM/Upgrade+Engine)`;
}
exports.buildDefaultValueNotCompliantError = buildDefaultValueNotCompliantError;
function buildGetValueNotCompliantError(propertyName) {
    return `The function ${propertyName}.getValue() is too complex to be converted to SQL :  you won't be able to use it in filter and sort operations`;
}
exports.buildGetValueNotCompliantError = buildGetValueNotCompliantError;
function buildIsClearedByResetNotCompliantError(propertyName) {
    return `The function ${propertyName}.isClearedByReset() is too complex to be converted to SQL :  you won't be able to use it in filter and sort operations`;
}
exports.buildIsClearedByResetNotCompliantError = buildIsClearedByResetNotCompliantError;
function getErrorWhenConverting(fn) {
    try {
        new xtrem_ts_to_sql_1.Converter({}).convertFunctionExpression(fn);
        return undefined;
    }
    catch (ex) {
        return ex;
    }
}
function checkDefaultValueFunction(context, fnToTest) {
    const error = getErrorWhenConverting(fnToTest);
    if (!error)
        return;
    const propertyName = getEnclosingPropertyName(fnToTest);
    context.report({
        node: error instanceof xtrem_ts_to_sql_1.ConversionError ? error.node : fnToTest,
        messageId: 'error',
        data: {
            error: buildDefaultValueNotCompliantError(propertyName),
        },
    });
}
function checkGetValueFunction(context, fnToTest) {
    // 'getValue' functions must be "ts-to-sql" compliant
    const error = getErrorWhenConverting(fnToTest);
    if (!error)
        return;
    const propertyName = getEnclosingPropertyName(fnToTest);
    context.report({
        node: fnToTest.parent,
        messageId: 'error',
        data: {
            error: buildGetValueNotCompliantError(propertyName),
        },
        fix: (fixer) => {
            // fnToTest.parent.key is the name of the function
            return fixer.replaceText(getEnclosingFunctionName(fnToTest), 'computeValue');
        },
    });
}
function checkIsClearedByResetFunction(context, fnToTest) {
    // 'isClearedByReset' functions must be "ts-to-sql" compliant
    const error = getErrorWhenConverting(fnToTest);
    if (!error)
        return;
    const propertyName = getEnclosingPropertyName(fnToTest);
    context.report({
        node: fnToTest.parent,
        messageId: 'error',
        data: {
            error: buildIsClearedByResetNotCompliantError(propertyName),
        },
    });
}
const rule = {
    defaultOptions: [],
    meta: {
        type: 'problem',
        docs: {
            url: '',
            description: 'Check property decorators',
            recommended: 'strict',
        },
        fixable: 'code',
        schema: [
        // fill in your schema
        ],
        messages: {
            error: '{{error}}',
        },
    },
    create(context) {
        // Retrieve all the property decorators
        const decoratorsSelector = 'PropertyDefinition > Decorator > CallExpression > ObjectExpression';
        return {
            // Rule: defaultValue() functions must be "ts-to-sql compliant"
            [`${decoratorsSelector} > Property[key.name="defaultValue"][value.type='FunctionExpression'] > FunctionExpression`](defaultValueFunction) {
                checkDefaultValueFunction(context, defaultValueFunction);
            },
            // Rule: getValue() functions must be "ts-to-sql compliant"
            [`${decoratorsSelector} > Property[key.name="getValue"][value.type='FunctionExpression'] > FunctionExpression`](getValueFunction) {
                checkGetValueFunction(context, getValueFunction);
            },
            // Rule: isClearedByResetFunction() functions must be "ts-to-sql compliant"
            [`${decoratorsSelector} > Property[key.name="isClearedByReset"][value.type='FunctionExpression'] > FunctionExpression`](isClearedByResetFunction) {
                checkIsClearedByResetFunction(context, isClearedByResetFunction);
            },
        };
    },
};
function getEnclosingPropertyName(fnToTest) {
    const propertyNode = fnToTest.parent.parent.parent.parent.parent;
    if (!(propertyNode &&
        (propertyNode.type === AST_NODE_TYPES.Property ||
            propertyNode.type === AST_NODE_TYPES.PropertyDefinition) &&
        propertyNode.key.type === AST_NODE_TYPES.Identifier)) {
        throw new Error('cannot find property name');
    }
    return propertyNode.key.name;
}
function getEnclosingFunctionName(fnToTest) {
    const propertyNode = fnToTest.parent;
    if (!(propertyNode &&
        (propertyNode.type === AST_NODE_TYPES.Property ||
            propertyNode.type === AST_NODE_TYPES.PropertyDefinition) &&
        propertyNode.key.type === AST_NODE_TYPES.Identifier)) {
        throw new Error('cannot find function name');
    }
    return propertyNode.key;
}
exports.default = rule;
//# sourceMappingURL=property-decorators-errors.js.map