"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.nodePropertyNameExtractor = exports.visitor = void 0;
const xtrem_shared_1 = require("@sage/xtrem-shared");
const lodash_1 = require("lodash");
const ts = require("typescript");
const utils_1 = require("../utils");
const transformer_utils_1 = require("./transformer-utils");
const excludedAstTypes = new Set([
    ts.SyntaxKind.ImportDeclaration,
    ts.SyntaxKind.TypePredicate,
    ts.SyntaxKind.TypeReference,
    ts.SyntaxKind.PropertyAssignment,
]);
const includedOperations = ['mutation', 'asyncMutation', 'bulkMutation', 'query'];
/**
 * Top-level TS visitor
 * @param prefix prefix for translation keys
 */
const visitor = (dictionary, packageName, dirName, filename, ctx, nodePrefix) => (node) => {
    if (excludedAstTypes.has(node.kind)) {
        return node;
    }
    let prefix = nodePrefix;
    if (ts.isClassDeclaration(node)) {
        const nodeName = node.name?.getText().trim();
        prefix = `${packageName}/${(0, transformer_utils_1.createDictionaryKey)(dirName, (0, lodash_1.snakeCase)(nodeName))}`;
        if (dirName === 'nodes')
            dictionary[`${prefix}__node_name`] = (0, xtrem_shared_1.titleCase)(nodeName);
        const decorator = ts.getDecorators(node)?.[0];
        if (decorator) {
            const bulkMutationNames = (0, utils_1.getCrudBulkMutationNames)(filename, decorator);
            bulkMutationNames.forEach(bulkMutationName => {
                dictionary[`${prefix}__bulkMutation__${bulkMutationName}`] = (0, xtrem_shared_1.titleCase)(bulkMutationName);
            });
            const storage = (0, utils_1.getNodeStorageType)(filename, decorator);
            if (storage === "'sql'") {
                dictionary[`${prefix}__asyncMutation__asyncExport`] = 'Export';
            }
        }
    }
    if (ts.isPropertyDeclaration(node) && !(0, lodash_1.isEmpty)(ts.getDecorators(node))) {
        const propertyName = node.name.getText().trim();
        (0, transformer_utils_1.addDictionaryEntry)(dictionary, `${prefix}__property__${propertyName}`, (0, xtrem_shared_1.titleCase)(propertyName));
        return node;
    }
    if (ts.isMethodDeclaration(node)) {
        const decorator = ts.getDecorators(node)?.[0];
        if (decorator?.expression &&
            ts.isCallExpression(decorator.expression) &&
            ts.isPropertyAccessExpression(decorator.expression?.expression)) {
            const decoratorName = decorator.expression.expression.name?.getText().trim();
            if (includedOperations.includes(decoratorName)) {
                const parameters = node.parameters
                    .map(parameter => {
                    return parameter.name?.getText().trim();
                })
                    .filter(name => {
                    return name !== 'context' && name !== '_context';
                });
                const nodeName = node.name?.getText().trim();
                const operationEntry = `${prefix}__${decoratorName}__${nodeName}`;
                (0, transformer_utils_1.addDictionaryEntry)(dictionary, operationEntry, (0, xtrem_shared_1.titleCase)(nodeName));
                parameters.forEach(parameter => {
                    (0, transformer_utils_1.addDictionaryEntry)(dictionary, `${operationEntry}__parameter__${parameter}`, (0, xtrem_shared_1.titleCase)(parameter));
                });
                return node;
            }
        }
    }
    return ts.visitEachChild(node, (0, exports.visitor)(dictionary, packageName, dirName, filename, ctx, prefix), ctx);
};
exports.visitor = visitor;
/**
 * This transformer is meant to be run BEFORE the 'message-transformer' and its purpose is
 * to wrap some page/sticker decorator properties (see 'includedProperties') with a call to the 'ui.localize' function.
 *
 * @param ctx transformation context
 * @returns the transformed file
 */
function nodePropertyNameExtractor(ctx) {
    return (file) => {
        if (!(0, transformer_utils_1.isNodeLikeArtifactFile)(file) || (0, transformer_utils_1.isTestFile)(file)) {
            return file;
        }
        const filename = file.fileName;
        try {
            const dictionary = {};
            const dirName = (0, utils_1.getDirName)(file, true);
            const nameAndRoot = (0, utils_1.getPackageNameAndRoot)(filename);
            const packageName = nameAndRoot.name;
            const result = ts.visitNode(file, (0, exports.visitor)(dictionary, packageName, dirName, filename, ctx));
            const key = `${packageName}/package__name`;
            dictionary[key] = (0, xtrem_shared_1.titleCase)(packageName);
            (0, transformer_utils_1.writeStringLiteralsToBase)(dictionary, nameAndRoot.root);
            return result;
        }
        catch (err) {
            throw new Error(`${filename}: decorator transformer failed: ${err.message}`);
        }
    };
}
exports.nodePropertyNameExtractor = nodePropertyNameExtractor;
//# sourceMappingURL=node-property-name-extractor.js.map