import type { DeepPartial } from 'ts-essentials';
export declare const pascalCase: (s: string) => string;
/**
 * Converts an identifier to a title
 * Lodash startCase does it but it capitalizes every word. We only want to capitalize the first one.
 */
export declare function titleCase(str: string | undefined): string;
/**
 * Type name for the graphql schema
 * @param fullName fully qualified name in the form of @vendor/package/ArtifactName. Example: @sage/x3-sales/SalesOrder
 */
export declare function schemaTypeName(fullName: string, suffix?: string): string;
export declare function nameToSqlName(name: string): string;
/** Useful to make OR condition between properties in our interface. If we put the two properties Typescript returns an a type error.
 * E.G RequireOnlyOne<interface, 'key1' | 'key2'> */
export declare type RequireOnlyOne<T, Keys extends keyof T = keyof T> = Pick<T, Exclude<keyof T, Keys>> & {
    [K in Keys]-?: Required<Pick<T, K>> & Partial<Record<Exclude<Keys, K>, undefined>>;
}[Keys];
/**
 * @TJS-additionalProperties true
 */
export interface Dict<T> {
    [key: string]: T;
}
export type MakeAllRequired<T> = {
    [P in keyof T]-?: MakeAllRequired<NonNullable<T[P]>>;
};
/**
 * Flattens an input object into a one-level deep object
 * having as keys dot-notated strings that represent object paths.
 * E.g.
 *
 * { a: 'a', b: { c: 'c', d: { e: 'e' } } } => { a: 'a', 'b.c': 'c', 'b.d.e': 'e' }
 *
 *
 * @export
 * @param {object} [obj={}] the input object
 * @param {string} [prefix=''] an optional prefix
 * @param {string} [suffix=''] an optional suffix
 * @returns {Dict<string>} a dot-notated object with paths as keys
 */
export declare function flat(obj?: object, prefix?: string, suffix?: string): Dict<string>;
export type Maybe<T> = T | undefined;
/**
 * Remove all the instrumentation code added by the code coverage
 * @param str
 */
export declare function removeCodeCoverageInstrumentation(str: string): string;
/**
 * Access status values
 * - authorized - user is authorized to access the property or operation
 * - unauthorized - user is unauthorized to access the property or operation
 * - unavailable - property or operation is disabled by a service option
 * - readonly - property is readonly
 * - inactive - operation is inactive
 */
export type AccessStatus = 'authorized' | 'unauthorized' | 'unavailable' | 'readonly' | 'inactive';
export type PromisifyProperties<T> = {
    [K in keyof T]: Promise<T[K] | T[K]>;
};
export interface ArrayMoveResult<R> {
    success?: boolean;
    data: R[];
}
export declare const deepMerge: <T = any>(first: T, second: T | DeepPartial<T>, useArrayAsStart?: boolean, overrideArrays?: boolean) => T;
export declare function arrayMoveMutable<T>(array: T[], fromIndex: number, toIndex: number): void;
export declare function arrayMoveImmutable<T>(array: T[], fromIndex: number, toIndex: number): T[];
export declare function arrayMovePreservingOrder<R extends Record<string, any>, K extends keyof R>({ fromIndex, toValue, orderByKey, data, }: {
    fromIndex: number;
    toValue: R[K];
    orderByKey: K;
    data: R[];
}): ArrayMoveResult<R>;
export type KeysOfValue<T, Condition> = {
    [K in keyof T]: T[K] extends Condition ? K : never;
}[keyof T];
export declare function arrayOrderContiguous<R extends {
    _id: string;
}, K extends KeysOfValue<R, number>>({ orderedData, key, }: {
    orderedData: R[];
    key: K;
}): {
    columnId: K;
    rowId: string;
    value: R[K];
    rowData: R;
}[];
export declare function remapObjectByKey(obj: any, keyToRemap: string, remapFunction: (value: any) => any): any;
//# sourceMappingURL=util.d.ts.map