import { DateRange } from '@sage/xtrem-date-time';
import { CONTAINS, ENDS_WITH, EQUALS, GREATER_THAN, GREATER_THAN_EQUAL, LESS_THAN, LESS_THAN_EQUAL, MATCHES, MULTI_NOT_EQUALS, NOT_EQUALS, RANGE, RANGE_DIVIDER, SET, STARTS_WITH, } from '@sage/xtrem-shared';
import { escapeRegExp } from 'lodash';
const geFilterQuery = ({ filterType, filterValue, id, }) => {
    switch (filterType) {
        case CONTAINS:
            return `${id}.match(${new RegExp(escapeRegExp(filterValue), 'i')})`;
        case STARTS_WITH:
            return `${id}.match(${new RegExp(`^${escapeRegExp(filterValue)}`, 'i')})`;
        case ENDS_WITH:
            return `${id}.match(${new RegExp(`${escapeRegExp(filterValue)}$`, 'i')})`;
        case EQUALS:
            return `${id} === ${filterValue}`;
        case MATCHES:
            return `${id} === ${filterValue}`;
        case NOT_EQUALS:
            return `${id} !== ${filterValue}`;
        case MULTI_NOT_EQUALS:
            return `${filterValue}.find(v => v === ${id}) === undefined`;
        case GREATER_THAN:
            return `${id} > ${filterValue}`;
        case GREATER_THAN_EQUAL:
            return `${id} >= ${filterValue}`;
        case LESS_THAN:
            return `${id} < ${filterValue}`;
        case LESS_THAN_EQUAL:
            return `${id} <= ${filterValue}`;
        case RANGE: {
            const values = filterValue.toString().split(RANGE_DIVIDER);
            return `${id} >= ${values[0]} && ${id} <= ${values[1]}`;
        }
        case SET: {
            return `${filterValue}.find(v => v === ${id}) !== undefined`;
        }
        default:
            return '';
    }
};
export function getJsFilter({ filterType, filterValue: filterVal, id, data: { type }, }) {
    const prefix = `item.${id}`;
    if (Array.isArray(filterVal) &&
        ((filterVal.length !== 2 && filterType === RANGE) ||
            (filterType !== SET && filterType !== MULTI_NOT_EQUALS && filterType !== RANGE))) {
        throw new Error(`Unsupported filter. Filter type: ${filterType}, filter value: ${JSON.stringify(filterVal)}`);
    }
    let filterValue = filterVal;
    let filter;
    if (filterType === 'timeFrame') {
        if (type !== 'Date' && type !== 'DateTime') {
            throw new Error(`Invalid filter value for filter type "timeFrame": expected one of ['Date', 'DateTime'], got '${type}'`);
        }
        const allowedFilterValues = [
            'same-month',
            'same-year',
            'previous-month',
            'previous-year',
            'last-7-days',
            'last-30-days',
        ];
        if (!allowedFilterValues.includes(filterValue)) {
            throw new Error(`Invalid filter value for type "timeFrame": expected one of ${JSON.stringify(allowedFilterValues)}, got '${filterValue}'`);
        }
        const range = DateRange.getDateRange({
            date: new Date().toISOString().slice(0, 10),
            range: filterValue,
        });
        filter = `${prefix} >= ${range.start} && ${prefix} <= ${range.end}`;
    }
    else {
        if (type === 'Boolean') {
            filterValue = filterValue === true || filterValue === 'true';
        }
        if (type === 'Date' || type === 'DateTime') {
            filterValue = Array.isArray(filterValue)
                ? filterValue.map(g => g.rawValue).join(RANGE_DIVIDER)
                : filterValue.rawValue;
        }
        filter = geFilterQuery({
            filterType,
            filterValue,
            id: prefix,
        });
    }
    return filter;
}
//# sourceMappingURL=conditional-block-utils.js.map