import { closeDialog } from '../../service/dialog-service';
import { dispatchContainerValidation } from '../../service/dispatch-service';
import { queryInitialMetadata } from '../../service/graphql-service';
import { showContainerValidationToast } from '../../service/toast-service';
import { WIZARD_FINISHED } from '../../utils/constants';
import { triggerFieldEvent } from '../../utils/events';
import { getPageDefinitionFromState, getVisibleSectionsFromPage } from '../../utils/state-utils';
import { parseScreenValues } from '../../utils/transformers';
import { ActionType } from '../action-types';
export const setPath = (path) => ({
    type: ActionType.SetPath,
    value: path,
});
export const setComponentLoading = (screenId, elementId, isLoading) => ({
    type: ActionType.SetComponentLoading,
    value: { screenId, elementId, isLoading },
});
export const setApplicationContext = (applicationContext) => ({
    type: ActionType.SetApplicationContext,
    value: applicationContext,
});
export const updateUserCustomization = (screenId, userCustomizations) => {
    return { type: ActionType.UpdateUserCustomization, value: { screenId, userCustomizations } };
};
export const addScreenDefinition = (pageDefinition) => ({
    type: ActionType.AddScreenDefinition,
    value: pageDefinition,
});
export const addPlugin = (pluginPackage, pluginDefinition) => ({
    type: ActionType.AddPlugin,
    value: {
        pluginPackage,
        pluginDefinition,
    },
});
export const setScreenDefinitionReady = (screenId) => ({
    type: ActionType.SetScreenDefinitionReady,
    value: screenId,
});
export const setScreenDefinitionDialogId = (screenId, dialogId, pageControlObject, title, subtitle) => ({
    type: ActionType.SetScreenDefinitionDialogId,
    value: { screenId, dialogId, pageControlObject, title, subtitle },
});
export const removeScreenDefinition = (screenId) => ({
    type: ActionType.RemoveScreenDefinition,
    value: screenId,
});
export const setFieldProperties = (screenId, elementId, fieldProperties) => ({
    type: ActionType.SetUiComponentProperties,
    value: {
        screenId,
        elementId,
        fieldProperties,
    },
});
export const setGlobalLoading = (loaderState) => ({
    type: ActionType.SetGlobalLoading,
    value: loaderState,
});
export const setUserCustomizations = (userCustomizations) => ({
    type: ActionType.SetUserCustomizations,
    value: userCustomizations,
});
export const actionStub = () => {
    throw new Error('Action stub called, did you forget to override the the action?');
};
export const addTranslations = (locale, literals) => ({
    type: ActionType.AddTranslations,
    value: {
        locale,
        literals,
    },
});
export const setInitialMetaData = () => async (dispatch, getState) => {
    const state = getState();
    if (!state.applicationPackages) {
        const initialMetaData = await queryInitialMetadata();
        dispatch({
            type: ActionType.SetInitialMetaData,
            value: initialMetaData,
        });
    }
};
export const clearWidgetOptions = () => ({
    type: ActionType.ClearWidgetOptions,
    value: null,
});
/**
 * Sets the active section in the state
 * @param screenId
 * @param activeSection the ID of the active section OR `WIZARD_FINISHED` token if the user is stepping to close the widget.
 * @param isInitialLoad
 * @returns
 */
export const setActiveSection = (screenId, activeSection, isInitialLoad = false) => async (dispatch, getState) => {
    const state = getState();
    const pageDefinition = getPageDefinitionFromState(screenId, state);
    const sections = getVisibleSectionsFromPage(screenId, pageDefinition);
    const currentlyActiveSection = pageDefinition.activeSection || getVisibleSectionsFromPage(screenId, pageDefinition)[0]?.id;
    if (currentlyActiveSection === activeSection) {
        // If the current active section is the same that we are about to change to, we should not proceed
        return;
    }
    if (currentlyActiveSection && pageDefinition.page.$.mode === 'wizard') {
        // In case of wizard mode we need to run the validations before we can change the tabs
        // We need to check if we are stepping forward or backwards in the wizard, we only validate on forward steps
        const indexOfCurrentSection = sections.findIndex(s => s.id === pageDefinition.activeSection);
        const indexOfNextSection = sections.findIndex(s => s.id === activeSection);
        if (indexOfCurrentSection < indexOfNextSection) {
            const result = await dispatchContainerValidation(pageDefinition.page._pageMetadata.screenId, currentlyActiveSection);
            // If the validation fails, we display the page error dialog
            if (result.blockingErrors.length > 0) {
                showContainerValidationToast(pageDefinition.page._pageMetadata, result);
                return;
            }
        }
    }
    if (!isInitialLoad && currentlyActiveSection) {
        await triggerFieldEvent(screenId, currentlyActiveSection, 'onInactive');
    }
    if (activeSection === WIZARD_FINISHED && pageDefinition.dialogId) {
        await closeDialog(pageDefinition.dialogId, parseScreenValues(screenId, true, false, state), true);
        return;
    }
    await dispatch({
        type: ActionType.SetActiveSection,
        value: { activeSection, screenId },
    });
    if (!isInitialLoad && activeSection) {
        await triggerFieldEvent(screenId, activeSection, 'onActive');
    }
};
//# sourceMappingURL=common-actions.js.map