import { cloneDeepWith, isEmpty, uniq } from 'lodash';
import { GENERIC_PLATFORM_WIDGET_PREFIX, getGenericWidgetDefinition } from '../../dashboard/generic-widgets';
import { WidgetType } from '../../dashboard/widgets/abstract-widget';
import * as dashboardService from '../../service/dashboard-service';
import { fetchEnumTranslations } from '../../service/graphql-service';
import * as screenLoaderService from '../../service/screen-loader-service';
import { arePlatformLiteralsMissing } from '../../utils/state-utils';
import { getArtifactDescription } from '../../utils/transformers';
import { ActionType } from '../action-types';
import { addTranslations } from './common-actions';
/**
 * Constructs the widget object depending its type. It checks if the widget is generic user configured or functional code provided, then constructs it.
 */
export const createWidgetObject = (widgetConstructors, type, widgetId, dashboardId, settings) => {
    const isGenericWidget = type.startsWith(GENERIC_PLATFORM_WIDGET_PREFIX);
    if (isGenericWidget) {
        if (!settings) {
            throw new Error('Generic widgets must have a settings object');
        }
        return getGenericWidgetDefinition(settings, dashboardId, widgetId);
    }
    const WidgetConstructor = widgetConstructors[type];
    if (!WidgetConstructor) {
        throw new Error(`No constructor found for widget: ${type}`);
    }
    WidgetConstructor.prototype.__id = widgetId;
    WidgetConstructor.prototype.__dashboardId = dashboardId;
    return new WidgetConstructor();
};
const loadDashboard = async (dashboard, dispatch, getState) => {
    const loadedWidgets = Object.keys(getState().dashboard.widgets);
    const filterWidgetsToLoad = (c) => loadedWidgets.indexOf(c._id) === -1;
    // We should not load widgets that we already have and generic user defined widgets which have the `@sage/xtrem-ui` prefix
    const filterWidgetsToFetch = (c) => filterWidgetsToLoad(c) && !c.type.startsWith(GENERIC_PLATFORM_WIDGET_PREFIX);
    const widgetsNeedLoading = dashboard ? uniq(dashboard.children.filter(filterWidgetsToFetch).map(c => c.type)) : [];
    const widgetConstructors = await screenLoaderService.fetchWidgetDefinitions(getState, dispatch, widgetsNeedLoading);
    if (!widgetConstructors) {
        return;
    }
    const enumsToLoad = dashboard?.children?.reduce((prevValue, currentValue) => {
        return uniq([...prevValue, ...(currentValue.settings?.usedEnums || [])]);
    }, []) || [];
    if (!isEmpty(enumsToLoad)) {
        const locale = getState().applicationContext?.locale || 'en-US';
        const enumTranslations = await fetchEnumTranslations(enumsToLoad, locale);
        dispatch(addTranslations(locale, enumTranslations));
    }
    const widgets = dashboard?.children.filter(filterWidgetsToLoad).map((item) => {
        const widgetId = String(item._id);
        const dashboardId = String(dashboard._id);
        const widgetObject = createWidgetObject(widgetConstructors, item.type, widgetId, dashboardId, item.settings);
        return {
            artifactName: item.type,
            _id: String(item._id),
            widgetObject,
            properties: widgetObject.constructor.prototype.__properties,
            widgetType: widgetObject.constructor.prototype.__type,
            runtimeSettings: {},
        };
    }) || [];
    dispatch({ type: ActionType.AddWidgets, value: widgets });
    if (dashboard) {
        dispatch({ type: ActionType.AddDashboard, value: dashboard });
    }
    const widgetPromises = widgets.map(w => {
        return dispatch(loadWidgetData({ widgetId: w._id }));
    });
    Promise.all(widgetPromises).then(() => {
        dispatch({ type: ActionType.SetLoadingDashboards, value: false });
    });
};
export const fetchUserDashboardDefinition = () => async (dispatch, getState) => {
    dispatch({ type: ActionType.SetLoadingDashboards, value: true });
    const state = getState();
    const locale = state.applicationContext?.locale || 'en-US';
    const shouldFetchPlatformLiterals = arePlatformLiteralsMissing(getState(), locale);
    const { dashboard, stringLiterals, availableDashboards } = await dashboardService.fetchCurrentDashboardDefinition(shouldFetchPlatformLiterals);
    if (shouldFetchPlatformLiterals) {
        dispatch(addTranslations(locale, stringLiterals));
    }
    await loadDashboard(dashboard, dispatch, getState);
    if (availableDashboards) {
        dispatch({ type: ActionType.AddAvailableDashboards, value: availableDashboards });
    }
};
export const setSelectedDashboard = (id) => async (dispatch, getState) => {
    getState().applicationContext?.onTelemetryEvent?.('dashboardTabSelected', {});
    dispatch({ type: ActionType.SetLoadingDashboards, value: true });
    const dashboard = await dashboardService.setSelectedDashboard(id);
    await loadDashboard(dashboard, dispatch, getState);
};
/**
 * Clone a dashboard based on its ID, then marks it automatically selected
 * @param idToClone
 * @returns
 */
export const cloneDashboard = (idToClone) => async (dispatch, getState) => {
    getState().applicationContext?.onTelemetryEvent?.('dashboardCloned', {});
    const { _id } = await dashboardService.cloneDashboard(idToClone);
    const dashboard = await dashboardService.setSelectedDashboard(_id);
    await loadDashboard(dashboard, dispatch, getState);
};
/**
 * Creates an empty dashboard, then marks it automatically selected
 * @param idToClone
 * @returns
 */
export const createEmptyDashboard = () => async (dispatch, getState) => {
    getState().applicationContext?.onTelemetryEvent?.('emptyDashboardCreated', {});
    const { _id } = await dashboardService.createEmptyDashboard();
    const dashboard = await dashboardService.setSelectedDashboard(_id);
    await loadDashboard(dashboard, dispatch, getState);
};
export const deleteDashboard = (dashboardId) => async (dispatch, getState) => {
    getState().applicationContext?.onTelemetryEvent?.('dashboardDeleted', {});
    await dashboardService.deleteDashboard(dashboardId);
    dispatch({ type: ActionType.RemoveDashboard, value: dashboardId });
    dispatch(fetchUserDashboardDefinition());
};
export function getNewTableData({ widgetDefinition, data }) {
    const tableProperties = widgetDefinition.properties;
    const widgetObject = cloneDeepWith(widgetDefinition.widgetObject, (_value, key, object) => {
        // Here we override the developer API so the new data can be accessed
        if (key === '_$' && object === widgetDefinition.widgetObject) {
            return { ...widgetDefinition.widgetObject.$, data };
        }
        return undefined;
    });
    const tableContent = typeof tableProperties.content === 'function'
        ? tableProperties.content.apply(widgetObject)
        : tableProperties.content;
    return (tableContent ?? []).map((r) => ({
        ...r,
        i: r._id,
        image: r.image ? `data:image;base64,${r.image}` : undefined,
    }));
}
export const loadWidgetData = ({ widgetId, forceRefetch = false, queryArgs, dataMode = 'SET', }) => async (dispatch, getState) => {
    const state = getState();
    const widgetDefinition = state.dashboard.widgets[widgetId];
    const isTableWidget = widgetDefinition.widgetType === WidgetType.table;
    if (dataMode === 'SET' || !isTableWidget) {
        dispatch({ type: ActionType.SetWidgetData, value: { widgetId, data: null } });
    }
    if (!widgetDefinition) {
        throw new Error(`Widget ${widgetId} is not in the state.`);
    }
    const artifactDescription = getArtifactDescription(widgetDefinition.artifactName);
    const pck = `${artifactDescription.vendor}/${artifactDescription.package}`;
    const version = state.applicationPackages[pck];
    const data = await dashboardService.fetchWidgetData({
        widgetDefinition,
        version,
        locale: state.applicationContext?.locale,
        forceRefetch,
        queryArgs,
    });
    if (dataMode === 'SET' || !isTableWidget) {
        dispatch({ type: ActionType.SetWidgetData, value: { widgetId, data } });
    }
    else {
        dispatch({ type: ActionType.AddWidgetData, value: { widgetId, data } });
    }
    if (isTableWidget) {
        return getNewTableData({ widgetDefinition, data });
    }
    return data;
};
export const setWidgetOptions = (dashboardId, widgetId, options) => ({
    type: ActionType.SetWidgetOptions,
    value: {
        dashboardId,
        widgetId,
        options,
    },
});
//# sourceMappingURL=dashboard-actions.js.map