import { isEmpty, isNil } from 'lodash';
import { openDirtyPageConfirmationDialog } from '../../service/dirty-state-service';
import { commitTransaction } from '../../service/transactions-service';
import { resolveByValue } from '../../utils/resolve-value-utils';
import { getPageDefinitionFromState, getSidebarNestedFields, isSidebarDirty } from '../../utils/state-utils';
import { cleanMetadataFromRecord } from '../../utils/transformers';
import { ActionType } from '../action-types';
const handleOnSidebarOpenEvent = async (sidebarState, screenId, pageDefinition, recordValue) => {
    if (sidebarState?.onRecordOpened) {
        // Trigger on record open event if the functional developer declared one
        await sidebarState.onRecordOpened.apply(pageDefinition.page, [
            recordValue._id,
            cleanMetadataFromRecord(recordValue),
        ]);
        commitTransaction(screenId);
    }
};
export const closeTableSidebar = (suppressDiscardEvent = false) => async (dispatch, getState) => {
    const state = getState();
    const sidebarState = state.sidebar;
    if (!suppressDiscardEvent) {
        if (!sidebarState) {
            throw new Error('Cannot close the sidebar if it is not even defined.');
        }
        if (isSidebarDirty(state)) {
            try {
                await openDirtyPageConfirmationDialog(sidebarState.screenId);
            }
            catch {
                // If the user rejects the dialog, we won't continue
                return;
            }
        }
        const pageDefinition = getPageDefinitionFromState(sidebarState.screenId, state);
        const tableProperties = pageDefinition.metadata.uiComponentProperties[sidebarState.elementId];
        if (!tableProperties.sidebar) {
            throw new Error(`No sidebar is defined for ${sidebarState.elementId}`);
        }
        if (tableProperties.sidebar.onRecordDiscarded) {
            await tableProperties.sidebar.onRecordDiscarded.apply(pageDefinition.page);
            commitTransaction(sidebarState.screenId);
        }
        const tableValue = pageDefinition.values[sidebarState.elementId];
        tableValue.cancelRecordTransaction({ recordId: sidebarState.recordId || '-1' });
    }
    dispatch({
        type: ActionType.CloseTableSidebar,
        value: undefined,
    });
};
export const openTableSidebar = (screenId, elementId, recordId) => async (dispatch, getState) => {
    dispatch({
        type: ActionType.OpenTableSidebar,
        value: {
            elementId,
            screenId,
        },
    });
    let pageDefinition = getPageDefinitionFromState(screenId, getState());
    const tableProperties = pageDefinition.metadata.uiComponentProperties[elementId];
    if (!tableProperties.sidebar) {
        throw new Error(`No sidebar is defined for ${elementId}`);
    }
    const sidebarLayoutDefinition = resolveByValue({
        screenId,
        propertyValue: tableProperties.sidebar.layout,
        rowValue: null,
        fieldValue: null,
        skipHexFormat: true,
    });
    if (!sidebarLayoutDefinition) {
        throw new Error(`No sidebar layout is defined for ${elementId}`);
    }
    pageDefinition = getPageDefinitionFromState(screenId, getState());
    const tableValue = pageDefinition.values[elementId];
    const recordValue = recordId
        ? tableValue.getRecordByIdAndLevel({ id: recordId })
        : await tableValue.createNewPhantomRow({ isUncommitted: true });
    // We need fresh values here in case of the callback made any changes
    pageDefinition = getPageDefinitionFromState(screenId, getState());
    let prevRecord;
    let nextRecord;
    if (recordId) {
        // We don't need to start a record transaction for new record because the createNewPhantomRow is already called with the isUncommitted flag.
        tableValue.startRecordTransaction({ recordId });
        prevRecord = await tableValue.getPreviousRecord({ recordId });
        nextRecord = await tableValue.getNextRecord({ recordId, tableProperties });
    }
    getState().applicationContext?.onTelemetryEvent?.('tableSidebarOpened', {
        elementId,
        screenId,
    });
    await handleOnSidebarOpenEvent(tableProperties.sidebar, screenId, pageDefinition, recordValue);
    dispatch({
        type: ActionType.SetTableSidebarRecord,
        value: {
            _id: recordId || recordValue._id,
            isNewRecord: isNil(recordId),
            hasNextRecord: !!nextRecord,
            hasPreviousRecord: !!prevRecord,
        },
    });
};
export const confirmTableSidebar = (addNewRecord = false) => async (dispatch, getState) => {
    const state = getState();
    const sidebarState = state.sidebar;
    if (!sidebarState || !sidebarState.screenId) {
        throw new Error('Cannot confirm the sidebar if it is not even defined.');
    }
    const pageDefinition = getPageDefinitionFromState(sidebarState.screenId, state);
    const tableProperties = pageDefinition.metadata.uiComponentProperties[sidebarState.elementId];
    if (!tableProperties.sidebar) {
        throw new Error(`No sidebar is defined for ${sidebarState.elementId}`);
    }
    const collectionValue = pageDefinition.values[sidebarState.elementId];
    const recordId = sidebarState.recordId;
    if (recordId) {
        const rawRecord = collectionValue.getRawRecord({
            id: recordId,
            isUncommitted: true,
        });
        const result = await collectionValue.runValidationOnRecord({
            recordData: rawRecord,
            columnsToRevalidate: getSidebarNestedFields(state),
            isUncommitted: true,
        });
        if (!isEmpty(result)) {
            // If validations fail, we won't proceed
            return;
        }
        const updateRecord = collectionValue.commitRecord({ recordId });
        if (tableProperties.sidebar.onRecordConfirmed) {
            await tableProperties.sidebar.onRecordConfirmed.apply(pageDefinition.page, [
                recordId,
                cleanMetadataFromRecord(updateRecord),
            ]);
        }
    }
    getState().applicationContext?.onTelemetryEvent?.('tableSidebarConfirmed', {
        elementId: sidebarState.elementId,
        screenId: sidebarState.screenId,
    });
    if (!addNewRecord) {
        dispatch(closeTableSidebar(true));
    }
    else {
        dispatch(openTableSidebar(sidebarState.screenId, sidebarState.elementId));
    }
};
const sidebarNextOrPrevRecord = async (dispatch, getState, isNext) => {
    dispatch({
        type: ActionType.SetTableSidebarLoading,
        value: true,
    });
    const sidebarState = getState().sidebar;
    if (!sidebarState || !sidebarState.screenId) {
        throw new Error('Cannot select the next record if the sidebar is not even defined.');
    }
    const pageDefinition = getPageDefinitionFromState(sidebarState.screenId, getState());
    const tableProperties = pageDefinition.metadata.uiComponentProperties[sidebarState.elementId];
    if (!tableProperties.sidebar) {
        throw new Error(`No sidebar is defined for ${sidebarState.elementId}`);
    }
    const collectionValue = pageDefinition.values[sidebarState.elementId];
    const recordId = sidebarState.recordId;
    if (!recordId) {
        return;
    }
    if (isNext) {
        getState().applicationContext?.onTelemetryEvent?.('tableSidebarNextRecordButtonClicked', {
            elementId: sidebarState.elementId,
            screenId: sidebarState.screenId,
        });
        const nextRecord = await collectionValue.getNextRecord({ recordId, tableProperties });
        if (nextRecord) {
            collectionValue.cancelRecordTransaction({ recordId });
            collectionValue.startRecordTransaction({ recordId: nextRecord._id });
            await handleOnSidebarOpenEvent(tableProperties.sidebar, sidebarState.screenId, pageDefinition, nextRecord);
            const hasNextRecord = await collectionValue.getNextRecord({
                recordId: nextRecord._id,
                tableProperties,
            });
            dispatch({
                type: ActionType.SetTableSidebarRecord,
                value: { _id: nextRecord._id, hasNextRecord: !!hasNextRecord, hasPreviousRecord: true },
            });
        }
    }
    else {
        getState().applicationContext?.onTelemetryEvent?.('tableSidebarPreviousRecordButtonClicked', {
            elementId: sidebarState.elementId,
            screenId: sidebarState.screenId,
        });
        const previousRecord = await collectionValue.getPreviousRecord({
            recordId,
        });
        if (previousRecord) {
            collectionValue.cancelRecordTransaction({ recordId });
            collectionValue.startRecordTransaction({ recordId: previousRecord._id });
            await handleOnSidebarOpenEvent(tableProperties.sidebar, sidebarState.screenId, pageDefinition, previousRecord);
            const hasPreviousRecord = await collectionValue.getPreviousRecord({
                recordId: previousRecord._id,
            });
            dispatch({
                type: ActionType.SetTableSidebarRecord,
                value: { _id: previousRecord._id, hasNextRecord: true, hasPreviousRecord: !!hasPreviousRecord },
            });
        }
    }
};
export const selectSidebarNextRecord = () => async (dispatch, getState) => {
    return sidebarNextOrPrevRecord(dispatch, getState, true);
};
export const selectSidebarPreviousRecord = () => async (dispatch, getState) => {
    return sidebarNextOrPrevRecord(dispatch, getState, false);
};
export const setSidebarDirty = () => ({ type: ActionType.SetTableSidebarDirty });
//# sourceMappingURL=table-sidebar-actions.js.map