/**
 * This service updates the page definitions to include user defined custom fields, which are loaded to the product
 * from the `customFields` page metadata property
 */
import { CUSTOM_DATA_PROPERTY } from '@sage/xtrem-shared';
import { insertColumn } from '../component/abstract-decorator-utils';
import { navigationPanelId } from '../component/container/navigation-panel/navigation-panel-types';
import { checkboxField } from '../component/field/checkbox/checkbox-decorator';
import { dateField } from '../component/field/date/date-decorator';
import { multiDropdownField } from '../component/field/multi-dropdown/multi-dropdown-decorator';
import { numericField } from '../component/field/numeric/numeric-decorator';
import { selectField } from '../component/field/select/select-decorator';
import { switchField } from '../component/field/switch/switch-decorator';
import { textField } from '../component/field/text/text-decorator';
import * as nestedFields from '../component/nested-fields';
import { FieldKey } from '../component/types';
const createBasicFieldPropertiesFromCustomFieldConfig = (f) => {
    const properties = f.componentAttributes || {};
    return {
        ...properties,
        title: properties.title || f.name,
        // We use standard deep binding for the field.
        bind: { [CUSTOM_DATA_PROPERTY]: { [f.name]: true } },
    };
};
const getOptionsFromProperties = (f) => {
    const availableOptions = f.componentAttributes.options || [];
    const optionDictionary = availableOptions
        .filter((item) => item.isActive === undefined || item.isActive)
        .reduce((prevValue, currentValue) => {
        prevValue[currentValue.technicalName] = currentValue.displayedName;
        return prevValue;
    }, {});
    return {
        options: Object.keys(optionDictionary),
        map: v => optionDictionary[v],
    };
};
export const addCustomFieldsToPageBody = (pageConstructor, pageMetadata, customFields) => {
    if (!customFields) {
        return;
    }
    customFields
        // If the page is not a selected destination type, we stop processing.
        .filter(f => f.destinationTypes?.includes('page') && f.anchorPropertyName)
        .forEach(f => {
        const properties = {
            ...createBasicFieldPropertiesFromCustomFieldConfig(f),
            ...(f.anchorPosition === 'before'
                ? {
                    insertBefore() {
                        return f.anchorPropertyName
                            ? pageMetadata.controlObjects[f.anchorPropertyName]
                            : null;
                    },
                }
                : {}),
            ...(f.anchorPosition === 'after'
                ? {
                    insertAfter() {
                        return f.anchorPropertyName
                            ? pageMetadata.controlObjects[f.anchorPropertyName]
                            : null;
                    },
                }
                : {}),
            parent() {
                const anchorFieldProps = f.anchorPropertyName
                    ? pageMetadata.uiComponentProperties[f.anchorPropertyName]
                    : null;
                return anchorFieldProps?.parent?.apply(pageMetadata.target);
            },
        };
        // We call the decorator function of the corresponding field type. It creates the control object, updates the layout, just like for an ordinary field.
        switch (f.componentType) {
            case 'textField':
                textField(properties)(pageConstructor.prototype, f.name);
                break;
            case 'numericField':
                numericField(properties)(pageConstructor.prototype, f.name);
                break;
            case 'checkboxField':
                checkboxField(properties)(pageConstructor.prototype, f.name);
                break;
            case 'switchField':
                switchField(properties)(pageConstructor.prototype, f.name);
                break;
            case 'dateField':
                dateField(properties)(pageConstructor.prototype, f.name);
                break;
            case 'selectField':
                selectField({ ...properties, ...getOptionsFromProperties(f) })(pageConstructor.prototype, f.name);
                break;
            case 'multiDropdownField':
                multiDropdownField({ ...properties, ...getOptionsFromProperties(f) })(pageConstructor.prototype, f.name);
                break;
            default:
                throw new Error(`Unhandled custom field: ${f.componentType}`);
        }
    });
};
const getNestedFieldFromCustomFieldDefinition = (f) => {
    const properties = {
        ...createBasicFieldPropertiesFromCustomFieldConfig(f),
        ...(f.anchorPosition === 'before' ? { insertBefore: f.anchorPropertyName } : {}),
        ...(f.anchorPosition === 'after' ? { insertAfter: f.anchorPropertyName } : {}),
    };
    switch (f.componentType) {
        case 'textField':
            return nestedFields.text(properties);
        case 'numericField':
            return nestedFields.numeric(properties);
        case 'checkboxField':
            return nestedFields.checkbox(properties);
        case 'switchField':
            return nestedFields.switch(properties);
        case 'dateField':
            return nestedFields.date(properties);
        case 'selectField':
            return nestedFields.select({ ...properties, ...getOptionsFromProperties(f) });
        case 'multiDropdownField':
            return nestedFields.multiDropdown({ ...properties, ...getOptionsFromProperties(f) });
        default:
            throw new Error(`Unhandled custom field column: ${f.componentType}`);
    }
};
export const componentsWithLookup = [
    FieldKey.Reference,
    FieldKey.FilterSelect,
    FieldKey.MultiReference,
    FieldKey.Pod,
];
export const addCustomColumnToNestedFields = (elementId, componentType, columns, onlyInsertIfFound, customFields) => {
    const nestedCustomFields = getCustomFields(elementId, componentType, customFields);
    if (nestedCustomFields.length === 0) {
        return;
    }
    nestedCustomFields.forEach(nestedField => {
        insertColumn(columns, nestedField, false, onlyInsertIfFound);
    });
};
export const getCustomFields = (elementId, componentType, customFields) => {
    if (!customFields) {
        return [];
    }
    return customFields
        .filter(f => {
        if (elementId === navigationPanelId) {
            return f.destinationTypes?.includes('navigationBar');
        }
        if (componentsWithLookup.includes(componentType)) {
            return f.destinationTypes?.includes('lookup');
        }
        return f.destinationTypes?.includes('page');
    })
        .map(getNestedFieldFromCustomFieldDefinition);
};
/*
 * This function checks the nodeTypes in order to determine if is a NOT a vital Node and set the custom field as readonly
 */
export const isCustomFieldReadOnly = (nodeTypes, pageInputNode, bind) => {
    const nodeType = nodeTypes[pageInputNode];
    // Copy of customFields to avoid mutation
    if (nodeType && Object.keys(nodeType).find(key => key === bind)) {
        return false;
    }
    return true;
};
//# sourceMappingURL=customization-service.js.map