import { MD5 } from 'crypto-js';
import { escape as lodashEscape, memoize, noop, sortBy, uniqBy } from 'lodash';
import * as showdown from 'showdown';
import { GENERIC_PLATFORM_WIDGET_PREFIX } from '../dashboard/generic-widgets';
import { WidgetType } from '../dashboard/widgets/abstract-widget';
import * as xtremRedux from '../redux';
import { xtremConsole } from '../utils/console';
import { PREVIEW_WIDGET_ID } from '../utils/constants';
import { resolveDetailedIcon } from '../utils/detailed-icons-utils';
import { resolveImageUrl } from '../utils/vp-utils';
import { getPlatformStringLiteralsSchemaFragment } from './graphql-query-builder';
import { executeGraphqlQuery } from './graphql-utils';
import { localize } from './i18n-service';
import { processStringLiteralsFromResponse } from './screen-loader-service';
import { executeVisualProcessLink } from './visual-process-service';
export const DASHBOARD_SCREEN_ID = '$dashboard';
export const DASHBOARD_DETAILS_QUERY = {
    _id: true,
    title: true,
    children: {
        _id: true,
        type: true,
        settings: true,
        x: true,
        y: true,
        h: true,
        w: true,
    },
};
export const fetchCurrentDashboardDefinition = async (shouldFetchPlatformLiterals = false) => {
    // Load source code of the page and its extensions from the server
    const query = {
        dashboard: {
            selectedDashboard: {
                ...DASHBOARD_DETAILS_QUERY,
            },
            availableDashboards: {
                _id: true,
                title: true,
            },
        },
    };
    if (shouldFetchPlatformLiterals) {
        query.strings = getPlatformStringLiteralsSchemaFragment();
    }
    const { data } = await executeGraphqlQuery({
        query,
        endpoint: '/metadata',
    });
    const dashboard = data.dashboard.selectedDashboard;
    if (!dashboard) {
        return { dashboard: null, stringLiterals: {}, availableDashboards: null };
    }
    dashboard.children = sortBy(dashboard.children?.map((c) => ({
        ...c,
        settings: JSON.parse(c.settings),
    })) || [], ['y', 'x']);
    const stringLiterals = shouldFetchPlatformLiterals
        ? processStringLiteralsFromResponse(undefined, data)
        : {};
    const availableDashboards = data.dashboard.availableDashboards;
    return { dashboard, stringLiterals, availableDashboards };
};
export const fetchWidgetCategories = async () => {
    const response = await executeGraphqlQuery({
        query: { dashboard: { getWidgetCategories: { key: true, title: true } } },
        endpoint: '/metadata',
    });
    if (Array.isArray(response.data?.dashboard?.getWidgetCategories)) {
        return response.data.dashboard.getWidgetCategories.reduce((prevValue, currentValue) => {
            return { ...prevValue, [currentValue.key]: currentValue.title };
        }, {});
    }
    throw new Error('Failed to load widget categories from the server');
};
export const fetchDashboardTemplates = async () => {
    const query = {
        dashboard: {
            templates: {
                _id: true,
                title: true,
                description: true,
                listIcon: true,
            },
        },
    };
    const { data } = await executeGraphqlQuery({
        query,
        endpoint: '/metadata',
    });
    return data.dashboard.templates;
};
export const setSelectedDashboard = async (selectedId) => {
    const { data } = await executeGraphqlQuery({
        query: {
            mutation: {
                dashboard: {
                    selectDashboard: {
                        __args: {
                            dashboardId: selectedId,
                        },
                        ...DASHBOARD_DETAILS_QUERY,
                    },
                },
            },
        },
        endpoint: '/metadata',
    });
    if (data.dashboard.selectDashboard) {
        const children = data.dashboard.selectDashboard.children?.map((c) => ({
            ...c,
            settings: c.settings ? JSON.parse(c.settings) : {},
        }));
        return { ...data.dashboard.selectDashboard, children };
    }
    throw new Error('Failed to select dashboard');
};
export const cloneDashboard = async (dashboardIdToClone) => {
    const { data, error } = await executeGraphqlQuery({
        query: {
            mutation: {
                dashboard: {
                    cloneDashboard: {
                        __args: {
                            dashboardId: dashboardIdToClone,
                        },
                        _id: true,
                        title: true,
                    },
                },
            },
        },
        endpoint: '/metadata',
    });
    if (!error && data.dashboard.cloneDashboard) {
        return data.dashboard.cloneDashboard;
    }
    throw new Error('Failed to clone dashboard');
};
export const deleteDashboard = async (dashboardIdToDelete) => {
    const { error } = await executeGraphqlQuery({
        query: {
            mutation: {
                dashboard: {
                    deleteDashboard: {
                        __args: {
                            dashboardId: dashboardIdToDelete,
                        },
                    },
                },
            },
        },
        endpoint: '/metadata',
    });
    if (error) {
        xtremConsole.error(error);
        throw new Error('Failed to delete dashboard');
    }
};
export const createEmptyDashboard = async () => {
    const { data, error } = await executeGraphqlQuery({
        query: {
            mutation: {
                dashboard: {
                    createDashboard: {
                        ...DASHBOARD_DETAILS_QUERY,
                    },
                },
            },
        },
        endpoint: '/metadata',
    });
    if (!error && data.dashboard.createDashboard) {
        return data.dashboard.createDashboard;
    }
    throw new Error('Failed to create dashboard');
};
export const updateDashboard = async (dashboardDefinition) => {
    const { data, error } = await executeGraphqlQuery({
        query: {
            mutation: {
                dashboard: {
                    updateDashboardLayout: {
                        __args: {
                            dashboardId: dashboardDefinition._id,
                            layout: JSON.stringify(dashboardDefinition),
                        },
                        ...DASHBOARD_DETAILS_QUERY,
                    },
                },
            },
        },
        endpoint: '/metadata',
    });
    if (!error && data.dashboard.updateDashboardLayout) {
        data.dashboard.updateDashboardLayout.children = sortBy(data.dashboard.updateDashboardLayout.children?.map((c) => ({
            ...c,
            settings: JSON.parse(c.settings),
        })) || [], ['y', 'x']);
        return data.dashboard.updateDashboardLayout;
    }
    throw new Error('Failed to update dashboards.');
};
/**
 * Fetches widget list from the server and adds "Others" category to those which doesn't have one.
 * It also adds a _sortValue as the items should be categorized alphabetically, e
 * except the Others, so we cannot sort them just by their label
 * @returns
 */
export const getWidgetList = async () => {
    const { data } = await executeGraphqlQuery({
        query: {
            widgets: {
                key: true,
                category: true,
                categoryLabel: true,
                type: true,
                title: true,
                description: true,
            },
        },
        endpoint: '/metadata',
    });
    if (!data.widgets || !Array.isArray(data.widgets)) {
        throw new Error('Failed to fetch widget list.');
    }
    const otherCategory = {
        _id: '_OTHERS',
        title: localize('@sage/xtrem-ui/dashboard-widget-category-others', 'Others'),
        sortValue: Number.MAX_SAFE_INTEGER,
    };
    const uniqueServerCategories = uniqBy(data.widgets.filter((w) => !!w.category).map((w) => ({ _id: w.category, title: w.categoryLabel })), w => w._id);
    const sortedServerCategories = sortBy(uniqueServerCategories, w => w.title).map((category, index) => ({ ...category, sortValue: index + 1 }));
    const categories = [...sortedServerCategories, otherCategory];
    const widgets = data.widgets.map((w) => ({
        _id: w.key,
        type: w.type,
        description: w.description,
        title: w.title,
        category: w.category ? categories.find(c => c._id === w.category) || otherCategory : otherCategory,
    }));
    return { widgets, categories };
};
const updateWidgetOptions = (dashboardId, widgetId, shouldRefreshData, options = {}) => {
    const thunkDispatch = xtremRedux.getStore().dispatch;
    thunkDispatch(xtremRedux.actions.setWidgetOptions(dashboardId, widgetId, options));
    if (shouldRefreshData) {
        refreshWidget(widgetId);
    }
};
const refreshWidget = (widgetId) => {
    const thunkDispatch = xtremRedux.getStore().dispatch;
    thunkDispatch(xtremRedux.actions.loadWidgetData({ widgetId, forceRefetch: true }));
};
export const fetchWidgetPage = async ({ queryArgs, widgetId, }) => {
    const thunkDispatch = xtremRedux.getStore().dispatch;
    return thunkDispatch(xtremRedux.actions.loadWidgetData({ widgetId, forceRefetch: false, queryArgs, dataMode: 'ADD' }));
};
const getStaticContentWidgetConfig = (widgetId, widgetDefinition) => {
    const staticContentProperties = widgetDefinition.properties;
    const mdContent = resolveWidgetProperty(staticContentProperties.content, widgetDefinition.widgetObject || widgetId) || '';
    const converter = new showdown.Converter();
    return {
        type: 'basic',
        data: {
            htmlContent: `<div style="font-size: 14px">${converter.makeHtml(lodashEscape(mdContent))}</div>`,
        },
    };
};
const createAxesConfig = (widgetDefinition) => (s) => {
    const { onClick, tooltipContent } = s;
    return {
        ...s,
        onClick: onClick
            ? ({ record, primaryValue, secondaryValue }) => resolveWidgetProperty(onClick, widgetDefinition.widgetObject, record, primaryValue, secondaryValue)
            : undefined,
        tooltipContent: tooltipContent
            ? ({ record, primaryValue, secondaryValue }) => resolveWidgetProperty(tooltipContent, widgetDefinition.widgetObject, record, primaryValue, secondaryValue)
            : undefined,
    };
};
const getBarChartWidgetConfig = (widgetId, widgetDefinition) => {
    const barChartProperties = widgetDefinition.properties;
    return {
        title: resolveWidgetProperty(barChartProperties.title, widgetDefinition.widgetObject || widgetId),
        type: 'bar-chart',
        data: {
            areAxesStacked: resolveWidgetProperty(barChartProperties.areAxesStacked, widgetDefinition.widgetObject || widgetId),
            areAxesSwapped: resolveWidgetProperty(barChartProperties.areAxesSwapped, widgetDefinition.widgetObject || widgetId),
            data: resolveWidgetProperty(barChartProperties.content, widgetDefinition.widgetObject || widgetId),
            isHistogram: resolveWidgetProperty(barChartProperties.isHistogram, widgetDefinition.widgetObject || widgetId),
            primaryAxis: resolveWidgetProperty(barChartProperties.primaryAxis, widgetDefinition.widgetObject || widgetId),
            primaryAxisLabel: resolveWidgetProperty(barChartProperties.primaryAxisLabel, widgetDefinition.widgetObject || widgetId),
            secondaryAxisLabel: resolveWidgetProperty(barChartProperties.secondaryAxisLabel, widgetDefinition.widgetObject || widgetId),
            secondaryAxes: barChartProperties.secondaryAxes.map(createAxesConfig(widgetDefinition)),
        },
    };
};
const getLineChartWidgetConfig = (widgetId, widgetDefinition) => {
    const lineChartProperties = widgetDefinition.properties;
    return {
        title: resolveWidgetProperty(lineChartProperties.title, widgetDefinition.widgetObject || widgetId),
        type: 'line-chart',
        data: {
            data: resolveWidgetProperty(lineChartProperties.content, widgetDefinition.widgetObject || widgetId),
            primaryAxis: resolveWidgetProperty(lineChartProperties.primaryAxis, widgetDefinition.widgetObject || widgetId),
            secondaryAxes: lineChartProperties.secondaryAxes.map(createAxesConfig(widgetDefinition)),
            primaryAxisLabel: resolveWidgetProperty(lineChartProperties.primaryAxisLabel, widgetDefinition.widgetObject || widgetId),
            secondaryAxisLabel: resolveWidgetProperty(lineChartProperties.secondaryAxisLabel, widgetDefinition.widgetObject || widgetId),
        },
    };
};
const getIndicatorTileWidgetConfig = (widgetId, widgetDefinition, 
// If set to true, all click listeners are disabled
isEditing = false) => {
    const indicatorTileProperties = widgetDefinition.properties;
    const icon = resolveWidgetProperty(indicatorTileProperties.icon, widgetDefinition.widgetObject || widgetId);
    return {
        title: undefined,
        type: 'simple-indicator',
        data: {
            description: resolveWidgetProperty(indicatorTileProperties.subtitle, widgetDefinition.widgetObject || widgetId),
            mainIndicator: {
                title: String(resolveWidgetProperty(indicatorTileProperties.value, widgetDefinition.widgetObject || widgetId)),
                subtitle: resolveWidgetProperty(indicatorTileProperties.title, widgetDefinition.widgetObject || widgetId),
            },
            icon: icon
                ? {
                    url: resolveDetailedIcon(icon),
                    color: resolveWidgetProperty(indicatorTileProperties.color, widgetDefinition.widgetObject || widgetId),
                }
                : undefined,
            onClick: isEditing
                ? noop
                : () => resolveWidgetProperty(indicatorTileProperties.onClick, widgetDefinition.widgetObject || widgetId),
            type: 'text',
        },
    };
};
const getGaugeWidgetConfig = (widgetId, widgetDefinition) => {
    const gaugeProperties = widgetDefinition.properties;
    return {
        title: resolveWidgetProperty(gaugeProperties.title, widgetDefinition.widgetObject || widgetId),
        type: 'gauge',
        data: {
            data: resolveWidgetProperty(gaugeProperties.content, widgetDefinition.widgetObject || widgetId),
            color: resolveWidgetProperty(gaugeProperties.color, widgetDefinition.widgetObject || widgetId),
            title: resolveWidgetProperty(gaugeProperties.title, widgetDefinition.widgetObject || widgetId),
            scale: resolveWidgetProperty(gaugeProperties.scale, widgetDefinition.widgetObject || widgetId),
            valueUnit: resolveWidgetProperty(gaugeProperties.valueUnit, widgetDefinition.widgetObject || widgetId),
            evolutionUnit: resolveWidgetProperty(gaugeProperties.evolutionUnit, widgetDefinition.widgetObject || widgetId),
        },
    };
};
const getWidgetDataOptions = (widgetDefinition, widgetId, tableProperties) => {
    const dropdownMenu = resolveWidgetProperty(tableProperties.dataDropdownMenu, widgetDefinition.widgetObject || widgetId) || {};
    return Object.keys(dropdownMenu).reduce((prevValue, key) => {
        prevValue[key] = Object.keys(dropdownMenu[key]).reduce((internalPrevValue, internalKey) => {
            internalPrevValue[internalKey] = dropdownMenu[key][internalKey].title;
            return internalPrevValue;
        }, {});
        return prevValue;
    }, {});
};
const getRowDefinition = (widgetDefinition, widgetId, tableProperties, 
// If set to true, all click listeners are disabled
isEditing = false) => {
    const rowDefinition = resolveWidgetProperty(tableProperties.rowDefinition, widgetDefinition.widgetObject || widgetId) || {};
    return Object.keys(rowDefinition).reduce((prevValue, k) => {
        prevValue[k] = {
            ...rowDefinition[k],
            onClick: isEditing
                ? noop
                : (i) => resolveWidgetProperty(rowDefinition[k]?.onClick, widgetDefinition.widgetObject || widgetId, String(i)),
            displayOptions: {
                colorVariant: (i) => resolveWidgetProperty(rowDefinition[k].displayOptions?.colorVariant, widgetDefinition.widgetObject || widgetId, String(i)),
                tooltipText: (i) => resolveWidgetProperty(rowDefinition[k].displayOptions?.tooltipText, widgetDefinition.widgetObject || widgetId, String(i)),
            },
            renderedAs: rowDefinition[k].renderedAs,
            valueFormat: rowDefinition[k].valueFormat,
        };
        return prevValue;
    }, {});
};
const getTableWidgetConfig = (widgetId, dashboardId, widgetDefinition, 
// If set to true, all click listeners are disabled
isEditing = false) => {
    const tableProperties = widgetDefinition.properties;
    const canSelect = isEditing
        ? false
        : !!resolveWidgetProperty(tableProperties.canSelect, widgetDefinition.widgetObject || widgetId);
    const mode = widgetDefinition.options?.mode ||
        resolveWidgetProperty(tableProperties.displayMode, widgetDefinition.widgetObject || widgetId) ||
        'table';
    const canChangeViewMode = resolveWidgetProperty(tableProperties.canSwitchViewMode, widgetDefinition.widgetObject || widgetId);
    const data = (resolveWidgetProperty(tableProperties.content, widgetDefinition.widgetObject || widgetId) || []).map((r) => ({
        ...r,
        i: r._id,
        image: r.image ? `data:image;base64,${r.image}` : undefined,
    }));
    const onDataOptionsChanged = (newDataOptions) => {
        updateWidgetOptions(dashboardId, widgetId, true, {
            ...widgetDefinition.options,
            dataOptions: { ...widgetDefinition.options?.dataOptions, ...newDataOptions },
        });
    };
    const onRepresentationModeChanged = (updateMode) => {
        updateWidgetOptions(dashboardId, widgetId, false, {
            ...widgetDefinition.options,
            mode: updateMode,
        });
    };
    const onSelectionChanged = (selectedItems) => {
        updateWidgetOptions(dashboardId, widgetId, false, { ...widgetDefinition.options, selectedItems });
    };
    return {
        type: 'table',
        data: {
            data,
            canSelect,
            mode,
            canChangeViewMode,
            selectedDataOptions: widgetDefinition.options?.dataOptions,
            rowDefinition: getRowDefinition(widgetDefinition, widgetId, tableProperties, isEditing),
            dataOptions: getWidgetDataOptions(widgetDefinition, widgetId, tableProperties),
            onDataOptionsChanged,
            onSelectionChanged,
            onRepresentationModeChanged,
            loadMoreRows: memoize(async (loadMoreParams = {
                startIndex: 0,
                stopIndex: 20,
                pageCount: 1,
                pageSize: 20,
                first: 20,
            }) => {
                if (loadMoreParams.after && widgetDefinition._id !== PREVIEW_WIDGET_ID) {
                    return fetchWidgetPage({
                        widgetId: widgetDefinition._id,
                        queryArgs: loadMoreParams,
                    });
                }
                return [];
            }),
            pageSize: 20,
        },
    };
};
export const getVisualProcessWidgetConfig = (widgetId, widgetDefinition, 
// If set to true, all click listeners are disabled
isEditing = false) => {
    const visualProcessProperties = widgetDefinition.properties;
    const visualProcessContent = resolveWidgetProperty(visualProcessProperties.content, widgetDefinition.widgetObject || widgetId);
    if (!visualProcessContent) {
        throw new Error('No content found for visual process');
    }
    return {
        type: 'visual-process',
        data: {
            value: visualProcessContent,
            resolveImageUrl,
            onLinkClick: isEditing
                ? noop
                : (link) => executeVisualProcessLink(DASHBOARD_SCREEN_ID, link),
        },
    };
};
const getCallToActionDefinition = (widgetId, widgetDefinition, callToActions) => {
    return Object.keys(callToActions).reduce((prevValue, key) => {
        const action = callToActions[key];
        prevValue[key] = {
            title: resolveWidgetProperty(action.title, widgetDefinition.widgetObject || widgetId),
            isDisabled: !!resolveWidgetProperty(action.isDisabled, widgetDefinition.widgetObject || widgetId),
            isHidden: !!resolveWidgetProperty(action.isHidden, widgetDefinition.widgetObject || widgetId),
            onClick: () => resolveWidgetProperty(action.onClick, widgetDefinition.widgetObject || widgetId),
        };
        return prevValue;
    }, {});
};
const addSettingsPageToProps = (widgetId, widgetDefinition, props) => {
    const isGenericWidget = widgetDefinition.artifactName.startsWith(GENERIC_PLATFORM_WIDGET_PREFIX);
    if (widgetDefinition.properties.settingsPage && props.dropdown) {
        props.dropdown.items = [
            {
                i: 'settings',
                label: localize('@sage/xtrem-ui/dashboard-widget-settings', 'Settings'),
                icon: 'settings',
                onClick: async () => {
                    try {
                        if (widgetDefinition.properties.settingsPage) {
                            const store = xtremRedux.getStore();
                            const state = store.getState();
                            const settings = state.dashboard.dashboardEditor.currentDashboardDefinition.children.find(c => c._id === widgetId)?.settings || {};
                            const updatedSettings = await widgetDefinition.widgetObject.$.dialog.page(widgetDefinition.properties.settingsPage, settings, { rightAligned: true });
                            const thunkDispatch = store.dispatch;
                            await thunkDispatch(xtremRedux.actions.updateWidgetSettings(widgetId, updatedSettings));
                        }
                    }
                    catch {
                        // Do nothing.
                    }
                },
            },
            ...(props.dropdown.items || []),
        ];
    }
    if (isGenericWidget && props.dropdown) {
        props.dropdown.items = [
            {
                i: 'edit',
                label: localize('@sage/xtrem-ui/dashboard-editor-widget-edit', 'Edit'),
                icon: 'settings',
                onClick() {
                    const store = xtremRedux.getStore();
                    const state = store.getState();
                    const settings = state.dashboard.dashboardEditor.currentDashboardDefinition.children.find(c => c._id === widgetId)?.settings;
                    if (settings) {
                        const thunkDispatch = store.dispatch;
                        thunkDispatch(xtremRedux.actions.openWidgetEditorDialog(widgetId, settings));
                    }
                },
            },
            ...(props.dropdown.items || []),
        ];
    }
};
const addHeaderActionsToProps = (widgetId, widgetDefinition, props) => {
    if (widgetDefinition.properties.headerActions && props.dropdown) {
        props.dropdown.items = [
            ...widgetDefinition.properties.headerActions.map(ha => ({
                i: `${ha.title}-${ha.icon}`,
                label: ha.title,
                icon: ha.icon,
                onClick: () => resolveWidgetProperty(ha.onClick, widgetDefinition.widgetObject || widgetId),
            })),
            { i: 'header-actions-separator', isDivider: true },
            ...(props.dropdown.items || []),
        ];
    }
};
const getDefaultProps = (widgets, p, isEditing, onCloseCallback) => {
    const dropDownItems = [
        {
            i: 'refresh',
            label: localize('@sage/xtrem-ui/dashboard-widget-refresh', 'Refresh'),
            icon: 'refresh',
            onClick: () => refreshWidget(p._id),
        },
    ];
    if (isEditing) {
        dropDownItems.push({ i: 'close-separator', isDivider: true });
        dropDownItems.push({
            i: 'close',
            label: localize('@sage/xtrem-ui/dashboard-widget-close', 'Close'),
            icon: 'cross',
            onClick: onCloseCallback ? () => onCloseCallback(p._id) : undefined,
        });
    }
    const widgetDefinition = widgets?.[p._id];
    return {
        i: String(p._id),
        type: 'basic',
        x: p.x,
        y: p.y,
        w: p.w,
        h: p.h,
        title: resolveWidgetProperty(widgetDefinition.properties?.title, widgetDefinition.widgetObject || p._id),
        dropdown: {
            items: dropDownItems,
        },
    };
};
// TO RENDER DASHBOARD CALL THIS
export const getWidgetsFromDashboardItems = (dashboardId, children, widgets, 
// If set to true, all click listeners are disabled
isEditing = false, onCloseCallback) => children.map(p => {
    const widgetId = String(p._id);
    const props = getDefaultProps(widgets, p, isEditing, onCloseCallback);
    if (!widgets?.[p._id] || !widgets?.[p._id]?.data) {
        // Set loading state of the widget
        props.type = 'preview';
        props.title = undefined;
        props.dropdown = undefined;
        props.data = {};
        return props;
    }
    const widgetDefinition = widgets[p._id];
    if (isEditing) {
        // The settings are only available in edit mode
        addSettingsPageToProps(widgetId, widgetDefinition, props);
    }
    addHeaderActionsToProps(widgetId, widgetDefinition, props);
    if (widgetDefinition.properties.callToActions) {
        props.callToActions = getCallToActionDefinition(widgetId, widgetDefinition, widgetDefinition.properties.callToActions);
    }
    switch (widgetDefinition.widgetType) {
        case WidgetType.barChart:
            return {
                ...props,
                ...getBarChartWidgetConfig(widgetId, widgetDefinition),
            };
        case WidgetType.lineChart:
            return {
                ...props,
                ...getLineChartWidgetConfig(widgetId, widgetDefinition),
            };
        case WidgetType.indicatorTile:
            return {
                ...props,
                ...getIndicatorTileWidgetConfig(widgetId, widgetDefinition, isEditing),
            };
        case WidgetType.gauge:
            return {
                ...props,
                ...getGaugeWidgetConfig(widgetId, widgetDefinition),
            };
        case WidgetType.staticContent:
            return {
                ...props,
                ...getStaticContentWidgetConfig(widgetId, widgetDefinition),
            };
        case WidgetType.table:
            return {
                ...props,
                ...getTableWidgetConfig(widgetId, dashboardId, widgetDefinition, isEditing),
            };
        case WidgetType.visualProcess:
            return {
                ...props,
                ...getVisualProcessWidgetConfig(widgetId, widgetDefinition, isEditing),
            };
        default:
            props.data = {
                htmlContent: `<div>Unknown widget: ${widgetDefinition.widgetType}</div>`,
            };
    }
    return props;
});
export const fetchWidgetData = async ({ forceRefetch = false, locale, version, widgetDefinition, queryArgs, }) => {
    const query = queryArgs
        ? resolveWidgetProperty(widgetDefinition.properties.getQuery, widgetDefinition.widgetObject, queryArgs)
        : resolveWidgetProperty(widgetDefinition.properties.getQuery, widgetDefinition.widgetObject);
    // The query is hashed and added to the key
    const settingsHash = MD5(JSON.stringify(query)).toString();
    const requestArgs = {
        query,
        cacheSettings: version && locale
            ? {
                key: `${widgetDefinition.artifactName}-${settingsHash}-widget-data`,
                locale,
                version,
            }
            : undefined,
        forceRefetch,
    };
    let queryResult = await executeGraphqlQuery({ ...requestArgs });
    const cacheLifespan = resolveWidgetProperty(widgetDefinition.properties.cacheLifespan, widgetDefinition.widgetObject || widgetDefinition._id);
    // Check whether the result is coming from the cache, validate its timestamp
    if (queryResult.cachedAt && queryResult.cachedAt < Date.now() - cacheLifespan) {
        // If it is cached and it was cached outside the cache time frame, force refetch the cache for the widget.
        queryResult = await executeGraphqlQuery({ ...requestArgs, forceRefetch: true });
    }
    return queryResult.data;
};
export const resolveWidgetProperty = (property, 
/** The widget ID or the AbstractWidget object */
widget, ...args) => {
    if (typeof property === 'function') {
        if (typeof widget === 'string') {
            const widgetDefinition = xtremRedux.getStore().getState().dashboard.widgets[widget];
            if (widgetDefinition) {
                return property.apply(widgetDefinition.widgetObject, args);
            }
            return null;
        }
        return property.apply(widget, args);
    }
    return property;
};
//# sourceMappingURL=dashboard-service.js.map