import { deepMerge, flat } from '@sage/xtrem-shared';
import { EnumType } from 'json-to-graphql-query';
import { camelCase, get, isEmpty, isPlainObject, isUndefined, mapValues, merge, omit, reduceRight, remove, set, uniq, without, } from 'lodash';
import { getReferenceValueFieldPath } from '../../component/field/reference/reference-utils';
import { FieldKey } from '../../component/types';
import { GraphQLTypes } from '../../types';
import { filterFields, getNestedFieldElementId } from '../../utils/abstract-fields-utils';
import { deepFindPropertyValues } from '../../utils/common-util';
import { INPUT_NODE_POSTFIX, NEW_PAGE } from '../../utils/constants';
import { convertDeepBindToPathNotNull, getNestedFieldsFromProperties, getTopLevelBindFromNestedBind, } from '../../utils/nested-field-utils';
import { findDeepPropertyDetails } from '../../utils/node-utils';
import { parseScreenValues, pascalCase, schemaTypeNameFromNodeName } from '../../utils/transformers';
import { AllAndOnly, reduceDict } from '../../utils/type-utils';
import { CollectionValue } from '../collection-data-service';
import { mergeGraphQLFilters } from '../filter-service';
import { wrapQuery } from '../graphql-utils';
import { getScreenElement } from '../screen-base-definition';
import { formatInputProperty } from '../value-serializer-service';
import { getFieldProperties } from './field-query-builder';
export const RESTRICTED_COLUMN_PREFIX = '__restricted_';
export const RESTRICTED_COLUMN_ALL_PREFIX = '__all_';
export const quantifiers = AllAndOnly()(['_atLeast', '_atMost', '_every', '_none']);
export * from './field-query-builder';
export const filterReservedWords = AllAndOnly()([
    '_and',
    '_or',
    '_not',
    '_eq',
    '_ne',
    '_in',
    '_nin',
    '_contains',
    '_containsRange',
    '_containedBy',
    'start',
    'end',
    '_gt',
    '_gte',
    '_lt',
    '_lte',
    '_atLeast',
    '_atMost',
    '_every',
    '_none',
    '_regex',
    '_options',
    '_mod',
]);
const filterReducer = reduceDict();
export const getPlatformStringLiteralsSchemaFragment = () => ({
    __args: {
        filter: {
            packageOrPage: '@sage/xtrem-ui,@sage/xtrem-date-time,@sage/xtrem-ui-components,@sage/xtrem-document-editor',
        },
    },
    key: true,
    content: true,
});
export const addTotalCountTransform = (filter, initial) => filterReducer(filter, initial, (_value, acc, key, path, source) => {
    if (quantifiers.includes(key)) {
        const parentKey = path[path.length - 2];
        const parentPath = path.slice(0, -1);
        const parentValue = get(source, parentPath);
        const fieldName = pascalCase(parentKey);
        acc[`${RESTRICTED_COLUMN_ALL_PREFIX}${fieldName}`] = {
            __aliasFor: parentKey,
            query: {
                totalCount: true,
            },
        };
        acc[`${RESTRICTED_COLUMN_PREFIX}${fieldName}`] = {
            __aliasFor: parentKey,
            query: {
                totalCount: true,
                __args: { filter: JSON.stringify(omit(parentValue, [key])) },
            },
        };
        delete acc[parentKey];
    }
    return acc;
});
export const addReferenceIdsAssign = (filter, node, nodeTypes) => filterReducer(filter, {}, (_value, acc, _key, path) => {
    const isReserved = (k) => filterReservedWords.includes(k);
    const cleanPath = remove([...path], k => typeof k !== 'number' && !isReserved(k));
    const targetType = findDeepPropertyDetails(schemaTypeNameFromNodeName(node, true), cleanPath.join('.'), nodeTypes, true);
    const isNodeProperty = cleanPath.length === 1;
    const isNodeReference = cleanPath.length >= 2;
    const isNode = (isNodeProperty || isNodeReference) && targetType?.type !== GraphQLTypes.Json;
    if (isNode) {
        return merge(acc, reduceRight(cleanPath, (acc2, k) => (isEmpty(acc2) ? { [k]: true } : { [k]: { ...acc2, _id: true } }), {}));
    }
    return acc;
});
export const parseFilterEnumFields = (source) => {
    let result = source;
    if (isPlainObject(source)) {
        result = mapValues(source, (x) => Array.isArray(x) ? x.map(parseFilterEnumFields) : parseFilterEnumFields(x));
        if (result._isEnum) {
            // TODO Should we throw an exception if an operator other than _eq is used or if no _eq is provided?
            result._eq = new EnumType(result._eq);
            delete result._isEnum;
        }
    }
    return result;
};
export const convertFilterDecoratorToGraphQLFilter = (screenDefinition, filter, recordContext) => {
    if (filter) {
        if (typeof filter === 'function') {
            const screenBase = getScreenElement(screenDefinition);
            return parseFilterEnumFields(filter.apply(screenBase, [recordContext]));
        }
        return parseFilterEnumFields(filter);
    }
    return undefined;
};
const buildNodeQueryFromProperties = (properties, startFrom = {}, isRequestingDefaults = false) => {
    let node = { ...startFrom };
    properties.forEach(prop => {
        if (typeof prop === 'string') {
            if (isRequestingDefaults && prop === '_sortValue') {
                return;
            }
            node[prop] = true;
        }
        if (typeof prop === 'object') {
            const merged = merge(prop, node);
            node = isRequestingDefaults ? sortValueCleaner(merged) : merged;
        }
    });
    return node;
};
const sortValueCleaner = (input) => {
    if (!input || typeof input !== 'object') {
        return input;
    }
    if (Array.isArray(input)) {
        return input
            .filter(e => e !== '_sortValue')
            .map((e) => {
            return sortValueCleaner(e);
        });
    }
    return Object.keys(input).reduce((r, k) => {
        if (k === '_sortValue') {
            return r;
        }
        r[k] = sortValueCleaner(input[k]);
        return r;
    }, {});
};
const buildCollectionNodeWithQuery = (options, wrapWithEdges = true, node, nodeTypes) => {
    if (!wrapWithEdges) {
        let singleNode = {};
        if (options.properties) {
            options.properties.forEach(prop => {
                if (typeof prop === 'string') {
                    singleNode[prop] = true;
                }
                if (typeof prop === 'object') {
                    singleNode = deepMerge(prop, singleNode);
                }
            });
        }
        return singleNode;
    }
    const collection = {
        query: {
            edges: {
                node: {
                    _id: true,
                },
                cursor: true,
            },
            // If we would have an empty object, the library puts empty parentheses and breaks the query
            __args: options.queryArguments && Object.keys(options.queryArguments).length > 0
                ? options.queryArguments
                : undefined,
        },
    };
    if (!options.noPageInfo) {
        collection.query.pageInfo = {
            startCursor: true,
            endCursor: true,
            hasPreviousPage: true,
            hasNextPage: true,
        };
    }
    if (options.properties) {
        collection.query.edges.node = buildNodeQueryFromProperties(options.properties, collection.query.edges.node);
    }
    if (options.queryArguments?.filter) {
        const parsedFilter = JSON.parse(options.queryArguments.filter);
        collection.query.edges.node = merge(collection.query.edges.node, addReferenceIdsAssign(parsedFilter, node, nodeTypes));
        collection.query.edges.node = addTotalCountTransform(parsedFilter, collection.query.edges.node);
    }
    return collection;
};
export const queryBuilder = (node, options) => {
    const query = buildCollectionNodeWithQuery(options);
    if (options.alias) {
        query.__aliasFor = camelCase(String(node).split('/').pop());
    }
    return query;
};
export const getUsedEnums = (pageMetadata, knownEnumDefinitions) => {
    const enums = deepFindPropertyValues('optionType', pageMetadata.uiComponentProperties);
    return uniq(without(enums, ...knownEnumDefinitions)).filter(v => !!v);
};
export const buildEnumQuery = (enumName) => {
    const typeName = schemaTypeNameFromNodeName(enumName);
    return {
        [typeName]: {
            __aliasFor: '__type',
            __args: { name: typeName },
            enumValues: {
                name: true,
            },
        },
    };
};
export const getUsedNodes = (pageMetadata, knownTypeDefinitions, rootNode) => {
    const nodes = deepFindPropertyValues('node', pageMetadata.uiComponentProperties);
    if (rootNode) {
        nodes.push(rootNode);
    }
    return uniq(without(nodes, ...knownTypeDefinitions)).filter(v => !!v);
};
export const buildTypeQuery = (nodeName, level = 3) => {
    const typeName = schemaTypeNameFromNodeName(nodeName);
    return {
        [typeName]: {
            __aliasFor: '__type',
            __args: { name: typeName },
            fields: {
                name: true,
                type: {
                    ofType: {
                        name: true,
                    },
                    name: true,
                    enumValues: {
                        name: true,
                    },
                    kind: true,
                    ...(level === 1
                        ? {}
                        : {
                            fields: {
                                name: true,
                                type: {
                                    ofType: {
                                        name: true,
                                    },
                                    name: true,
                                    enumValues: {
                                        name: true,
                                    },
                                    kind: true,
                                    ...(level === 2
                                        ? {}
                                        : {
                                            fields: {
                                                name: true,
                                                type: {
                                                    ofType: {
                                                        name: true,
                                                    },
                                                    name: true,
                                                    enumValues: {
                                                        name: true,
                                                    },
                                                    kind: true,
                                                },
                                            },
                                        }),
                                },
                            },
                        }),
                },
            },
        },
        [`${typeName}${INPUT_NODE_POSTFIX}`]: {
            __aliasFor: '__type',
            __args: { name: `${typeName}${INPUT_NODE_POSTFIX}` },
            inputFields: {
                name: true,
                type: {
                    ofType: {
                        name: true,
                    },
                    name: true,
                    enumValues: {
                        name: true,
                    },
                    kind: true,
                    ...(level === 1
                        ? {}
                        : {
                            inputFields: {
                                name: true,
                                type: {
                                    ofType: {
                                        name: true,
                                    },
                                    name: true,
                                    enumValues: {
                                        name: true,
                                    },
                                    kind: true,
                                    ...(level === 2
                                        ? {}
                                        : {
                                            inputFields: {
                                                name: true,
                                                type: {
                                                    ofType: {
                                                        name: true,
                                                    },
                                                    name: true,
                                                    enumValues: {
                                                        name: true,
                                                    },
                                                    kind: true,
                                                },
                                            },
                                        }),
                                },
                            },
                        }),
                },
            },
        },
    };
};
export const getRequestableFieldIds = (fieldControlObjects, screenDefinition) => Object.keys(fieldControlObjects).filter(f => {
    return (!screenDefinition.metadata.uiComponentProperties?.[f]?.isTransient &&
        !screenDefinition.metadata.uiComponentProperties?.[f]?.isTransientInput);
});
export const buildEnumQueries = (pageMetadata, knownTypes) => getUsedEnums(pageMetadata, knownTypes)
    .map(buildEnumQuery)
    .reduce((optionsQuery, nextQuery) => ({ ...optionsQuery, ...nextQuery }), {});
export const buildTypeQueries = (pageMetadata, knownTypes, rootNode) => getUsedNodes(pageMetadata, knownTypes, rootNode)
    .map(buildTypeQuery)
    .reduce((optionsQuery, nextQuery) => ({ ...optionsQuery, ...nextQuery }), {});
export const buildTableTotalCountQuery = ({ filter }) => {
    return {
        query: {
            __args: { filter },
            totalCount: true,
        },
    };
};
export const buildFieldQuery = (rootNode, fieldControlObject, screenDefinition, fieldProperties, elementId, recordId, nodeTypes) => {
    const queryProps = getFieldProperties(elementId, fieldControlObject.componentType, fieldProperties, screenDefinition, fieldProperties.bind, nodeTypes, rootNode);
    const filter = JSON.stringify({ _id: { _eq: recordId } });
    const options = { queryArguments: { filter }, properties: [queryProps] };
    return queryBuilder(rootNode, options);
};
export const buildRootNodeQuery = (rootNode, fieldControlObjects, screenDefinition, _id, alias, nodeTypes, isDuplicate = false) => {
    const fieldIds = getRequestableFieldIds(fieldControlObjects, screenDefinition);
    const queryProps = getFieldsProperties(fieldIds, fieldControlObjects, screenDefinition, nodeTypes, rootNode);
    return wrapQuery(rootNode, {
        [isDuplicate ? 'getDuplicate' : 'read']: {
            __args: { _id: String(_id) },
            ...buildNodeQueryFromProperties([...queryProps, '_etag', '_id']),
        },
    }, alias);
};
const getFieldsProperties = (fieldIds, fieldControlObjects, screenDefinition, nodeTypes, node) => {
    return fieldIds.reduce((props, fieldId) => {
        const fieldType = fieldControlObjects[fieldId].componentType;
        const fieldProperties = screenDefinition.metadata.uiComponentProperties[fieldId];
        return [
            ...props,
            getFieldProperties(fieldId, fieldType, fieldProperties, screenDefinition, fieldProperties.bind, nodeTypes, node),
        ];
    }, []);
};
export const getFilterAsString = (filters, screenDefinition, recordContext) => {
    const graphQLFilters = mergeFilters(filters, screenDefinition, recordContext);
    return !isEmpty(graphQLFilters) ? JSON.stringify(graphQLFilters) : undefined;
};
export const mergeFilters = (filters, screenDefinition, recordContext) => {
    const graphQLFilters = filters.reduce((currentGraphQLFilters, filter) => {
        const graphQLFilter = convertFilterDecoratorToGraphQLFilter(screenDefinition, filter, recordContext);
        if (graphQLFilter) {
            currentGraphQLFilters.push(graphQLFilter);
        }
        return currentGraphQLFilters;
    }, []);
    if (!isEmpty(graphQLFilters)) {
        return graphQLFilters.length === 1 ? graphQLFilters[0] : mergeGraphQLFilters(graphQLFilters);
    }
    return {};
};
export const applyDeepNestedKey = (elementId, fieldDetails, addAlias = false, bind, parentNode, nodeTypes) => {
    const pathComponents = convertDeepBindToPathNotNull(bind || elementId).split('.');
    const topPropertyType = findDeepPropertyDetails(schemaTypeNameFromNodeName(parentNode), convertDeepBindToPathNotNull(bind || elementId).split('.')[0], nodeTypes);
    if (topPropertyType?.type === GraphQLTypes.Json) {
        /**
         * We cannot deep query JSON columns, we need to fetch the JSON column, then
         * parse it and then finally return the property value we really need.
         *  */
        return {
            [pathComponents[0]]: true,
        };
    }
    const object = [...pathComponents].reverse().reduce((prevFilter, currentKey) => {
        return { [currentKey]: prevFilter };
    }, fieldDetails);
    if (!addAlias) {
        return object;
    }
    // On primitive types we just return an alias object
    if (typeof object[pathComponents[0]] !== 'object') {
        if (elementId === pathComponents[0]) {
            return object;
        }
        return {
            [elementId]: { __aliasFor: pathComponents[0] },
        };
    }
    // Swap the alias with the object key
    const currentObjectKey = Object.keys(object)[0];
    if (currentObjectKey !== elementId) {
        object[pathComponents[0]].__aliasFor = pathComponents[0];
        object[elementId] = object[currentObjectKey];
        delete object[currentObjectKey];
    }
    return object;
};
/**
 * Inspects the filter object, and prepare it for server side consumption.
 * It remaps id field names to match current server inconsistencies and creates nested sorting conditions for reference
 * fields.
 * @param orderBy
 */
export const createOrderByQueryArgument = (orderBy) => {
    const remappedOrderBy = {};
    Object.keys(orderBy).forEach(fieldKey => {
        const filterValue = orderBy[fieldKey];
        const keyToSet = fieldKey.split('__').join('.'); // Split by the nested bind separator
        set(remappedOrderBy, keyToSet, filterValue);
    });
    return JSON.stringify(remappedOrderBy);
};
export const getAggregationValues = (aggregations) => Object.entries(flat(aggregations)).reduce((acc, [key, value]) => {
    set(acc, `${key}.${value}`, true);
    return acc;
}, {});
export const nestedFieldsToQueryProperties = ({ screenDefinition, nestedFields, node, nodeTypes, }) => nestedFields
    ?.filter(nestedField => !nestedField.properties.isTransient && !nestedField.properties.isTransientInput)
    .map(nestedField => {
    return getFieldProperties(getTopLevelBindFromNestedBind(nestedField.properties.bind), nestedField.type, nestedField.properties, screenDefinition, nestedField.properties.bind, nodeTypes, node);
});
export const getNestedFieldsQuery = ({ screenDefinition, nestedFields, queryArguments, alias, wrapNodeWithEdges = true, node, nodeTypes, }) => {
    const properties = nestedFieldsToQueryProperties({ screenDefinition, nestedFields, node, nodeTypes });
    return buildCollectionNodeWithQuery({ alias, properties, queryArguments }, wrapNodeWithEdges, node, nodeTypes);
};
// This function avoids adding undefined properties to the GraphQL queries that would result in errors
export const parseQueryArguments = (inputArguments) => {
    const outputArguments = {};
    if (inputArguments.after) {
        outputArguments.after = inputArguments.after;
    }
    if (inputArguments.before) {
        outputArguments.before = inputArguments.before;
    }
    if (inputArguments.filter) {
        outputArguments.filter = inputArguments.filter;
    }
    if (inputArguments.first) {
        outputArguments.first = inputArguments.first;
    }
    if (inputArguments.last) {
        outputArguments.last = inputArguments.last;
    }
    if (inputArguments.orderBy) {
        outputArguments.orderBy = inputArguments.orderBy;
    }
    return outputArguments;
};
const getDirtyNestedValues = ({ screenDefinition, elementId, recordId, level = 0, }) => {
    if (!recordId) {
        return {};
    }
    const value = screenDefinition.values[elementId];
    if (!value || value instanceof CollectionValue === false) {
        return {};
    }
    const actualLevel = level === 0 ? undefined : level;
    const row = screenDefinition.values[elementId].getRawRecord({
        id: recordId,
        cleanMetadata: false,
        level: actualLevel,
    });
    if (!row) {
        return {};
    }
    return Array.from(row.__dirtyColumns || []).reduce((acc, dirtyColumn) => {
        acc[dirtyColumn] = row[dirtyColumn];
        return acc;
    }, {});
};
export const buildRowDefaultQuery = ({ screenDefinition, elementId, isNewRow = true, alias, recordId, data, nodeTypes, }) => {
    if (!isNewRow && recordId === undefined) {
        throw new Error('A row id is needed in order to fetch default values for an existing row.');
    }
    const fieldProperties = screenDefinition.metadata.uiComponentProperties[elementId];
    const inputData = data || getDirtyNestedValues({ screenDefinition, elementId, recordId });
    const queryData = formatInputProperty({
        propertyType: schemaTypeNameFromNodeName(fieldProperties.node || '', true),
        propertyValue: inputData,
        nodeTypes,
    });
    let additionalData;
    if (!queryData) {
        additionalData = { [fieldProperties.bind ? String(fieldProperties.bind) : elementId]: [{}] };
    }
    else {
        additionalData = { [fieldProperties.bind ? String(fieldProperties.bind) : elementId]: [queryData] };
    }
    return buildDefaultsQuery({
        screenDefinition,
        alias,
        requestedFieldIds: [elementId],
        additionalData,
        sendNonDirtyFields: true,
    });
};
export const buildNestedDefaultsQuery = ({ screenDefinition, elementId, isNewRow = true, alias, recordId, level = 0, data, nodeTypes, }) => {
    if (!isNewRow && recordId === undefined) {
        throw new Error('A row id is needed in order to fetch default values for an existing row.');
    }
    const fieldProperties = screenDefinition.metadata.uiComponentProperties[elementId];
    const isNestedGrid = Object.prototype.hasOwnProperty.call(fieldProperties, 'levels');
    const inputData = data || getDirtyNestedValues({ screenDefinition, elementId, recordId, level });
    const columnDefinitions = isNestedGrid
        ? fieldProperties.levels[level].columns
        : getNestedFieldsFromProperties(fieldProperties);
    let columns = [];
    if (!isUndefined(recordId) || !isNewRow || data) {
        const dirtyKeys = Object.keys(inputData).map(key => key.split('__')[0]);
        columns = columnDefinitions.filter(column => dirtyKeys.indexOf(getNestedFieldElementId(column).split('__')[0]) === -1);
    }
    else {
        columns = columnDefinitions;
    }
    columns = columns.filter(f => f.properties.bind !== '_sortValue');
    if (isEmpty(columns)) {
        return {};
    }
    const queryProps = [
        {
            ...getNestedFieldsQuery({
                screenDefinition,
                nestedFields: columns,
                wrapNodeWithEdges: false,
            }),
        },
    ];
    const fieldsToRequest = buildNodeQueryFromProperties(queryProps);
    // We don't request the fake ID that the defaults service returns
    delete fieldsToRequest._id;
    const node = String(isNestedGrid
        ? fieldProperties.levels[level].node
        : fieldProperties.node);
    const queryData = formatInputProperty({
        propertyType: schemaTypeNameFromNodeName(node, true),
        propertyValue: inputData,
        nodeTypes,
    });
    return wrapQuery(node, {
        getDefaults: {
            ...((!isNewRow || data) && {
                __args: {
                    data: queryData,
                },
            }),
            ...fieldsToRequest,
        },
    }, alias);
};
/**
 * Builds a query to request default values from the server.
 * If a list of fields is not provided as the last argument, the function checks the screen definition and requests values for the non-dirty fields.
 */
export const buildDefaultsQuery = ({ screenDefinition, alias, requestedFieldIds, clean = false, additionalData, sendNonDirtyFields = false, node, nodeTypes, }) => {
    const screenId = screenDefinition.metadata.screenId;
    const controlObjects = filterFields(screenDefinition.metadata.controlObjects);
    const isNewPage = Boolean(screenDefinition.queryParameters?._id === NEW_PAGE);
    const isMainList = screenDefinition.selectedRecordId === null &&
        screenDefinition.isMainPage === true &&
        !isNewPage;
    const hasFetchedDefaults = Object.keys(controlObjects).some(key => {
        const ctrlObjProps = controlObjects[key].properties;
        return ctrlObjProps?.fetchesDefaults || ctrlObjProps.columns?.some((c) => c.properties?.fetchesDefaults);
    });
    const hasQuickEntry = Object.keys(controlObjects).some(key => {
        return controlObjects[key].properties?.canAddNewLine;
    });
    if (isMainList || (!hasFetchedDefaults && !hasQuickEntry)) {
        return {};
    }
    const parsedScreenValues = parseScreenValues(screenId, false, true);
    // Only dirty fields that the user manually modified should be sent to the server
    let dataValues = {};
    if (!clean && !sendNonDirtyFields) {
        dataValues = Object.keys(parsedScreenValues).reduce((prevValue, fieldId) => {
            if (screenDefinition.dirtyStates[fieldId] === true &&
                (!requestedFieldIds || requestedFieldIds.indexOf(fieldId) === -1)) {
                const bind = screenDefinition.metadata.uiComponentProperties?.[fieldId]?.bind ||
                    fieldId;
                set(prevValue, convertDeepBindToPathNotNull(bind), parsedScreenValues[fieldId]);
            }
            return prevValue;
        }, {});
    }
    else if (sendNonDirtyFields) {
        // All fields should be sent to the server
        dataValues = parsedScreenValues;
    }
    const fieldIds = requestedFieldIds ||
        getRequestableFieldIds(controlObjects, screenDefinition).filter(fieldId => {
            // Dirty fields will not be requested as we don't want the user's manual changes to be overwritten by the default values.
            return !screenDefinition.dirtyStates[fieldId];
        });
    if (isEmpty(fieldIds)) {
        return {};
    }
    const queryProps = getFieldsProperties(fieldIds, controlObjects, screenDefinition, nodeTypes, node);
    const screenProperties = screenDefinition.metadata.uiComponentProperties[screenId];
    const fieldsToRequest = buildNodeQueryFromProperties(queryProps, {}, true);
    // We don't request the fake ID that the defaults service returns
    delete fieldsToRequest._id;
    if (isEmpty(fieldsToRequest)) {
        return {};
    }
    // If the fake $new ID is used in duplication mode, we remove it from the payload because that is only used in the client
    if (dataValues._id === NEW_PAGE) {
        delete dataValues._id;
    }
    else if (!dataValues._id && screenDefinition.selectedRecordId !== NEW_PAGE) {
        dataValues._id = screenDefinition.selectedRecordId;
    }
    // TODO Filter out dirty fields
    return wrapQuery(screenProperties.node || '', {
        getDefaults: {
            __args: {
                data: { ...dataValues, ...additionalData },
            },
            ...fieldsToRequest,
        },
    }, alias);
};
export const getNavigationPanelOrderByDefinition = (listItem, explicitOrderBy) => {
    if (explicitOrderBy) {
        return explicitOrderBy;
    }
    const sortableFields = [
        FieldKey.Text,
        FieldKey.Reference,
        FieldKey.Label,
        FieldKey.Numeric,
        FieldKey.Date,
        FieldKey.Progress,
    ];
    const titleProps = listItem.title.properties;
    const line2Props = listItem.line2?.properties;
    const titleRightProps = listItem.titleRight
        ?.properties;
    const valueField = titleProps &&
        !titleProps.isTransient &&
        titleProps._controlObjectType &&
        sortableFields.includes(titleProps._controlObjectType) &&
        getReferenceValueFieldPath(titleProps);
    const orderBy = valueField ? set({}, valueField, 1) : {};
    if (line2Props?.bind &&
        !line2Props?.isTransient &&
        line2Props._controlObjectType &&
        sortableFields.includes(line2Props._controlObjectType)) {
        const line2ValueField = getReferenceValueFieldPath(line2Props);
        if (line2ValueField) {
            set(orderBy, line2ValueField, 1);
        }
    }
    else if (titleRightProps?.bind &&
        !titleRightProps.isTransient &&
        titleRightProps._controlObjectType &&
        sortableFields.includes(titleRightProps._controlObjectType)) {
        const titleLineRightValueField = getReferenceValueFieldPath(titleRightProps);
        if (titleLineRightValueField) {
            set(orderBy, titleLineRightValueField, 1);
        }
    }
    return orderBy;
};
//# sourceMappingURL=index.js.map