import { CUSTOM_DATA_PROPERTY } from '@sage/xtrem-shared';
import { chain } from 'lodash';
import { collectTypeDefinitionsAndReturn } from '../redux/actions/node-type-actions';
import { GraphQLTypes, textStreams } from '../types';
import { buildTypeQuery } from './graphql-query-builder';
import { executeGraphqlQuery } from './graphql-utils';
function getGraphqlTypeFromNodeInfoType(type) {
    switch (type) {
        case 'boolean':
            return 'Boolean';
        case 'string':
        case 'enum': // Temporary solution until the server can send us the enum members
            return 'String';
        case 'float':
            return 'Float';
        case 'integer':
            return 'Int';
        case 'decimal':
            return 'Decimal';
        case 'date':
            return 'Date';
        case '_InputTextStream':
            return 'InputTextStream';
        case '_OutputTextStream':
            return 'OuputTextStream';
        default:
            return type;
    }
}
function determinePropertyName(propertyKey, nodePropertyInfo, nodeDefinition) {
    if (nodePropertyInfo?.isCustom) {
        return `${CUSTOM_DATA_PROPERTY}.${propertyKey}`;
    }
    let propertyType;
    if (nodeDefinition[propertyKey]?.type) {
        propertyType = nodeDefinition[propertyKey]?.type;
    }
    else {
        const inferredType = getGraphqlTypeFromNodeInfoType(nodePropertyInfo.type);
        if (inferredType in GraphQLTypes) {
            propertyType = inferredType;
        }
        else {
            throw new Error(`Invalid inferred type: ${inferredType}`);
        }
    }
    const isTextStream = textStreams.includes(propertyType);
    return isTextStream ? `${propertyKey}.value` : propertyKey;
}
function determinePropertyKind(propertyKey, nodeDefinition, propertyType) {
    const kind = nodeDefinition[propertyKey]?.kind || 'SCALAR';
    if (textStreams.includes(propertyType)) {
        return 'SCALAR';
    }
    return kind;
}
export const fetchNodeNames = async (onlyPublishedNodes = false) => {
    const response = await executeGraphqlQuery({
        query: {
            getNodeNames: {
                __args: {
                    onlyPublishedNodes,
                },
                name: true,
                title: true,
            },
        },
        endpoint: '/metadata',
    });
    if (Array.isArray(response.data?.getNodeNames)) {
        return response.data.getNodeNames
            .sort((a, b) => a.title.localeCompare(b.title))
            .reduce((prevValue, currentValue) => {
            return { ...prevValue, [currentValue.name]: currentValue.title };
        }, {});
    }
    throw new Error('Failed to load node names from the server');
};
export const fetchNodeDetails = async ({ nodeName, getCollections = false, }) => {
    const introspectionQuery = buildTypeQuery(nodeName, 1);
    const introspectionResult = await executeGraphqlQuery({ query: introspectionQuery });
    const nodeDefinition = collectTypeDefinitionsAndReturn(introspectionResult.data[nodeName], nodeName);
    const nodeDetailsResponse = await executeGraphqlQuery({
        query: {
            getNodeDetails: {
                __args: { nodeName },
                name: true,
                title: true,
                defaultDataType: true,
                hasAttachments: true,
                properties: {
                    name: true,
                    title: true,
                    canSort: true,
                    canFilter: true,
                    enumType: true,
                    type: true,
                    isCustom: true,
                    dataType: true,
                    targetNode: true,
                    isOnInputType: true,
                    isOnOutputType: true,
                    isMutable: true,
                },
            },
        },
        endpoint: '/metadata',
    });
    const customPropertyKeys = nodeDetailsResponse.data.getNodeDetails.properties
        .filter((p) => p.isCustom)
        .map((p) => p.name);
    return chain([...Object.keys(nodeDefinition ?? {}), ...customPropertyKeys])
        .filter(k => ((k === '_id' || !k.startsWith('_')) && !nodeDefinition[k]?.type?.startsWith('_')) ||
        textStreams.includes(nodeDefinition[k].type))
        .thru(keys => getCollections
        ? keys
        : keys.filter(k => !nodeDefinition[k]?.isCollection && nodeDefinition[k]?.kind !== 'LIST'))
        .reduce((prevValue, propertyKey) => {
        const nodePropertyInfo = nodeDetailsResponse.data.getNodeDetails.properties.find((p) => p.name === propertyKey);
        if (propertyKey === '_id') {
            prevValue[propertyKey] = {
                ...nodeDefinition[propertyKey],
                type: GraphQLTypes.IntOrString,
                kind: 'SCALAR',
                name: propertyKey,
                canFilter: true,
                canSort: true,
                label: '_id',
                isOnInputType: false,
                isOnOutputType: true,
                dataType: '',
                targetNode: '',
                enumType: null,
                isCustom: false,
                isMutable: false,
            };
        }
        else {
            const type = nodePropertyInfo?.enumType ||
                nodeDefinition[propertyKey]?.type ||
                getGraphqlTypeFromNodeInfoType(nodePropertyInfo.type);
            prevValue[propertyKey] = {
                ...nodeDefinition[propertyKey],
                type,
                node: nodeDefinition[propertyKey]?.kind === 'OBJECT' ? nodeDefinition[propertyKey].type : undefined,
                kind: determinePropertyKind(propertyKey, nodeDefinition, type),
                name: determinePropertyName(propertyKey, nodePropertyInfo, nodeDefinition),
                canFilter: nodePropertyInfo?.canFilter || false,
                canSort: (nodePropertyInfo?.canSort && !textStreams.includes(type)) || false,
                label: nodePropertyInfo?.title || propertyKey,
                isOnInputType: nodePropertyInfo?.isOnInputType || false,
                isOnOutputType: nodePropertyInfo?.isOnOutputType || false,
                dataType: nodePropertyInfo?.dataType || '',
                targetNode: nodePropertyInfo?.targetNode || '',
                enumType: nodePropertyInfo?.enumType || null,
                isCustom: nodePropertyInfo?.isCustom || false,
                isMutable: nodePropertyInfo?.isMutable || false,
            };
        }
        return prevValue;
    }, {})
        .value();
};
//# sourceMappingURL=node-information-service.js.map