import { get, set, uniq } from 'lodash';
import { getLevelMap } from '../component/field/nested-grid/nested-grid-utils';
import { getReferenceValueField } from '../component/field/reference/reference-utils';
import { ActionKey, ContainerKey, FieldKey } from '../component/types';
import { GraphQLKind, GraphQLTypes } from '../types';
import { convertDeepBindToPath, convertDeepBindToPathNotNull, getNestedFieldsFromProperties, } from '../utils/nested-field-utils';
import { findDeepPropertyDetails } from '../utils/node-utils';
import { resolveByValue } from '../utils/resolve-value-utils';
import { schemaTypeNameFromNodeName, splitValueToMergedValue } from '../utils/transformers';
import { CollectionValue } from './collection-data-service';
import { CollectionFieldTypes } from './collection-data-types';
import { getUiComponentProperties } from './transactions-service';
export const extractDeepBoundValueFromJsonColumn = (record, bindSegments) => {
    const jsonFieldContent = record[bindSegments[0]];
    if (!jsonFieldContent) {
        return null;
    }
    // If the field is deep bound to a JSON field, then we need to parse the JSON string first, then find the value in it.
    const parsedCustomData = typeof jsonFieldContent === 'string' ? JSON.parse(jsonFieldContent) : jsonFieldContent;
    const customPropertyPath = [...bindSegments].splice(1, bindSegments.length).join('.');
    return get(parsedCustomData, customPropertyPath, null);
};
const unwrapAggregateValue = (aggregateValue) => {
    const firstKey = Object.keys(aggregateValue)[0];
    const firstItem = aggregateValue[firstKey];
    return !firstItem || typeof firstItem === 'string' || typeof firstItem === 'number'
        ? firstItem
        : unwrapAggregateValue(firstItem);
};
/**
 * Gets the value of the fields by element ID. In case of nested bind, it denormalizes the value.
 * @param elementId
 * @param fieldProperties
 * @param values
 * @returns
 */
const getValue = (elementId, values, topPropertyType, bind) => {
    if (bind) {
        const bindSegments = convertDeepBindToPathNotNull(bind).split('.');
        // If the field is not deeply bound (having only one path segment), then the user probably bound the column to the full JSON string, so we don't try to parse it.
        if (topPropertyType?.type === GraphQLTypes.Json && bindSegments.length > 1) {
            return extractDeepBoundValueFromJsonColumn(values, bindSegments);
        }
        // Given that the element ID is used as an alias in the query, we need to replace the first component of the bind path with the element ID.
        bindSegments[0] = elementId;
        return get(values, bindSegments.join('.'), null);
    }
    return values[elementId];
};
/**
 * Formats a single field into a suitable format based on its server-side GraphQL type or the corresponding control
 * object
 *
 * @param screenId
 * @param elementId
 * @param nodeTypes
 * @param fieldType
 * @param fieldProperties
 * @param value
 * @param parentNode
 */
export const formatValue = ({ pageDefinition, elementId, fieldProperties, fieldType, nodeTypes, parentNode, plugins, rowValue, screenId, value, }) => {
    // Empty values are set to null except tables/trees as they need to be wrapped to the CollectionValue object
    if ((value === null || value === undefined) &&
        fieldType !== FieldKey.Table &&
        fieldType !== FieldKey.TableSummary &&
        fieldType !== FieldKey.Tree &&
        fieldType !== FieldKey.PodCollection &&
        fieldType !== FieldKey.NestedGrid) {
        return null;
    }
    // IDs always formatted as strings
    if (elementId === '_id') {
        return String(value);
    }
    // First we try to format primitive values by their server side scalar type.
    if (parentNode && !fieldProperties.isTransient && !fieldProperties.isTransientInput) {
        const formattedNodeName = schemaTypeNameFromNodeName(parentNode);
        const nodePropertyDetails = findDeepPropertyDetails(formattedNodeName, elementId, nodeTypes);
        if (nodePropertyDetails) {
            const type = nodePropertyDetails.type;
            const kind = nodePropertyDetails.kind;
            switch (type) {
                case GraphQLTypes.Float:
                case GraphQLTypes.Decimal:
                    const convertFloat = (cValue) => Number.isNaN(Number(cValue)) ? null : parseFloat(cValue);
                    if (kind === GraphQLKind.List && Array.isArray(value)) {
                        return value.map(convertFloat);
                    }
                    return convertFloat(value);
                case GraphQLTypes.Int:
                    const convertInt = (cValue) => Number.isNaN(Number(cValue)) ? null : parseInt(cValue, 10);
                    if (kind === GraphQLKind.List && Array.isArray(value)) {
                        return value.map(convertInt);
                    }
                    return convertInt(value);
                case GraphQLTypes.String:
                    const convertString = (cValue) => String(cValue);
                    if (kind === GraphQLKind.List && Array.isArray(value)) {
                        return value.map(convertString);
                    }
                    if (value !== Object(value)) {
                        return convertString(value);
                    }
                    return null;
                case GraphQLTypes.Boolean:
                case GraphQLTypes.Date:
                    // Format date?
                    return value;
                case GraphQLTypes.Json:
                    /* We only parse JSON if we are in the dynamic pod context
                    the reason behind this is because functional devs are already parsing the JSON in the
                    functional code in other fields, so we don't want to parse it twice in order to avoid errors.
                     */
                    if (fieldProperties._controlObjectType === FieldKey.DynamicPod) {
                        // We only parse if is a string, otherwise we return the value as is.
                        if (typeof value === 'string') {
                            return JSON.parse(value);
                        }
                    }
                    return value;
                default:
                // Do nothing.
            }
        }
    }
    // If we couldn't format by the primitive type, we format by the control object type
    switch (fieldType) {
        case FieldKey.Plugin:
            const pluginProperties = fieldProperties;
            const plugin = plugins[pluginProperties.pluginPackage];
            return plugin.transformFromGraphValue ? plugin.transformFromGraphValue(value) : value;
        case FieldKey.NestedGrid:
            // Table values wrapped into CollectionValue objects
            const nestedGridProps = fieldProperties;
            const nestedGridFilters = nestedGridProps.levels.map(l => resolveByValue({
                screenId,
                skipHexFormat: true,
                propertyValue: l.filter,
                rowValue: null,
                fieldValue: null,
            }));
            const nestedGridOptionsMenu = resolveByValue({
                propertyValue: nestedGridProps.optionsMenu,
                rowValue: null,
                screenId,
                fieldValue: null,
                skipHexFormat: true,
            });
            const nodes = nestedGridProps.levels.map(l => String(l.node));
            return new CollectionValue({
                bind: nestedGridProps.bind,
                columnDefinitions: nestedGridProps.levels.map(level => level.columns),
                elementId,
                fieldType: CollectionFieldTypes.NESTED_GRID,
                filter: nestedGridFilters,
                hasNextPage: value?.pageInfo ? value.pageInfo.hasNextPage : false,
                initialValues: value?.data ?? [],
                isTransient: Boolean(nestedGridProps.isTransient) || Boolean(nestedGridProps.isTransientInput),
                levelMap: getLevelMap(nestedGridProps.levels),
                mapServerRecordFunctions: nestedGridProps.levels.map(l => l.mapServerRecord),
                nodes,
                nodeTypes,
                orderBy: nestedGridProps.levels.map(l => l.orderBy),
                screenId,
                activeOptionsMenuItem: pageDefinition?.tableViews?.[elementId]?.$current?.level?.[0].optionsMenuItem ||
                    nestedGridOptionsMenu?.[0],
            });
        case FieldKey.Table:
        case FieldKey.TableSummary:
        case FieldKey.Tree:
        case FieldKey.Calendar:
        case FieldKey.PodCollection:
            // Table values wrapped into CollectionValue objects
            const props = fieldProperties;
            const filter = typeof props.filter !== 'function' ? props.filter : undefined;
            const tableOptionsMenu = resolveByValue({
                propertyValue: props.optionsMenu,
                rowValue: null,
                screenId,
                fieldValue: null,
                skipHexFormat: true,
            });
            return new CollectionValue({
                bind: props.bind,
                screenId,
                elementId,
                isTransient: Boolean(props.isTransient) || Boolean(props.isTransientInput),
                hasNextPage: value?.pageInfo ? value.pageInfo.hasNextPage : false,
                orderBy: [props.orderBy],
                columnDefinitions: [getNestedFieldsFromProperties(props)],
                fieldType: fieldType === FieldKey.Table
                    ? CollectionFieldTypes.DESKTOP_TABLE
                    : fieldType === FieldKey.Tree
                        ? CollectionFieldTypes.TREE
                        : CollectionFieldTypes.POD,
                filter: [filter],
                initialValues: value?.data ? value.data : value instanceof Array ? value : [],
                mapServerRecordFunctions: [props.mapServerRecord],
                nodes: [props.node],
                nodeTypes,
                activeOptionsMenuItem: pageDefinition?.tableViews?.[elementId]?.$current?.level?.[0].optionsMenuItem ||
                    tableOptionsMenu?.[0],
                sublevelProperty: convertDeepBindToPath(fieldProperties.sublevelProperty),
            });
        case FieldKey.Text:
            // Check for objects, arrays etc
            if (value !== Object(value)) {
                return String(value);
            }
            return null;
        case FieldKey.MultiReference:
            if (value) {
                return value.map(splitValueToMergedValue);
            }
            return null;
        case FieldKey.FilterEditor:
        case FieldKey.ContentTable:
            if (typeof value === 'string') {
                return JSON.parse(value);
            }
            return null;
        case FieldKey.FilterSelect:
            const filterSelectValueFieldPath = getReferenceValueField(fieldProperties);
            if (filterSelectValueFieldPath) {
                const filterSelectActualValue = get(value, filterSelectValueFieldPath, null);
                return set(value, filterSelectValueFieldPath, filterSelectActualValue);
            }
            return value;
        case FieldKey.Reference:
            const referenceValue = value ? splitValueToMergedValue(value) : value;
            const referenceValueFieldPath = getReferenceValueField(fieldProperties);
            if (referenceValueFieldPath) {
                const referenceActualValue = get(referenceValue, referenceValueFieldPath, null);
                return set(referenceValue, referenceValueFieldPath, referenceActualValue);
            }
            return referenceValue;
        case FieldKey.Pod:
        case FieldKey.VitalPod:
            return value ? splitValueToMergedValue(value) : null;
        case FieldKey.Numeric:
            if (Number.isNaN(Number(value))) {
                return null;
            }
            const numericProps = fieldProperties;
            const scale = resolveByValue({
                screenId,
                skipHexFormat: true,
                rowValue: rowValue ? splitValueToMergedValue(rowValue) : rowValue,
                fieldValue: value,
                propertyValue: numericProps.scale,
            });
            return scale && scale !== 0 ? parseFloat(value) : parseInt(value, 10);
        case FieldKey.Aggregate:
            if (!value || !value.readAggregate) {
                return null;
            }
            const aggregateProperties = fieldProperties;
            if (typeof aggregateProperties.aggregateOn === 'string') {
                return (value.readAggregate[aggregateProperties.aggregateOn][aggregateProperties.aggregationMethod] || null);
            }
            return unwrapAggregateValue(value.readAggregate);
        case FieldKey.Count:
            if (!value?.query) {
                return null;
            }
            return parseInt(value.query.totalCount || 0, 10);
        case FieldKey.NodeBrowserTree:
            if (!value) {
                return null;
            }
            if (typeof value === 'string') {
                return JSON.parse(value);
            }
            if (typeof value === 'object') {
                return value;
            }
            return null;
        default:
            // If the value cannot be formatted, we return the original value as is.
            return value;
    }
};
/**
 * Format all values of a nested collection item into client-side suitable format based on various rules depending on
 * the server-side node GraphQL types and the control object types.
 * @param screenId
 * @param nodeTypes
 * @param collectionItem
 * @param columnsDefinitions
 * @param parentNode
 */
export const formatCollectionItem = ({ screenId, nodeTypes, collectionItem, plugins = {}, columnsDefinitions, parentNode, contextRow, }) => {
    const formattedCollectionItem = Object.keys(collectionItem).reduce((previousItem, key) => {
        const columnDefinitions = columnsDefinitions.filter(c => c.properties.bind === key);
        if (columnDefinitions.length === 0) {
            previousItem[key] = collectionItem[key];
            return previousItem;
        }
        columnDefinitions.forEach(columnDefinition => {
            if (columnDefinition &&
                !columnDefinition.properties.isTransient &&
                !columnDefinition.properties.isTransientInput) {
                previousItem[key] = formatValue({
                    elementId: key,
                    fieldProperties: columnDefinition.properties,
                    fieldType: columnDefinition.type,
                    nodeTypes,
                    parentNode,
                    plugins,
                    rowValue: contextRow || collectionItem,
                    screenId,
                    value: collectionItem[key],
                });
            }
            else {
                previousItem[key] = collectionItem[key];
            }
        });
        return previousItem;
    }, {});
    uniq(columnsDefinitions
        .filter(c => {
        const bind = convertDeepBindToPathNotNull(c.properties.bind);
        const bindSegments = bind.split('.');
        if (bindSegments.length === 1 || !parentNode) {
            return false;
        }
        const topPropertyType = nodeTypes[schemaTypeNameFromNodeName(parentNode)]?.[bindSegments[0]];
        return topPropertyType?.type === GraphQLTypes.Json;
    })
        .map(c => {
        const bind = convertDeepBindToPathNotNull(c.properties.bind);
        const bindSegments = bind.split('.');
        return bindSegments[0];
    })).forEach(c => {
        if (typeof collectionItem[c] === 'string') {
            formattedCollectionItem[c] = JSON.parse(collectionItem[c]);
        }
    });
    return formattedCollectionItem;
};
/**
 * Format all values of a screen into client-side suitable format based on various rules depending on the server-side
 * node GraphQL types and the control object types.
 *
 * If the `onlyElementIds` parameter is supplied, it will only format those values which are provided in this parameter.
 *
 * @param screenId
 * @param controlObjects
 * @param uiComponentProperties
 * @param nodeTypes
 * @param values
 * @param parentNode
 */
export const formatScreenValues = ({ pageDefinition, screenId, controlObjects, plugins, nodeTypes, values, parentNode, onlyElementIds, }) => {
    const nonBindingControlObjects = [
        ContainerKey.Section,
        ContainerKey.Block,
        ContainerKey.Page,
        FieldKey.Separator,
        ActionKey.PageAction,
    ];
    const resultValues = { _id: values._id, _etag: values._etag };
    Object.keys(controlObjects).forEach(elementId => {
        const fieldProperties = getUiComponentProperties(screenId, elementId);
        const fieldType = controlObjects[elementId].componentType;
        const topPropertyType = findDeepPropertyDetails(schemaTypeNameFromNodeName(parentNode), convertDeepBindToPathNotNull(fieldProperties?.bind || elementId).split('.')[0], nodeTypes);
        if (fieldProperties != null &&
            elementId !== screenId &&
            !nonBindingControlObjects.includes(fieldType) &&
            (!onlyElementIds || onlyElementIds.includes(elementId)) &&
            !fieldProperties.isTransient &&
            !fieldProperties.isTransientInput) {
            resultValues[elementId] = formatValue({
                pageDefinition,
                screenId,
                elementId,
                nodeTypes,
                fieldType,
                plugins,
                fieldProperties,
                value: getValue(elementId, values, topPropertyType, fieldProperties.bind),
                parentNode,
                rowValue: values,
            });
        }
    });
    return resultValues;
};
//# sourceMappingURL=value-formatter-service.js.map