import { DateValue, datePropertyValueToDateString } from '@sage/xtrem-date-time';
import { Decimal } from '@sage/xtrem-decimal';
import { isEmpty, isNil, isObject, merge, set } from 'lodash';
import { FieldKey } from '../component/types';
import { GraphQLTypes } from '../types';
import { xtremConsole } from '../utils/console';
import { convertDeepBindToPathNotNull } from '../utils/nested-field-utils';
import { findDeepPropertyDetails } from '../utils/node-utils';
import { cleanMetadataFromRecord } from '../utils/transformers';
import { isDevMode } from '../utils/window';
import { CollectionValue } from './collection-data-service';
import { INPUT_NODE_POSTFIX } from '../utils/constants';
const isEmptyValue = (value) => value === null || value === undefined || value === '';
// Sometimes not the same version of the decimal library is used, so the first check can fail, that's why we need the second, more nasty approach
const isDecimal = (value) => Decimal.isDecimal(value) || (isObject(value) && value.constructor?.name === 'Decimal');
// Checks if the object and all of its properties is null or undefined
const isEmptyObject = (obj) => {
    return Object.values(obj).every((value) => {
        if (isNil(value))
            return true;
        if ((value instanceof Array || value instanceof Object) && isEmpty(value))
            return true;
        if (value instanceof Array && !isEmpty(value))
            return isEmptyObject(value);
        if (value instanceof Object && !isEmpty(value))
            return isEmptyObject(value);
        return false;
    });
};
/**
 * Gets a saveable dataset from a `CollectionValue` instance.
 *
 * @param value
 * @param fieldType
 * @param isRequestingDefaults
 * @returns
 */
const getCollectionValueChangeSet = (value, fieldType, bind, isRequestingDefaults = false, targetNode, nodeTypes) => {
    const targetProperty = findDeepPropertyDetails(targetNode, bind, nodeTypes, false);
    if (fieldType === FieldKey.NestedGrid) {
        // nested grid
        const changeSet = value.getChangedRecordsAsTree();
        if (changeSet.length > 0) {
            return changeSet;
        }
    }
    else if (targetProperty?.type === GraphQLTypes.Json) {
        return value.getData({ cleanMetadata: true, noLimit: true });
    }
    else {
        const changeSet = value.getNormalizedChangedRecords(isRequestingDefaults);
        const cleanChangeSet = changeSet.map(element => cleanMetadataFromRecord(element));
        if (changeSet.length > 0) {
            return cleanChangeSet;
        }
    }
    return [];
};
/**
 * Formats any data to a GraphQL input based on provided node type definition.
 * It checks the expected GraphQL input type and converts the value to it wherever it is possible. In case of nested structures in
 * converts the properties deeply by following the graph definitions.
 *
 * Whenever a client side value cannot be converted to the server defined type (e.g. boolean to an object), an exception is thrown
 *
 * @param param0
 * @returns
 */
export const formatInputProperty = ({ isRequestingDefaults = false, nodeTypes, propertyKind, propertyType, propertyValue, }) => {
    if (propertyKind === 'LIST' && propertyValue) {
        if (Array.isArray(propertyValue)) {
            return propertyValue.map(v => formatInputProperty({ nodeTypes, propertyValue: v, propertyType }));
        }
        throw new Error(`Cannot serialize non iterable value to a collection. Value: ${propertyValue}`);
    }
    switch (propertyType) {
        case GraphQLTypes.ID:
            if (propertyValue === null) {
                throw new Error("Couldn't serialize the ID type property because its value is null.");
            }
            if (propertyValue instanceof Object) {
                if (propertyValue._id) {
                    return String(propertyValue._id);
                }
                throw new Error(`Couldn't serialize ${propertyValue} to an input string, no _id property was found.`);
            }
            return String(propertyValue);
        case GraphQLTypes.Boolean:
            if (propertyValue === null) {
                return null;
            }
            return propertyValue === 'false' ? false : Boolean(propertyValue);
        case GraphQLTypes.Decimal:
            if (isEmptyValue(propertyValue)) {
                return null;
            }
            if (isDecimal(propertyValue)) {
                return propertyValue.toNumber();
            }
            return propertyValue;
        case GraphQLTypes.IntOrString:
            if (isEmptyValue(propertyValue)) {
                return null;
            }
            if (isDecimal(propertyValue)) {
                return propertyValue.round().toNumber();
            }
            if (propertyValue instanceof Object) {
                if (propertyValue._id) {
                    return String(propertyValue._id);
                }
                throw new Error(`Couldn't serialize ${propertyValue} to an input string, no _id property was found.`);
            }
            const intOrStringReturnValue = Math.round(Number(propertyValue));
            if (Number.isNaN(intOrStringReturnValue)) {
                throw new Error(`Couldn't serialize ${propertyValue} to a IntOrString input type`);
            }
            return String(intOrStringReturnValue);
        case GraphQLTypes.Int:
            if (isEmptyValue(propertyValue)) {
                return null;
            }
            if (isDecimal(propertyValue)) {
                return propertyValue.round().toNumber();
            }
            const intReturnValue = Math.round(Number(propertyValue));
            if (Number.isNaN(intReturnValue)) {
                throw new Error(`Couldn't serialize ${propertyValue} to a Int input type`);
            }
            return intReturnValue;
        case GraphQLTypes.Float:
            if (isEmptyValue(propertyValue)) {
                return null;
            }
            if (isDecimal(propertyValue)) {
                return propertyValue.toNumber();
            }
            const floatReturnValue = Number(propertyValue);
            if (Number.isNaN(floatReturnValue)) {
                throw new Error(`Couldn't serialize ${propertyValue} to a Float input type`);
            }
            return floatReturnValue;
        case GraphQLTypes.String:
            if (propertyValue instanceof Object) {
                if (propertyValue._id) {
                    return String(propertyValue._id);
                }
                throw new Error(`Couldn't serialize ${propertyValue} to an input string, no _id property was found.`);
            }
            return propertyValue ? String(propertyValue) : null;
        case GraphQLTypes.Date:
            if (isEmptyValue(propertyValue)) {
                return null;
            }
            if (!(propertyValue instanceof Date) && Number.isInteger(Number(propertyValue))) {
                return datePropertyValueToDateString(DateValue.fromJsDate(new Date(Number(propertyValue))));
            }
            return datePropertyValueToDateString(propertyValue);
        case GraphQLTypes.DateTime:
            if (isEmptyValue(propertyValue)) {
                return null;
            }
            if (!(propertyValue instanceof Date) && Number.isInteger(Number(propertyValue))) {
                return datePropertyValueToDateString(new Date(Number(propertyValue)), true);
            }
            return datePropertyValueToDateString(propertyValue, true);
        case GraphQLTypes.IntReference:
        case GraphQLTypes.ExternalReference:
            if (!propertyValue) {
                return null;
            }
            if (typeof propertyValue === 'number' || typeof propertyValue === 'string') {
                return String(propertyValue);
            }
            if (typeof propertyValue !== 'object' || propertyValue._id === null || propertyValue._id === undefined) {
                return null;
            }
            // Non-vital references
            return propertyValue && propertyValue._id ? `_id:${propertyValue._id}` : null;
        case GraphQLTypes.InputStream:
            if (!propertyValue) {
                return null;
            }
            if (typeof propertyValue === 'string') {
                return { value: propertyValue };
            }
            if (typeof propertyValue === 'object' && Object.prototype.hasOwnProperty.call(propertyValue, 'value')) {
                return propertyValue.value ? propertyValue : null;
            }
            throw new Error(`Cannot serialize ${propertyValue} to Input stream`);
        case GraphQLTypes.Json:
            if (!propertyValue) {
                return null;
            }
            if (typeof propertyValue === 'string') {
                return propertyValue;
            }
            return JSON.stringify(propertyValue);
        default:
            // Here we handel vital object and collection types and enums
            if (!propertyValue) {
                return null;
            }
            if (!propertyType) {
                // property type is not defined, just return the original value
                return propertyValue;
            }
    }
    // Here we handel vital object and enums
    if (!isEmpty(nodeTypes) && propertyType.endsWith(INPUT_NODE_POSTFIX)) {
        if (typeof propertyValue !== 'object') {
            throw new Error(`Cannot serialize ${propertyValue} into input object`);
        }
        // Vital references
        return Object.keys(propertyValue).reduce((prevValue, currentValue) => {
            if (!nodeTypes[propertyType]) {
                throw new Error(`Cannot serialize, unknown node type: ${propertyType}`);
            }
            const childNodeType = findDeepPropertyDetails(propertyType, currentValue, nodeTypes);
            if (!childNodeType) {
                return prevValue;
            }
            if (currentValue === '_id' && !propertyValue[currentValue]) {
                return prevValue;
            }
            const mergedValue = merge(prevValue, {
                [currentValue]: formatInputProperty({
                    nodeTypes,
                    propertyType: childNodeType.type,
                    propertyKind: childNodeType.kind,
                    propertyValue: propertyValue[currentValue],
                    isRequestingDefaults,
                }),
            });
            // If all keys of the object is null, then we send a null value instead
            if (isEmptyObject(mergedValue)) {
                return null;
            }
            return mergedValue;
        }, {});
    }
    // Enums
    if (propertyType.endsWith('_EnumInput')) {
        if (isEmptyValue(propertyValue)) {
            return null;
        }
        if (Number.isInteger(Number(propertyValue))) {
            return Number(propertyValue);
        }
        if (typeof propertyValue === 'string') {
            return propertyValue;
        }
        throw new Error(`Cannot serialize ${propertyValue} to enum input type (${propertyType})`);
    }
    if (isDevMode()) {
        xtremConsole.log(`Couldn't serialize ${propertyType} GraphQL property type.`);
    }
    return null;
};
/**
 * Builds the mutation `data` payload from the values and properties, first it normalizes the value structure and filters out the
 * non saveable fields, then format the values to match the input value.
 */
export const serializePageData = ({ values, uiComponentProperties, isRequestingDefaults, nodeTypes, targetNode, }) => {
    const output = {};
    // Normalize flat data structure to structured payload
    Object.keys(values).forEach(elementId => {
        if (elementId === '_etag' && values._etag) {
            set(output, '_etag', values._etag);
        }
        const properties = uiComponentProperties[elementId];
        if (!properties) {
            return;
        }
        const fieldType = properties._controlObjectType;
        const stateValue = values[elementId];
        // Rule out certain fields
        if (properties.isTransient ||
            fieldType === FieldKey.Aggregate ||
            fieldType === FieldKey.Count ||
            fieldType === FieldKey.Separator) {
            return;
        }
        if (elementId === '_id' && !stateValue) {
            return;
        }
        const bind = properties.bind ? convertDeepBindToPathNotNull(properties.bind) : elementId;
        const isCollectionValue = stateValue instanceof CollectionValue;
        const valueToSet = isCollectionValue
            ? getCollectionValueChangeSet(stateValue, fieldType, bind, isRequestingDefaults, targetNode, nodeTypes)
            : stateValue;
        /**
         * Empty collection values are not included into the payload because it would reset the collection. If the changeset is empty, we don't need to send anything to the server
         * because it indicates that nothing has actually changed.
         *  */
        if (isCollectionValue && Array.isArray(valueToSet) && valueToSet.length === 0) {
            return;
        }
        set(output, bind, valueToSet);
    });
    const formattedOutput = formatInputProperty({
        isRequestingDefaults,
        nodeTypes,
        propertyType: targetNode,
        propertyValue: output,
    });
    return formattedOutput;
};
//# sourceMappingURL=value-serializer-service.js.map