import * as tokens from '@sage/design-tokens/js/base/common';
import { DateValue, Datetime, datePropertyValueToDateString, isStringDate, date as xtremDate, } from '@sage/xtrem-date-time';
import * as xtremDecimal from '@sage/xtrem-decimal';
import { ChartTypes } from '../component/chart-types';
import { getStore } from '../redux';
import { resolveByValue } from './resolve-value-utils';
import { splitValueToMergedValue } from './transformers';
const defaultFormat = 'YYYY-MM-DD';
export const getIsoDate = (date) => {
    if (date) {
        if (date instanceof Date) {
            return xtremDate.fromJsDate(date).format('YYYY-MM-DD');
        }
        if (typeof date === 'string' && date.match(/[1-2][0-9]{3}-[0-1][0-9]-[0-3][0-9]/)) {
            return date;
        }
        throw new Error(`Invalid date: ${date}`);
    }
    return undefined;
};
export const isInvalidJsDate = (jsDate) => Number.isNaN(jsDate.getTime());
export const dateToString = (date, dateFieldFormat = defaultFormat) => {
    return DateValue.parse(datePropertyValueToDateString(date)).format(dateFieldFormat);
};
export const isDateOrDateTime = (value) => {
    if (typeof value === 'string' && isStringDate(value)) {
        return value.length === 10 ? 'date' : 'datetime';
    }
    if (value instanceof DateValue) {
        return 'date';
    }
    if (value instanceof Datetime) {
        return 'datetime';
    }
    if (value instanceof Date) {
        return 'datetime';
    }
    return null;
};
export const chartColors = [
    tokens.colorsActionMajor500,
    tokens.colorsSemanticCaution500,
    tokens.colorsSemanticPositive500,
    tokens.colorsSemanticNegative500,
    tokens.colorsSemanticInfo500,
    tokens.colorsUtilityMajor200,
    tokens.colorsUtilityMajor100,
];
export function getAgChartProperties(screenId, properties, collection) {
    const isPieChart = properties.chart.type === ChartTypes.Pie;
    const series = properties.chart.series.map((currentSeries, index) => {
        const currentSeriesProperties = currentSeries.properties;
        const color = chartColors.length < index + 1 ? chartColors[0] : chartColors[index];
        const baseOptions = {
            // type: representation type
            type: properties.chart.type,
            outerRadiusOffset: isPieChart ? -10 : undefined,
            stroke: color,
            marker: {
                fill: color,
                stroke: color,
            },
            tooltip: {
                renderer: (params) => `${params.xValue || params.labelKey}: ${formatNumericValue({
                    screenId,
                    value: params.yValue || params.angleValue,
                    scale: currentSeriesProperties.scale,
                })}`,
            },
        };
        if (isPieChart) {
            baseOptions.labelKey = properties.chart.xAxis.properties.bind;
            baseOptions.angleKey = currentSeriesProperties.bind;
            baseOptions.fills = chartColors;
            baseOptions.strokes = [tokens.colorsUtilityMajor100];
            baseOptions.highlightStyle = {
                fill: tokens.colorsActionMajor600,
                stroke: tokens.colorsYin090,
            };
        }
        else {
            baseOptions.xKey = properties.chart.xAxis.properties.bind;
            baseOptions.yKey = currentSeriesProperties.bind;
            baseOptions.yName = currentSeriesProperties.title;
        }
        return baseOptions;
    });
    // Remapping Y Axes values to ensure that they are numbers
    const yAxesProperties = series.map(s => s.yKey || s.angleKey);
    const data = collection?.data?.map(d => Object.keys(d).reduce((prevValue, currentValue) => {
        if (yAxesProperties.indexOf(currentValue) !== -1) {
            prevValue[currentValue] = Number(d[currentValue]);
        }
        else {
            prevValue[currentValue] = d[currentValue];
        }
        return prevValue;
    }, {})) || [];
    const chartOptions = {
        autoSize: true,
        data,
        series,
        legend: {
            enabled: !properties.isLegendHidden,
            position: 'bottom',
        },
    };
    if (!isPieChart) {
        chartOptions.axes = [
            {
                type: 'category',
                position: 'bottom',
                label: {
                    rotation: -45,
                },
            },
            {
                type: 'number',
                position: 'left',
            },
        ];
    }
    return chartOptions;
}
export const getFirstDayOfMonth = (date) => date.begOfMonth();
// TODO Pass the week configuration (e.g. consider weeks starting on Sunday)
export const getFirstDayOfWeek = (date) => date.begOfWeek();
export const getLastDayOfMonth = (date) => date.endOfMonth();
// TODO Pass the week configuration (e.g. consider weeks starting on Sunday)
export const getLastDayOfWeek = (date) => date.endOfWeek();
export const formatNumericValue = ({ screenId, value, ignoreLocale = false, scale, rowValue, locale, }) => {
    const localeToUse = locale || getStore().getState()?.applicationContext?.locale || 'en-US';
    const tempValue = parseFloat(value);
    if (Number.isNaN(Number(tempValue))) {
        return '';
    }
    if (scale !== undefined && scale !== null) {
        const resolvedScale = resolveByValue({
            fieldValue: value,
            propertyValue: scale,
            rowValue: splitValueToMergedValue(rowValue),
            skipHexFormat: true,
            screenId,
        });
        const formatOptions = {
            minimumFractionDigits: resolvedScale,
            maximumFractionDigits: resolvedScale,
            ...(ignoreLocale && { useGrouping: false }),
        };
        return new Intl.NumberFormat(ignoreLocale ? 'en-US' : localeToUse || 'en-US', formatOptions).format(tempValue);
    }
    return new Intl.NumberFormat(ignoreLocale ? 'en-US' : localeToUse, {
        ...(ignoreLocale && { useGrouping: false }),
    }).format(tempValue);
};
export const parseLocalizedNumberStringToNumber = (value, separator) => {
    if (typeof value === 'number') {
        return value;
    }
    if (value === undefined || value === null || String(value.trim()).length === 0) {
        return null;
    }
    if (value.trim() === '-') {
        return 0;
    }
    const isNegative = value.trim().charAt(0) === '-';
    const parts = value.trim().split(separator);
    const decimalPart = parts.length === 1 ? '0' : parts[1].trim().replace(/\D/g, '');
    const wholeNumberPart = parts[0].replace(/\D/g, '');
    return xtremDecimal.Decimal.make(`${isNegative ? '-' : ''}${wholeNumberPart || '0'}.${decimalPart || '0'}`).toNumber();
};
export const getNumberComponentsFromLocalizedNumberString = (value, separator) => {
    let internal = typeof value === 'string' ? value : typeof value === 'number' ? `${value}` : '';
    let sign = '+';
    if (internal[0] === '-' || internal[0] === '+') {
        sign = internal[0];
        internal = internal.substring(1);
    }
    const [integer, decimal] = internal.split(separator);
    return { integer: integer || '', decimal: decimal || '', separator, sign };
};
//# sourceMappingURL=formatters.js.map