import { navigationPanelId } from '../component/container/navigation-panel/navigation-panel-types';
import { AbstractUiControlObject, SectionControlObject } from '../component/control-objects';
import { getStore } from '../redux/store';
import { xtremConsole } from './console';
import { resolveByValue } from './resolve-value-utils';
import { convertDeepBindToPathNotNull } from './nested-field-utils';
export const isScreenDefinitionDirty = (screenDefinition) => !screenDefinition.metadata.uiComponentProperties[screenDefinition.metadata.screenId].skipDirtyCheck &&
    !!Object.keys(screenDefinition.dirtyStates).find(key => screenDefinition.dirtyStates[key] === true) &&
    screenDefinition.type !== 'sticker';
export const hasAnyDirtyScreenDefinitions = (state) => !!Object.keys(state.screenDefinitions).find(screenId => isScreenDefinitionDirty(state.screenDefinitions[screenId]) === true) ||
    state.dashboard.dashboardEditor.isDirty ||
    state.dashboard.widgetEditor.isDirty ||
    isSidebarDirty(state);
export const getPageDefinitionFromState = (screenId, state = getStore().getState()) => {
    const pageDefinition = state.screenDefinitions[screenId];
    if (!pageDefinition) {
        throw new Error(`No page definition found: ${screenId}`);
    }
    return pageDefinition;
};
export const getMainPageDefinitionFromState = (state = getStore().getState()) => Object.keys(state.screenDefinitions)
    .filter(key => state.screenDefinitions[key].type === 'page')
    .map(key => state.screenDefinitions[key])
    .find(definition => definition.isMainPage);
export const getPagePropertiesFromState = (screenId, state = getStore().getState()) => {
    const properties = getPageDefinitionFromState(screenId, state).metadata.uiComponentProperties[screenId];
    if (!properties) {
        xtremConsole.error(getPageDefinitionFromState(screenId, state));
        throw new Error(`No properties found for page: ${screenId}`);
    }
    return properties;
};
export const getPageControlObjectFromState = (screenId, state = getStore().getState()) => getPageDefinitionFromState(screenId, state).metadata.controlObjects[screenId] || null;
export const getPagePropertiesFromPageDefinition = (pageDefinition) => pageDefinition.metadata.uiComponentProperties[pageDefinition.metadata.screenId];
export const getNavigationPanelState = (screenId, state = getStore().getState()) => state.screenDefinitions[screenId]?.navigationPanel || null;
export const getNavigationPanelDefinitionFromState = (screenId, state = getStore().getState()) => getPagePropertiesFromState(screenId, state).navigationPanel || null;
export const getNavigationPanelTablePropertiesFromPageDefinition = (pageDefinition) => pageDefinition.metadata.uiComponentProperties[navigationPanelId];
export const checkIfPageIsLoaded = (screenId, state = getStore().getState()) => {
    if (!getPageDefinitionFromState(screenId, state)) {
        throw new Error(`${screenId} page has already been destroyed, further operations are not allowed.`);
    }
};
export const arePlatformLiteralsMissing = (state = getStore().getState(), locale = state.applicationContext?.locale || 'en-US') => !state.translations ||
    !state.translations[locale] ||
    !Object.keys(state.translations[locale]).find(key => key.startsWith('@sage/xtrem-ui'));
export const getSidebarLayoutDefinitionFromState = (state = getStore().getState()) => {
    if (!state.sidebar?.screenId || !state.sidebar.elementId) {
        return {};
    }
    const pageDefinition = getPageDefinitionFromState(state.sidebar.screenId, state);
    const tableProperties = pageDefinition.metadata.uiComponentProperties[state.sidebar?.elementId];
    if (!tableProperties.sidebar) {
        throw new Error(`No sidebar is defined for ${state.sidebar.elementId}`);
    }
    return resolveByValue({
        screenId: state.sidebar.screenId,
        propertyValue: tableProperties.sidebar.layout,
        rowValue: null,
        fieldValue: null,
        skipHexFormat: true,
    });
};
export const getSidebarNormalFields = (state = getStore().getState()) => {
    const collector = [];
    const layoutDefinition = getSidebarLayoutDefinitionFromState(state);
    Object.keys(layoutDefinition || {}).forEach(sectionKey => {
        Object.keys(layoutDefinition[sectionKey].blocks || []).forEach(blockKey => {
            (layoutDefinition[sectionKey].blocks[blockKey].fields || []).forEach(f => {
                if (f instanceof AbstractUiControlObject) {
                    collector.push(f);
                }
            });
        });
    });
    return collector;
};
export const getSidebarNestedFields = (state = getStore().getState()) => {
    const collector = [];
    const layoutDefinition = getSidebarLayoutDefinitionFromState(state);
    Object.keys(layoutDefinition || {}).forEach(sectionKey => {
        Object.keys(layoutDefinition[sectionKey].blocks || []).forEach(blockKey => {
            (layoutDefinition[sectionKey].blocks[blockKey].fields || []).forEach(f => {
                if (!(f instanceof AbstractUiControlObject)) {
                    collector.push(convertDeepBindToPathNotNull(f));
                }
            });
        });
    });
    return collector;
};
export const isSidebarDirty = (state = getStore().getState()) => {
    if (!state.sidebar) {
        return false;
    }
    const tableValue = state.screenDefinitions[state.sidebar.screenId].values[state.sidebar.elementId];
    const records = tableValue.getData({
        isUncommitted: true,
        includePhantom: true,
        cleanMetadata: false,
        where: {
            ...(state.sidebar.recordId ? { _id: state.sidebar.recordId } : { __phantom: true }),
        },
    });
    if (records?.[0]?.__dirtyColumns && (records?.[0]?.__dirtyColumns).size > 0) {
        return true;
    }
    const normalFields = getSidebarNormalFields(state);
    const screenId = state.sidebar.screenId;
    return !!normalFields.find(n => state.screenDefinitions[screenId].dirtyStates[n.id]);
};
export const getVisibleSectionsFromPage = (screenId, screenDefinition = getStore().getState().screenDefinitions[screenId]) => Object.values(screenDefinition.metadata.controlObjects).filter(controlObject => controlObject instanceof SectionControlObject &&
    !resolveByValue({
        screenId,
        skipHexFormat: true,
        propertyValue: controlObject.properties.isHidden,
        rowValue: null,
        fieldValue: null,
    }));
//# sourceMappingURL=state-utils.js.map