import React, { useState, useEffect, PureComponent, useRef, useLayoutEffect, useCallback } from 'react';
import { ResponsiveContainer, LineChart, Line, CartesianGrid, XAxis, YAxis, Treemap, RadialBarChart, PolarAngleAxis, RadialBar, ComposedChart, Tooltip, Legend as Legend$1, Bar, PieChart, Pie, Cell, Sector } from 'recharts';
import { cloneDeepWith, isString, get, omit, isEqual, cloneDeep, noop as noop$1, camelCase, merge } from 'lodash';
import styled from 'styled-components';
import Typography from 'carbon-react/lib/components/typography';
import Decimal from 'decimal.js';
import Image from 'carbon-react/lib/components/image';
import Pill from 'carbon-react/lib/components/pill';
import { v4 } from 'uuid';
import Link from 'carbon-react/lib/components/link';
import Icon from 'carbon-react/lib/components/icon';
import IconButton from 'carbon-react/lib/components/icon-button';
import { Checkbox } from 'carbon-react/lib/components/checkbox';
import Portrait from 'carbon-react/lib/components/portrait';
import Tooltip$1 from 'carbon-react/lib/components/tooltip';
import * as tokens from '@sage/design-tokens/js/base/common';
import { InfiniteLoader, AutoSizer, List, ColumnSizer, Table } from 'react-virtualized';
import Loader from 'carbon-react/lib/components/loader';
import Button from 'carbon-react/lib/components/button';
import { ActionPopover, ActionPopoverItem, ActionPopoverDivider, ActionPopoverMenuButton } from 'carbon-react/lib/components/action-popover';
import { Chart } from 'react-charts';
import useLatestWhen from 'react-charts/lib/hooks/useLatestWhen';
import { VisualProcessEditorComponent } from '@sage/visual-process-editor';
import CarbonPreview from 'carbon-react/lib/components/preview';
import axios from 'axios';
import { Tabs, Tab } from 'carbon-react/lib/components/tabs';
import ReactGridLayout from 'react-grid-layout';

function LineGraph({ data }) {
    return (React.createElement("div", { className: "db-widget-content widgetLineGraph" },
        React.createElement("div", { className: "fg1" },
            React.createElement(ResponsiveContainer, { width: "100%", aspect: 3 },
                React.createElement(LineChart, { width: 600, height: 300, data: data, margin: { top: 5, right: 20, bottom: 5, left: 0 } },
                    React.createElement(Line, { type: "monotone", dataKey: "uv", stroke: "#8884d8" }),
                    React.createElement(CartesianGrid, { stroke: "#ccc", strokeDasharray: "5 5" }),
                    React.createElement(XAxis, { dataKey: "name" }),
                    React.createElement(YAxis, null)))),
        React.createElement("div", { className: "db-widget-skeletonFooter" }, "Sticky footer of the content")));
}

const currenciesMapping = {
    'en-US': 'USD',
    'en-GB': 'GBP',
    'fr-FR': 'EUR',
    'es-ES': 'EUR',
    'pt-PT': 'EUR',
    'pt-BR': 'BRL',
    'ar-SA': 'SAR',
    'de-DE': 'EUR',
    'it-IT': 'EUR',
    'pl-PL': 'PLN',
    'zh-CN': 'CNY',
};
const formatNumber = (num, locale = 'en-US', precision = 2) => {
    const numberFormat = new Intl.NumberFormat(locale, {
        maximumFractionDigits: precision,
        minimumFractionDigits: precision,
    });
    return numberFormat.format(num);
};
const formatCurrencyNumber = (num, locale = 'en-US', precision = 2, currency) => {
    const numberFormat = new Intl.NumberFormat(locale, {
        maximumFractionDigits: precision,
        minimumFractionDigits: precision,
        style: 'currency',
        currency: currency ?? currenciesMapping[locale],
    });
    return numberFormat.format(num);
};

const tokenReplace = '######';
function translateObject(obj, stringsLiterals) {
    if (!obj) {
        return obj;
    }
    const translated = cloneDeepWith(obj, objValue => {
        if (isString(objValue)) {
            const arrayVal = objValue
                .split('{')
                .join(`${tokenReplace}{`)
                .split('}')
                .join(`}${tokenReplace}`)
                .split(tokenReplace);
            const arrayValTranslated = arrayVal.map(val => {
                const valWithoutArc = val.split('{').join('').split('}').join('');
                return get(stringsLiterals, valWithoutArc) ?? val;
            });
            return arrayValTranslated.join('');
        }
    });
    return translated;
}
function resolveTranslationKey(stringLiterals, key) {
    if (key == null) {
        return key;
    }
    const valWithoutArc = key.split('{').join('').split('}').join('');
    const resolvedValue = get(stringLiterals, valWithoutArc);
    if (resolvedValue != null) {
        return resolvedValue;
    }
    return key;
}

const TableWidget = ({ data, locale, stringLiterals, }) => {
    const [comparing, setComparing] = useState(true);
    const isPositiveValue = (element) => {
        return parseInt(getPercentage(element.value, element.previousValue, false)) > 0;
    };
    const generateRows = (elements) => {
        const rows = [];
        for (let index = 0; index < elements.length; index++) {
            const element = elements[index];
            rows.push(React.createElement("tr", null,
                React.createElement("td", { className: "values taRight" }, index + 1 + '.'),
                React.createElement("td", null,
                    React.createElement("a", { href: element.url, target: "_blank", rel: "noreferrer" }, element.name)),
                React.createElement("td", { className: comparing ? 'values' : 'hidden' },
                    formatNumber(element.previousValue, locale),
                    "\u20AC"),
                React.createElement("td", { className: "values" },
                    formatNumber(element.value, locale),
                    "\u20AC"),
                React.createElement("td", { className: comparing ? 'smallText taRight' : 'hidden' },
                    getPercentage(element.value, element.previousValue, true),
                    "%",
                    ' ',
                    React.createElement("svg", { version: "1.1", xmlns: "http://www.w3.org/2000/svg", x: "0px", y: "0px", className: isPositiveValue(element) ? 'arrow arrowUp' : 'arrow arrowDown', viewBox: "0 0 213.333 213.333" },
                        React.createElement("g", null,
                            React.createElement("g", null,
                                React.createElement("polygon", { points: "0,53.333 106.667,160 213.333,53.333 \t\t" }))))),
                React.createElement("td", { className: "taRight" },
                    element.share,
                    "%")));
        }
        return rows;
    };
    function getPercentage(value, previousValue, withoutSign = false) {
        let difference = value - previousValue;
        if (withoutSign && difference < 0) {
            difference = difference * -1;
        }
        return ((difference * 100) / previousValue).toFixed(2);
    }
    try {
        return (React.createElement(React.Fragment, null, data ? (React.createElement(React.Fragment, null,
            React.createElement("table", { className: "tableWidget fg1" },
                React.createElement("thead", null,
                    React.createElement("tr", null,
                        React.createElement("th", { className: "numberColumn taRight" }, "#"),
                        React.createElement("th", null, "Name"),
                        React.createElement("th", { className: comparing ? 'taRight' : 'hidden' }, "April"),
                        React.createElement("th", { className: "taRight" }, "Purchases May \u20AC"),
                        React.createElement("th", { className: comparing ? '' : 'hidden' }),
                        React.createElement("th", { className: "taRight" }, "Share"))),
                generateRows(data.data),
                React.createElement("tr", null,
                    React.createElement("td", null),
                    React.createElement("td", null,
                        React.createElement("a", { href: "https://www.google.com", target: "_blank", rel: "noreferrer" }, data.others.name)),
                    React.createElement("td", { className: comparing ? 'values' : 'hidden' },
                        formatNumber(data.others.previousValue, locale),
                        "\u20AC"),
                    React.createElement("td", { className: "values" },
                        formatNumber(data.others.value, locale),
                        "\u20AC"),
                    React.createElement("td", { className: comparing ? '' : 'hidden' }),
                    React.createElement("td", { className: "taRight" },
                        data.others.share,
                        "%"))),
            ' ',
            React.createElement("div", { className: "mt2 mb1" },
                React.createElement("label", null,
                    React.createElement("input", { type: "checkbox", checked: comparing, onChange: () => setComparing(!comparing) }),
                    "\u00A0 Compare with previous period")))) : (React.createElement(React.Fragment, null))));
    }
    catch (error) {
        return React.createElement("p", null, resolveTranslationKey(stringLiterals, 'general_invalid_json'));
    }
};
TableWidget.displayName = 'TableWidget';

class CustomizedContent extends PureComponent {
    render() {
        const { x, y, width, height, size, locale } = this.props;
        return (React.createElement("g", null,
            React.createElement("rect", { x: x, y: y, width: width, height: height, style: {
                    stroke: '#FFFFFF',
                    strokeWidth: 5,
                } }),
            React.createElement("text", { x: x + 15, y: y + 15 + 14, textAnchor: "left", fill: "#00000", style: {
                    stroke: '#000000',
                    strokeWidth: 1,
                }, fontSize: 14 },
                size ? formatNumber(size, locale) : React.createElement(React.Fragment, null),
                "\u20AC"),
            React.createElement("text", { x: x + 15, y: y + 35 + 14, textAnchor: "left", fill: "#6292c3", style: {
                    stroke: '#6292c3',
                    strokeWidth: 1,
                }, fontSize: 14 }, name)));
    }
}
const TreemapWidget = ({ data, locale, stringLiterals, }) => {
    useEffect(() => { });
    try {
        return (React.createElement("div", { className: "tableWidget treemapWidget fg1" },
            React.createElement(ResponsiveContainer, { width: "100%" },
                React.createElement(Treemap, { data: data.tree, dataKey: "size", stroke: "#FFFFFF", 
                    // strokeWidth="5"
                    fill: "#e8ecec", content: React.createElement(CustomizedContent, { locale: locale }) }))));
    }
    catch (error) {
        return React.createElement("p", null, resolveTranslationKey(stringLiterals, 'general_invalid_json'));
    }
};
TreemapWidget.displayName = 'TreemapWidget';

const TableTreeMap = ({ data, stringLiterals, locale, }) => {
    const [mode, setMode] = useState('table');
    const [rowsNumber, setRowsNumber] = useState(10);
    const [tableData, setTableData] = useState();
    const [treemapData, setTreemapData] = useState();
    useEffect(() => {
        if (rowsNumber === 10) {
            const elements = getNElements(data.elements, 10);
            setTableData(elements);
            setTreemapData(formatDataToTreemap(data.elements, 10));
        }
        else if (rowsNumber === 20) {
            const elements = getNElements(data.elements, 20);
            setTableData(elements);
            setTreemapData(formatDataToTreemap(data.elements, 20));
        }
        else if (rowsNumber === 30) {
            setTableData(data.elements);
            setTreemapData(formatDataToTreemap(data.elements, 30));
        }
    }, [rowsNumber]);
    const getNElements = (data, numberOfElements) => {
        const slicedData = data?.data?.slice(0, numberOfElements);
        const elements = { ...data, data: slicedData };
        return elements;
    };
    const formatDataToTreemap = (data, numberOfElements) => {
        const slicedData = data?.data?.slice(0, numberOfElements);
        const treeData = slicedData?.map(e => {
            return {
                name: e.name,
                size: e.value,
            };
        });
        const elements = { ...data, tree: treeData };
        return elements;
    };
    const renderWidget = () => {
        if (mode === 'table') {
            return React.createElement(TableWidget, { data: tableData, locale: locale, stringLiterals: stringLiterals });
        }
        else {
            return React.createElement(TreemapWidget, { data: treemapData, locale: locale, stringLiterals: stringLiterals });
        }
    };
    try {
        return (React.createElement("div", { className: "db-widget-content widgetTreeMap" },
            React.createElement("div", { className: "mb1" },
                React.createElement("select", { className: "topSelector", name: "top", id: "top", onChange: e => setRowsNumber(parseInt(e.target.value)) },
                    React.createElement("option", { value: "10" }, "Top 10"),
                    React.createElement("option", { value: "20" }, "Top 20"),
                    React.createElement("option", { value: "30" }, "Top 30")),
                React.createElement("select", { className: "modeSelector", name: "mode", id: "mode", onChange: e => setMode(e.target.value) },
                    React.createElement("option", { value: "table" }, "Table"),
                    React.createElement("option", { value: "area" }, "Area"))),
            tableData ? renderWidget() : React.createElement(React.Fragment, null)));
    }
    catch (error) {
        return React.createElement("p", null, resolveTranslationKey(stringLiterals, 'general_invalid_json'));
    }
};
TableTreeMap.displayName = 'TableTreeMap';

function ProgressTracker({ title = '', progress, current, total, currency = '€', locale, stringLiterals, }) {
    const progressArray = [{ value: progress, name: 'value' }];
    const circleSize = 125;
    const getColor = (value) => {
        if (value <= 33) {
            return 'red';
        }
        else if (value > 33 && value <= 69) {
            return '#FFB500';
        }
        return '#009900';
    };
    try {
        return (React.createElement("div", { className: "db-widget-content widgetProgressTracker" },
            React.createElement("h2", { className: "mt0" }, resolveTranslationKey(stringLiterals, title)),
            React.createElement("div", { className: "responsiveContainer" },
                React.createElement(RadialBarChart, { width: circleSize, height: circleSize, cx: circleSize / 2, cy: circleSize / 2, innerRadius: circleSize / 2 - 10, outerRadius: circleSize - 10, barSize: 5, data: progressArray, startAngle: 90, endAngle: -270 },
                    React.createElement(PolarAngleAxis, { type: "number", domain: [0, 100], angleAxisId: 0, tick: false }),
                    React.createElement(RadialBar, { background: true, dataKey: "value", cornerRadius: circleSize / 2, fill: getColor(progressArray[0].value) }),
                    React.createElement("text", { x: circleSize / 2, y: circleSize / 2, textAnchor: "middle", dominantBaseline: "middle", className: "progress-label", fontSize: "2em", fontWeight: "bold", fill: getColor(progressArray[0].value) },
                        progress,
                        "%"))),
            React.createElement("div", { className: "mt1 mb1" },
                React.createElement("div", { className: "budget" },
                    React.createElement("a", null,
                        resolveTranslationKey(stringLiterals, total.name),
                        ":"),
                    React.createElement("a", { className: "data" },
                        formatNumber(total.value, locale),
                        " ",
                        currency)),
                React.createElement("div", { className: "balance" },
                    React.createElement("a", null,
                        resolveTranslationKey(stringLiterals, current.name),
                        ": "),
                    React.createElement("a", { className: "data" },
                        formatNumber(current.value, locale),
                        " ",
                        currency)))));
    }
    catch (error) {
        return React.createElement("p", null, resolveTranslationKey(stringLiterals, 'general_invalid_json'));
    }
}

const DEFAULT_HEIGHT = 48;
const DEFAULT_WIDTH = 48;
const getImageUri = (serializedSVG) => `data:image/svg+xml;utf-8,${encodeURIComponent(serializedSVG)}`;
const onImageLoad = ({ canvas, width, height }) => (e) => {
    const image = e.target;
    const context = canvas.getContext('2d');
    if (!context) {
        return;
    }
    context.drawImage(image, 0, 0, width, height);
};
const fixHtmlColor = (html, color) => {
    if (!color)
        return html;
    return html.replace(/fill:[^;]*/gm, `fill:${color}`).replace(/fill="[^"]*"/gm, `fill="${color}"`);
};
const fetchSVG = async ({ canvasRef, url, color, width, height }) => {
    try {
        const response = await fetch(url);
        const html = await response.text();
        const parser = new DOMParser();
        const htmlWithColorFixed = fixHtmlColor(html, color);
        const doc = parser.parseFromString(htmlWithColorFixed, 'text/html');
        const svg = doc.body.children[0];
        const xmlSerializer = new XMLSerializer();
        const customSVGAttributes = {
            preserveAspectRatio: 'none',
        };
        Object.keys(customSVGAttributes).forEach(key => svg.setAttribute(key, customSVGAttributes[key]));
        const canvas = canvasRef.current;
        svg.setAttribute('width', String(width));
        svg.setAttribute('height', String(height));
        const serializedSVG = xmlSerializer.serializeToString(svg);
        const image = new window.Image();
        image.crossOrigin = 'Anonymous';
        image.onload = onImageLoad({ canvas, width, height });
        image.src = getImageUri(serializedSVG);
    }
    catch (error) {
        console.warn(`Could not load image from '${url}' due to the following error: `, error);
    }
};
const SvgIcon = ({ url, color }) => {
    const canvasRef = useRef(null);
    useEffect(() => {
        fetchSVG({ canvasRef, url, color, width: DEFAULT_WIDTH, height: DEFAULT_HEIGHT });
    }, [url, color]);
    return (React.createElement("canvas", { height: DEFAULT_HEIGHT, width: DEFAULT_WIDTH, ref: canvasRef, className: "db-draggable-element-indicator" }));
};

const DifferenceValue = ({ differenceValue, locale }) => {
    return (React.createElement(React.Fragment, null,
        formatNumber(differenceValue, locale),
        "%",
        React.createElement("svg", { version: "1.1", xmlns: "http://www.w3.org/2000/svg", x: "0px", y: "0px", className: differenceValue > 0 ? 'arrow arrowUp' : 'arrow arrowDown', viewBox: "0 0 213.333 213.333" },
            React.createElement("g", null,
                React.createElement("g", null,
                    React.createElement("polygon", { points: "0,53.333 106.667,160 213.333,53.333 \t\t" }))))));
};
const SimpleCurrencyIndicator = ({ currencyCode, description = '', differenceValue, indicator, locale, title = '', }) => {
    return (React.createElement("div", { className: "indicator-widget-content", "data-indicator-type": "currency" },
        React.createElement("h3", { className: "indicatorWidgetTitle db-draggable-element-indicator" }, title),
        differenceValue !== undefined && !isNaN(differenceValue) ? (React.createElement("div", { className: "differenceValue" },
            React.createElement(DifferenceValue, { locale: locale, differenceValue: differenceValue }))) : null,
        indicator?.value !== undefined && !isNaN(indicator.value) ? (React.createElement("div", { className: "mainIndicatorValue" }, formatCurrencyNumber(indicator.value, locale, 2, currencyCode))) : null,
        indicator?.amount !== undefined && !isNaN(indicator.amount) ? (React.createElement("div", { className: "mainIndicatorAmount" }, formatNumber(indicator.amount, locale, 0))) : null,
        React.createElement("div", { className: "mainIndicatorTitle db-draggable-element-indicator" }, indicator?.title),
        React.createElement("div", { className: "description db-draggable-element-indicator" },
            React.createElement("a", null, description))));
};

const SimpleNumberIndicator = ({ description = '', locale, prefix, suffix, subtitle = '', value, }) => {
    const hasPrefix = typeof prefix === 'string';
    const hasSuffix = typeof suffix === 'string';
    return (React.createElement("div", { className: "indicator-widget-content", "data-indicator-type": "number" },
        React.createElement("div", { className: "indicator-widget-content__line indicator-widget-content__line--line1 db-draggable-element-indicator" },
            hasPrefix && (React.createElement("span", { className: "indicator-widget-content__line-content indicator-widget-content__line-content--prefix", "data-testid": "indicator-widget-prefix" }, prefix)),
            React.createElement("span", { className: "indicator-widget-content__line-content indicator-widget-content__line-content--value", "data-testid": "indicator-widget-value" }, formatNumber(value, locale)),
            hasSuffix && (React.createElement("span", { className: "indicator-widget-content__line-content indicator-widget-content__line-content--suffix", "data-testid": "indicator-widget-suffix" }, suffix))),
        React.createElement("div", { className: "indicator-widget-content__line indicator-widget-content__line--line2 db-draggable-element-indicator" },
            React.createElement("span", { className: "indicator-widget-content__line-content indicator-widget-content__line-content--subtitle", "data-testid": "indicator-widget-subtitle" }, subtitle)),
        React.createElement("div", { className: "indicator-widget-content__line indicator-widget-content__line--line3 db-draggable-element-indicator" },
            React.createElement("span", { className: "indicator-widget-content__line-content indicator-widget-content__line-content--description", "data-testid": "indicator-widget-description" }, description))));
};
SimpleNumberIndicator.displayName = 'SimpleNumberIndicator';

const SimpleTextIndicator = ({ description = '', subtitle = '', title, }) => {
    return (React.createElement("div", { className: "indicator-widget-content", "data-indicator-type": "text" },
        React.createElement("div", { className: "indicator-widget-content__line indicator-widget-content__line--line1 db-draggable-element-indicator" },
            React.createElement("span", { className: "indicator-widget-content__line-content indicator-widget-content__line-content--title", "data-testid": "indicator-widget-title" }, title)),
        React.createElement("div", { className: "indicator-widget-content__line indicator-widget-content__line--line2 db-draggable-element-indicator" },
            React.createElement("span", { className: "indicator-widget-content__line-content indicator-widget-content__line-content--subtitle", "data-testid": "indicator-widget-subtitle" }, subtitle)),
        React.createElement("div", { className: "indicator-widget-content__line indicator-widget-content__line--line3 db-draggable-element-indicator" },
            React.createElement("span", { className: "indicator-widget-content__line-content indicator-widget-content__line-content--description", "data-testid": "indicator-widget-description" }, description))));
};
SimpleTextIndicator.displayName = 'SimpleTextIndicator';

const getIndicatorProps = (type = 'currency', props) => {
    switch (type) {
        case 'currency':
            return {
                currencyCode: props.currencyCode,
                description: resolveTranslationKey(props.stringLiterals, props.description),
                differenceValue: props.differenceValue,
                indicator: props.mainIndicator,
                locale: props.locale,
                title: props.title,
            };
        case 'number': {
            const indicator = props.mainIndicator;
            return {
                description: resolveTranslationKey(props.stringLiterals, props.description),
                locale: props.locale,
                prefix: indicator.prefix,
                suffix: indicator.suffix,
                subtitle: indicator.subtitle,
                value: indicator.value,
            };
        }
        case 'text': {
            const indicator = props.mainIndicator;
            return {
                description: resolveTranslationKey(props.stringLiterals, props.description),
                subtitle: indicator.subtitle,
                title: indicator.title,
            };
        }
        default:
            return {
                currencyCode: props.currencyCode,
                description: props.description,
                differenceValue: props.differenceValue,
                indicator: props.mainIndicator,
                locale: props.locale,
                title: props.title,
            };
    }
};
function SimpleIndicator(props) {
    const indicatorProps = getIndicatorProps(props.type, props);
    try {
        return (React.createElement("div", { className: "db-widget-content widgetSimpleIndicator" },
            props.icon ? (React.createElement("div", { className: "indicatorWidgetIcon" },
                React.createElement(SvgIcon, { ...props.icon }))) : null,
            React.createElement("div", { className: "indicatorWidgetTexts" },
                (props.type === undefined || props.type === 'currency') && (React.createElement(SimpleCurrencyIndicator, { ...indicatorProps })),
                props.type === 'number' && (React.createElement(SimpleNumberIndicator, { ...indicatorProps })),
                props.type === 'text' && (React.createElement(SimpleTextIndicator, { ...indicatorProps })))));
    }
    catch (error) {
        return React.createElement("p", null, resolveTranslationKey(props.stringLiterals, 'general_invalid_json'));
    }
}

const WidgetMainContainer = styled.div `
  & p {
    margin-bottom: 0;
  }
`;
const SummaryContainer = styled.div `
  display: flex;
  justify-content: space-between;
  margin-bottom: 50px;
  align-items: center;
`;
const MainTitleContainer = styled.div `
  display: flex;
  gap: 2rem;
  align-items: center;
`;
const IndicatorTitleContainer = styled.div `
  margin-bottom: 8px;
  & p {
    color: rgba(0, 0, 0, 0.55);
  }
`;
const IndicatorAmountContainer = styled.div `
  margin-bottom: 8px;
`;
const DescriptionContainer = styled.div `
  margin-top: 4px;
  & p {
    color: rgba(0, 0, 0, 0.55);
  }
`;
const IndicatorsContainerComponent = ({ children, className, }) => {
    return React.createElement("div", { className: className }, children);
};
const IndicatorsContainer = styled(IndicatorsContainerComponent) `
  display: flex;
  justify-content: space-around;
  & > div:not(:first-child):not(:last-child) {
    border-left: 0.5px solid #c6c6c6;
    border-right: 0.5px solid #c6c6c6;
  }
  ${({ indicatorsAmount }) => indicatorsAmount > 1
    ? `
    & > div:last-child {
      border-left: 0.5px solid #C6C6C6;
    }
    & > div:first-child {
      border-right: 0.5px solid #C6C6C6;
    }`
    : ''}
  & > div {
    width: ${({ indicatorsAmount }) => (indicatorsAmount > 0 ? 100 / indicatorsAmount : 100)}%;
  }
`;
const IndicatorContainer = styled.div `
  display: flex;
  flex-direction: column;
  align-items: flex-end;
`;
const RightContainer = styled.div `
  display: flex;
  align-items: center;
  justify-content: center;
  gap: 2rem;
`;
const RoundedImage = styled(Image) `
  border-radius: 9999px;
`;
const computeTotal = (indicators) => indicators
    .reduce((acc, currentIndicator) => currentIndicator.value ? Decimal.add(acc, currentIndicator.value) : acc, new Decimal(0))
    .toNumber();
const getTotalAmount$1 = (indicators) => indicators
    .reduce((acc, currentIndicator) => currentIndicator.amount ? Decimal.add(acc, currentIndicator.amount) : acc, new Decimal(0))
    .toNumber();
const Indicator = ({ indicator, locale, currencyCode }) => {
    const { title, value, amount } = indicator;
    return (React.createElement(IndicatorContainer, null,
        React.createElement(IndicatorTitleContainer, null,
            React.createElement(Typography, { mr: "18px", fontWeight: "500", fontSize: "12px", lineHeight: "18px" }, title)),
        amount !== undefined && !isNaN(amount) ? (React.createElement(IndicatorAmountContainer, null,
            React.createElement(Typography, { mr: "18px", fontWeight: "700", fontSize: "16px", lineHeight: "19px" }, formatNumber(amount, locale, 0)))) : null,
        value !== undefined && !isNaN(value) ? (React.createElement(Typography, { mr: "18px", fontWeight: "700", fontSize: "16px", lineHeight: "19px" }, currencyCode
            ? formatCurrencyNumber(value, locale, 2, currencyCode)
            : formatCurrencyNumber(value, locale))) : null));
};
const MultipleIndicators = ({ currencyCode, customTotal, customTotalAmount, description, icon, image, indicators, isTotalAmountDisplayed, isTotalDisplayed, locale, pillColor, pillTextContent, title, }) => {
    return (React.createElement(WidgetMainContainer, null,
        React.createElement(SummaryContainer, null,
            React.createElement(MainTitleContainer, null,
                image !== undefined && (React.createElement(RoundedImage, { src: `data:image;base64,${image}`, alt: "Image", size: 48 })),
                image === undefined && icon !== undefined && React.createElement(SvgIcon, { ...icon }),
                React.createElement("div", null,
                    React.createElement(Typography, { fontWeight: "700", fontSize: "22px", lineHeight: "26.4px" },
                        title,
                        isTotalAmountDisplayed
                            ? ` (${formatNumber(customTotalAmount ?? getTotalAmount$1(indicators), locale, 0)})`
                            : ''),
                    React.createElement(DescriptionContainer, null,
                        React.createElement(Typography, { fontWeight: "400", fontSize: "14px", lineHeight: "16.8px" }, description)))),
            React.createElement(RightContainer, null,
                isTotalDisplayed && (React.createElement(Typography, { fontWeight: "700", fontSize: "22px", lineHeight: "26.4px" }, currencyCode
                    ? formatCurrencyNumber(customTotal ?? computeTotal(indicators), locale, 2, currencyCode)
                    : formatCurrencyNumber(customTotal ?? computeTotal(indicators), locale))),
                pillTextContent && (React.createElement(Pill, { borderColor: pillColor, size: "S", fill: true }, pillTextContent)))),
        React.createElement(IndicatorsContainer, { indicatorsAmount: indicators.length }, indicators.map((indicator, indicatorIndex) => (React.createElement(Indicator, { key: indicatorIndex, indicator: indicator, locale: locale, currencyCode: currencyCode }))))));
};

const DivWithIndent = ({ children, className }) => {
    return React.createElement("div", { className: className }, children);
};
const LinkContainer = styled(DivWithIndent) `
  margin-left: ${({ indent }) => indent * 16 + 16}px;
  & button {
    text-decoration: none;
  }
  & button:hover {
    text-decoration: underline;
  }
  margin-top: 4px;
  margin-bottom: 4px;
`;
const LabelContainer = styled(DivWithIndent) `
  margin-left: ${({ indent }) => indent * 16 + 16}px;
  margin-top: 4px;
  margin-bottom: 4px;
`;
const LeftLabelContainer = styled.div `
  margin-top: 4px;
  margin-bottom: 4px;
`;
const ListAndTotalContainer = styled.div `
  display: flex;
  flex-direction: column;
`;
const ListContainer = styled.div `
  display: flex;
  flex-direction: column;
`;
const ListLineContainer = styled.div `
  display: flex;
  flex-direction: row;
  justify-content: space-between;
`;
const ListLine = ({ outstanding, linksCallback }) => {
    const { label, value, indent, link } = outstanding;
    return (React.createElement(ListLineContainer, null,
        link ? (React.createElement(LinkContainer, { indent: indent },
            React.createElement(Link, { ...(linksCallback
                    ? {
                        onClick: () => {
                            linksCallback(link);
                        },
                    }
                    : { href: link }) }, label))) : (React.createElement(LabelContainer, { indent: indent }, label)),
        value ? React.createElement(LeftLabelContainer, null, value) : null));
};
const TotalLine = ({ total }) => {
    const { label, value } = total;
    return (React.createElement(ListLineContainer, null,
        React.createElement(Typography, { variant: "strong", ml: 16, mt: 16, mb: "4px" }, label ?? 'Total'),
        React.createElement(Typography, { variant: "strong", mb: "4px", mt: 16 }, value)));
};
const SimpleList = ({ simpleListData, linksCallback }) => {
    const { outstandings, total } = simpleListData;
    return (React.createElement(ListAndTotalContainer, null,
        React.createElement(ListContainer, null,
            outstandings?.map?.(outstanding => (React.createElement(ListLine, { key: v4(), outstanding: outstanding, linksCallback: linksCallback }))) ?? null,
            total ? React.createElement(TotalLine, { total: total }) : null)));
};

const HorizontalBar = ({ title, values, totalText, currency, locale, stringLiterals, }) => {
    const total = getTotalAmount(values);
    const totalAbsolue = getTotalAmountAbsolute(values);
    const divChart = useRef(null);
    const [canDisplay, setCanDisplay] = useState(false);
    function generateBar(amountElements, currency, total, stringLiterals) {
        const elements = [];
        const chartWidth = divChart.current?.offsetWidth ?? 0;
        let totalBarWidth = 0;
        amountElements?.forEach((element, i) => {
            const percentage = getPercentage(Math.abs(element.amount), totalAbsolue);
            const colourClass = getColourClass(element.id);
            let overlay = null;
            if (colourClass === 'oldest' ||
                colourClass === 'positive' ||
                colourClass === 'late' ||
                colourClass === 'latest') {
                overlay = generateSummaryOverlay(element, colourClass, currency, stringLiterals);
            }
            const widthelem = getWidthBar(Math.abs(element.amount), totalAbsolue, chartWidth);
            const calculateStyletooltipText = generateStyleTooltip(widthelem, chartWidth, totalBarWidth);
            totalBarWidth = totalBarWidth + widthelem;
            elements.push(React.createElement("div", { key: i, className: colourClass + ' bmsTooltip', style: {
                    width: percentage,
                } },
                React.createElement("span", { className: "tooltipText", style: calculateStyletooltipText },
                    resolveTranslationKey(stringLiterals, element.tooltipTitle),
                    " ",
                    React.createElement("br", null),
                    " ",
                    element.amount,
                    currency,
                    React.createElement("br", null),
                    " ",
                    resolveTranslationKey(stringLiterals, element.tooltipInfo)),
                overlay ? overlay : React.createElement(React.Fragment, null)));
        });
        return elements;
    }
    function calculateOverlayStyle() {
        const chartRight = divChart?.current?.getBoundingClientRect().right;
        if (chartRight) {
            const arrayOverly = ['positive', 'late', 'latest'];
            arrayOverly.forEach(overly => {
                const currentOverly = document.querySelector(`div[class^='${overly}']`);
                if (currentOverly) {
                    const lastspan = currentOverly.querySelector('span:last-child');
                    const lastSpanRight = lastspan?.getBoundingClientRect()?.right ?? 0;
                    if (lastspan && lastSpanRight > chartRight) {
                        lastspan.style.right = '0px';
                        lastspan.style.left = 'unset';
                    }
                }
            });
            const oldestBar = document.querySelector(`div[class^='oldest']`);
            let previousSize = { left: 0, right: 0, bottom: 120 };
            if (oldestBar) {
                const oldestLastspan = oldestBar.querySelector('span:last-child');
                if (oldestLastspan) {
                    const { left, right } = oldestLastspan.getBoundingClientRect() ?? { left: 0, right: 0 };
                    previousSize = { left, right, bottom: 120 };
                }
            }
            arrayOverly.forEach(overly => {
                const currentOverly = document.querySelector(`div[class^='${overly}']`);
                if (currentOverly) {
                    const lastspan = currentOverly.querySelector('span:last-child');
                    if (lastspan) {
                        const currentSize = {
                            left: lastspan.getBoundingClientRect().left,
                            right: lastspan.getBoundingClientRect().right,
                        };
                        if (currentSize.left < previousSize.right) {
                            lastspan.style.bottom = `${previousSize.bottom === 120 ? 180 : 120}%`;
                            previousSize = { ...previousSize, bottom: previousSize.bottom };
                        }
                        else {
                            previousSize = { ...currentSize, bottom: 120 };
                        }
                    }
                }
            });
        }
        if (divChart.current) {
            divChart.current.style.opacity = '1';
        }
    }
    useEffect(() => {
        setCanDisplay(true);
        setTimeout(() => {
            calculateOverlayStyle();
        }, 1000);
    }, []);
    try {
        return (React.createElement("div", { className: "db-widget-content widgetHorizontalBar" },
            React.createElement("div", { className: "responsiveContainer" },
                React.createElement(Typography, { variant: "h4", mt: 0, mb: 3 }, resolveTranslationKey(stringLiterals, title)),
                React.createElement("div", { ref: divChart, className: "chart" }, canDisplay && generateBar(values, currency, total, stringLiterals)),
                React.createElement("div", { className: "total mt2" },
                    formatNumber(total, locale),
                    currency,
                    " ",
                    resolveTranslationKey(stringLiterals, totalText)))));
    }
    catch (error) {
        return React.createElement("p", null, resolveTranslationKey(stringLiterals, 'general_invalid_json'));
    }
};
function getColourClass(id) {
    switch (id) {
        case 1:
            return 'oldest';
        case 2:
            return 'older';
        case 3:
            return 'justPast';
        case 4:
            return 'positive';
        case 5:
            return 'next';
        case 6:
            return 'shortLong';
        case 7:
            return 'late';
        case 8:
            return 'moreThanLate';
        case 9:
            return 'superLate';
        case 10:
            return 'latest';
    }
}
function getTotalAmount(amountElements) {
    if (amountElements) {
        return amountElements?.reduce((a, b) => a + b.amount, 0);
    }
    return 0;
}
function getTotalAmountAbsolute(amountElements) {
    if (amountElements) {
        return amountElements?.reduce((a, b) => a + Math.abs(b.amount), 0);
    }
    return 0;
}
function getPercentage(amount, total) {
    return (amount * 100) / total + '%';
}
function getWidthBar(amount, total, chartWidth) {
    return chartWidth * (amount / total);
}
function generateSummaryOverlay(element, colourClass, currency, stringLiterals) {
    let text = '';
    if (element.overlayText) {
        if (colourClass === 'oldest')
            text = ' ' + resolveTranslationKey(stringLiterals, element.overlayText);
        else if (colourClass === 'late')
            text = ' ' + resolveTranslationKey(stringLiterals, element.overlayText) + ' ';
        else if (colourClass === 'latest')
            text = '  ' + resolveTranslationKey(stringLiterals, element.overlayText) + ' ';
    }
    return React.createElement("span", { className: "overlyInfoText" }, element.amount + currency + text);
}
function generateStyleTooltip(widthelem, chartWidth, totalBarWidth) {
    let calculateStyletooltipText = {};
    const positionTooltip = totalBarWidth + widthelem / 2 - 60;
    if (positionTooltip < 0) {
        calculateStyletooltipText = { ...calculateStyletooltipText, left: '0', marginLeft: '0' };
    }
    else if (positionTooltip <= chartWidth - 120) {
        calculateStyletooltipText = { ...calculateStyletooltipText, left: '50%', marginLeft: '-60px' };
    }
    else {
        calculateStyletooltipText = { ...calculateStyletooltipText, right: '0', marginLeft: '0' };
    }
    return calculateStyletooltipText;
}
HorizontalBar.displayName = 'HorizontalBar';

function StatusReport({ data, elements, locale, stringLiterals, }) {
    const [legend, setLegend] = useState(false);
    const getElement = (component) => {
        switch (component.type) {
            case 'bar':
                return React.createElement(Bar, { dataKey: component.name, barSize: 20, className: component.class });
            case 'line':
                return (React.createElement(Line, { type: "linear", dataKey: component.name, dot: false, strokeWidth: 3, stroke: '#ffb500', className: component.class }));
            default:
                return React.createElement(React.Fragment, null);
        }
    };
    const getChartComponents = () => {
        const chartComponents = [];
        elements.forEach(element => {
            chartComponents.push(getElement(element));
        });
        return chartComponents;
    };
    const customLegend = () => {
        const items = [];
        elements?.forEach((element) => {
            let total = 0;
            data?.forEach((col) => {
                total += col[element.name];
            });
            const className = `legendItem ${element.class}`;
            items.push(React.createElement("div", { className: className },
                React.createElement("a", { className: "legendItemValue" },
                    formatNumber(total, locale),
                    "\u20AC"),
                React.createElement("a", { className: "legendItemLabel" }, element.name)));
        });
        setLegend(React.createElement("div", { className: "legendBody" }, items));
    };
    const getLegend = () => {
        return legend;
    };
    useEffect(() => {
        customLegend();
    }, []);
    try {
        return (React.createElement("div", { className: "db-widget-content widgetStatusReport", style: { width: '100%' } }, getLegend() ? (React.createElement(ResponsiveContainer, { width: "100%", aspect: 3 },
            React.createElement(ComposedChart, { data: data, barGap: 0 },
                React.createElement(CartesianGrid, { vertical: false }),
                React.createElement(XAxis, { dataKey: "name", tickLine: false, fontWeight: "bold", tickMargin: 15 }),
                React.createElement(YAxis, { tickMargin: 15 }),
                React.createElement(Tooltip, null),
                React.createElement(Legend$1, { content: getLegend, align: "right", verticalAlign: "middle", layout: "vertical" }),
                React.createElement(CartesianGrid, { stroke: "#f5f5f5" }),
                getChartComponents()))) : (React.createElement(React.Fragment, null))));
    }
    catch (error) {
        return React.createElement("p", null, resolveTranslationKey(stringLiterals, 'general_invalid_json'));
    }
}

const hexToRgb = (hex) => {
    const result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})?$/i.exec(hex);
    return result
        ? {
            r: parseInt(result[1], 16),
            g: parseInt(result[2], 16),
            b: parseInt(result[3], 16),
        }
        : null;
};

function TileIndicator({ value = '', title = '', color, icon, onClick, }) {
    const getBgColor = () => {
        if (color.startsWith('#')) {
            const components = hexToRgb(color);
            if (components == null) {
                throw new Error(`Cannot convert ${color} to hex.`);
            }
            return `rgba(${components.r}, ${components.g}, ${components.b}, 0.2)`;
        }
        return color;
    };
    const body = (React.createElement(React.Fragment, null,
        React.createElement("div", { className: "iconContainer", style: { background: getBgColor() } },
            React.createElement(Icon, { type: icon, color: color, fontSize: "large" })),
        React.createElement("div", { className: "tileIndicatorBody" },
            React.createElement(Typography, { fontWeight: "700", lineHeight: "30px", fontSize: "30px", mb: "2", "data-test-id": "db-tile-indicator-value" }, value),
            React.createElement(Typography, { fontSize: "18px", mb: 0, color: "blackOpacity65" }, title))));
    return onClick ? (React.createElement("button", { className: "db-widget-content widgetTileIndicator db-draggable-element-indicator", onClick: onClick }, body)) : (React.createElement("div", { className: "db-widget-content widgetTileIndicator db-draggable-element-indicator" }, body));
}

var TableRowStatus;
(function (TableRowStatus) {
    TableRowStatus[TableRowStatus["LOADING"] = 0] = "LOADING";
    TableRowStatus[TableRowStatus["LOADED"] = 1] = "LOADED";
    TableRowStatus[TableRowStatus["FAILED"] = 2] = "FAILED";
    TableRowStatus[TableRowStatus["STALE"] = 3] = "STALE";
})(TableRowStatus || (TableRowStatus = {}));

const noop = () => {
    // Intentionally left empty.
};
const resolveValueOrCallback = (property, i) => {
    if (!property) {
        return null;
    }
    if (property instanceof Function) {
        return property(i);
    }
    return property;
};

const StyledLinkForTable = styled(Link) `
  line-height: 30px;
  display: block;
  width: 100%;
  button {
    font-size: 12px;
    width: 100%;
    span {
      width: 100%;
      overflow: hidden;
      white-space: nowrap;
      text-overflow: ellipsis;
      display: block;
      text-align: left;
    }
  }
`;
const StyledLink$1 = styled(Link) `
  line-height: 18px;
  display: block;
  width: 100%;
  button {
    font-size: 14px;
    width: 100%;
    span {
      width: 100%;
      overflow: hidden;
      white-space: nowrap;
      text-overflow: ellipsis;
      display: block;
      text-align: left;
    }
  }
`;
const StyledLargeLink = styled(Link) `
  line-height: 18px;
  button {
    font-size: 16px;
    font-weight: ${tokens.fontWeights500};
  }
`;
/**
 * Renders a value of the table or card view with formatting options
 */
const TableValueComponent = React.memo(({ value, definition, locale, isLeftPosition: isLeftPosition, i, isInTable, }) => {
    const formattedValue = definition?.valueFormat === 'number'
        ? formatNumber(Number(value), locale, definition?.decimalDigits)
        : value;
    const onClick = () => definition?.onClick?.(i);
    const tooltipMessage = resolveValueOrCallback(definition?.displayOptions?.tooltipText, i);
    const getValueBody = () => {
        const renderedAs = resolveValueOrCallback(definition?.renderedAs, i);
        switch (renderedAs) {
            case 'pill':
                const colorVariant = resolveValueOrCallback(definition?.displayOptions?.colorVariant, i);
                // The span is needed here so the tooltip component can apply on the pill.
                return (React.createElement("span", { className: "db-table-pill-wrapper" },
                    React.createElement(Pill, { size: "S", onClick: onClick, colorVariant: colorVariant ?? undefined, pillRole: "status", fill: !!colorVariant }, formattedValue)));
            case 'link':
                if (isInTable) {
                    return React.createElement(StyledLinkForTable, { onClick: onClick }, formattedValue);
                }
                return isLeftPosition ? (React.createElement(StyledLargeLink, { onClick: onClick }, formattedValue)) : (React.createElement(StyledLink$1, { onClick: onClick }, formattedValue));
            default:
                return React.createElement("span", { className: "db-table-value" }, formattedValue);
        }
    };
    if (tooltipMessage) {
        return React.createElement(Tooltip$1, { message: tooltipMessage }, getValueBody());
    }
    return getValueBody();
});
TableValueComponent.displayName = 'TableValueComponent';

const CardRow = React.memo(({ row, rowDefinition, locale, hasImage, hasIcon, canSelect, isSelected, onSelectionChange, style, }) => {
    return (React.createElement("div", { style: style, key: row.i, className: isSelected ? 'db-card-row db-table-selected' : 'db-card-row', "data-testid": `db-card-row-${row.i}` },
        canSelect && (React.createElement("span", { className: "db-table-card-select" },
            React.createElement(Checkbox, { "data-testid": `db-table-card-select-${row.i}`, checked: isSelected, onChange: () => onSelectionChange(row.i) }))),
        (hasImage || hasIcon) && (React.createElement("span", { className: "db-table-card-image" },
            hasImage && (React.createElement(Portrait, { size: "ML", src: row.image, initials: String(row.title), alt: String(row.title) })),
            hasIcon && React.createElement(Icon, { fontSize: "large", type: row.icon ?? 'none' }))),
        React.createElement("div", { className: "db-table-card-content" },
            React.createElement("div", { className: "db-table-card-row" },
                React.createElement("span", { className: "db-table-card-title", "data-testid": `db-card-row-field-${row.i}-title` },
                    React.createElement(TableValueComponent, { value: row.title, locale: locale, definition: rowDefinition.title, i: row.i, isLeftPosition: true })),
                row.titleRight && (React.createElement("span", { className: "db-table-card-row-right", "data-testid": `db-card-row-field-${row.i}-title-right` },
                    React.createElement(TableValueComponent, { value: row.titleRight, locale: locale, definition: rowDefinition.titleRight, i: row.i, isLeftPosition: false })))),
            React.createElement("div", { className: "db-table-card-row" },
                row.line2 && (React.createElement("div", { className: "db-table-card-line-2", "data-testid": `db-card-row-field-${row.i}-line2` },
                    React.createElement(TableValueComponent, { value: row.line2, locale: locale, definition: rowDefinition.line2, i: row.i, isLeftPosition: true }))),
                row.line2Right && (React.createElement("span", { className: "db-table-card-row-right", "data-testid": `db-card-row-field-${row.i}-line2-right` },
                    React.createElement(TableValueComponent, { value: row.line2Right, locale: locale, definition: rowDefinition.line2Right, i: row.i, isLeftPosition: false })))),
            React.createElement("div", { className: "db-table-card-row" },
                row.line3 && (React.createElement("div", { className: "db-table-card-line-3", "data-testid": `db-card-row-field-${row.i}-line3` },
                    React.createElement(TableValueComponent, { value: row.line3, locale: locale, definition: rowDefinition.line3, i: row.i }))),
                row.line3Right && (React.createElement("span", { className: "db-table-card-row-right", "data-testid": `db-card-row-field-${row.i}-line3-right` },
                    React.createElement(TableValueComponent, { value: row.line3Right, locale: locale, definition: rowDefinition.line3Right, i: row.i })))))));
});
CardRow.displayName = 'CardRow';

const CardBody = React.memo(({ canSelect, data, hasIcon, hasImage, height, loadMoreRows: loadMore, locale, onSelectionChange, rowDefinition, selectedRows, setHasAllDataLoaded, }) => {
    const infiniteLoaderRef = React.useRef(null);
    const [loadedMap, setLoadedMap] = React.useState(data.map(() => TableRowStatus.LOADED));
    const isRowLoaded = React.useCallback(({ index }) => {
        return loadedMap[index] != null;
    }, [loadedMap]);
    const rowCount = React.useMemo(() => {
        return loadedMap.filter(l => l === TableRowStatus.LOADED).length;
    }, [loadedMap]);
    const reload = React.useCallback(() => {
        infiniteLoaderRef.current?.resetLoadMoreRowsCache(true);
    }, []);
    const rowRenderer = React.useCallback(({ style, index }) => {
        if (loadedMap[index] === TableRowStatus.LOADING) {
            return (React.createElement("div", { style: style, className: "db-row-loading" },
                React.createElement(Loader, { size: "small" })));
        }
        if (loadedMap[index] === TableRowStatus.FAILED) {
            return (React.createElement("div", { style: style, className: "db-row-error" },
                React.createElement(Button, { buttonType: "tertiary", fullWidth: true, destructive: true, iconType: "refresh", onClick: reload }, "Reload")));
        }
        const rowData = data[index];
        if (!rowData) {
            return null;
        }
        return (React.createElement(CardRow, { style: omit(style, 'columnWidth'), key: rowData.i, row: rowData, rowDefinition: rowDefinition, locale: locale, hasImage: hasImage, hasIcon: hasIcon, canSelect: canSelect, isSelected: selectedRows.includes(rowData.i), onSelectionChange: onSelectionChange }));
    }, [
        data,
        canSelect,
        hasIcon,
        hasImage,
        locale,
        onSelectionChange,
        rowDefinition,
        selectedRows,
        loadedMap,
        reload,
    ]);
    const loadMoreRows = React.useCallback(async ({ startIndex, stopIndex }) => {
        const pageSize = stopIndex - startIndex + 1;
        const pageCount = Math.ceil((startIndex + 1) / pageSize);
        let after = data[startIndex - 1];
        let first = pageSize;
        if (loadedMap[startIndex - 1] !== TableRowStatus.LOADED) {
            const idx = loadedMap
                .slice(0, startIndex)
                .reverse()
                .findIndex(item => item === TableRowStatus.LOADED);
            if (idx !== -1) {
                const newStartIndex = startIndex - idx - 1;
                after = data[newStartIndex];
                first = stopIndex - newStartIndex + 1;
            }
        }
        try {
            setLoadedMap(current => {
                const clone = [...current];
                for (let i = startIndex; i <= stopIndex; i++) {
                    clone[i] = TableRowStatus.LOADING;
                }
                return clone;
            });
            const newRows = await loadMore?.({
                startIndex,
                stopIndex,
                pageCount,
                pageSize,
                after,
                first,
            });
            // No rows returned or same rows as before
            if (newRows?.length === 0 || isEqual(newRows, data)) {
                setHasAllDataLoaded(true);
                setLoadedMap(current => {
                    const clone = [...current];
                    for (let i = startIndex; i <= stopIndex; i++) {
                        clone[i] = TableRowStatus.STALE;
                    }
                    return clone;
                });
                return;
            }
            // Less rows than pageSize
            if (newRows && newRows?.length < pageSize) {
                setHasAllDataLoaded(true);
                setLoadedMap(current => {
                    const clone = [...current];
                    for (let i = startIndex + newRows.length - 1; i <= stopIndex; i++) {
                        clone[i] = TableRowStatus.STALE;
                    }
                    return clone;
                });
                return;
            }
            setLoadedMap(current => {
                const clone = [...current];
                for (let i = startIndex; i <= stopIndex; i++) {
                    if (newRows?.[i - startIndex]) {
                        clone[i] = TableRowStatus.LOADED;
                    }
                    else {
                        clone[i] = TableRowStatus.STALE;
                    }
                }
                return clone;
            });
        }
        catch (err) {
            setLoadedMap(current => {
                const clone = [...current];
                for (let i = startIndex; i <= stopIndex; i++) {
                    clone[i] = TableRowStatus.FAILED;
                }
                return clone;
            });
        }
    }, [loadMore, data, loadedMap, setHasAllDataLoaded]);
    return (React.createElement("div", { className: "db-table-card-representation", "data-testid": "db-table-card-representation", style: { height: height || '90%' } },
        React.createElement(InfiniteLoader, { ref: infiniteLoaderRef, isRowLoaded: isRowLoaded, loadMoreRows: loadMoreRows, rowCount: Number.MAX_SAFE_INTEGER, minimumBatchSize: 20 }, ({ onRowsRendered, registerChild: registerInfiniteLoaderChild }) => (React.createElement(AutoSizer, { disableHeight: true, "data-testid": "db-table-table-representation", defaultHeight: height || 100 }, ({ width }) => {
            return (React.createElement(List, { ref: registerInfiniteLoaderChild, onRowsRendered: onRowsRendered, width: width, containerStyle: { width: '100%' }, height: height || 100, rowHeight: 86, rowCount: rowCount, rowRenderer: rowRenderer, estimatedRowSize: 20 }));
        })))));
});
CardBody.displayName = 'CardBody';

const SELECT_COLUMN_WIDTH = 40;
const TableRow = React.memo(({ row, rowDefinition, locale, canSelect, isSelected, hasImage, hasIcon, onSelectionChange, style: incomingStyle = {}, }) => {
    const rowDefs = React.useMemo(() => Object.keys(rowDefinition).filter(k => k !== 'icon' && k !== 'image'), [rowDefinition]);
    const style = React.useMemo(() => omit(incomingStyle, 'columnWidth'), [incomingStyle]);
    return (React.createElement("div", { key: row.i, className: "db-table-table-row ReactVirtualized__Table__row", "data-testid": "db-table-table-row", style: style, role: "row" },
        canSelect && (React.createElement("div", { key: "__selection", className: "db-table-table-selection-column db-table-table-cell", style: {
                width: SELECT_COLUMN_WIDTH,
                minWidth: SELECT_COLUMN_WIDTH,
                maxWidth: SELECT_COLUMN_WIDTH,
                verticalAlign: 'middle',
            }, role: "cell" },
            React.createElement(Checkbox, { checked: isSelected, onChange: () => onSelectionChange(row.i), "data-testid": `db-table-table-select-${row.i}` }))),
        rowDefs.map((k, i) => (React.createElement("div", { key: k, role: "cell", className: rowDefinition?.[k]?.valueFormat === 'number'
                ? 'db-table-table-cell db-table-table-numeric-column'
                : 'db-table-table-cell', style: {
                width: incomingStyle.columnWidth,
                minWidth: incomingStyle.columnWidth,
                maxWidth: incomingStyle.columnWidth,
                verticalAlign: 'middle',
            } },
            React.createElement("div", { className: "db-table-table-cell-wrapper" },
                i === 0 && hasImage && (React.createElement("span", { className: "db-table-table-image" },
                    React.createElement(Portrait, { size: "XS", src: row.image, initials: String(row[k]), alt: String(row[k]) }))),
                i === 0 && hasIcon && (React.createElement("span", { className: "db-table-table-icon" },
                    React.createElement(Icon, { fontSize: "small", type: row.icon ?? 'none' }))),
                React.createElement(TableValueComponent, { i: row.i, value: row[k], definition: rowDefinition[k], locale: locale, isInTable: true })))))));
});
TableRow.displayName = 'TableRow';

const MIN_COLUMN_WIDTH = 100;
const TableBody = React.memo(({ canSelect, data, hasIcon, hasImage, height, loadMoreRows: loadMore, locale, onSelectAllChange, onSelectionChange, rowDefinition, selectedRows, setHasAllDataLoaded, }) => {
    const infiniteLoaderRef = React.useRef(null);
    const [loadedMap, setLoadedMap] = React.useState(data.map(() => TableRowStatus.LOADED));
    const isRowLoaded = React.useCallback(({ index }) => {
        return loadedMap[index] != null;
    }, [loadedMap]);
    const reload = React.useCallback(() => {
        infiniteLoaderRef.current?.resetLoadMoreRowsCache(true);
    }, []);
    const rowRenderer = React.useCallback(({ rowData, style, index }) => {
        if (loadedMap[index] === TableRowStatus.LOADING) {
            return (React.createElement("div", { style: style, className: "db-row-loading" },
                React.createElement(Loader, { size: "small" })));
        }
        if (loadedMap[index] === TableRowStatus.FAILED) {
            return (React.createElement("div", { style: style, className: "db-row-error" },
                React.createElement(Button, { buttonType: "tertiary", fullWidth: true, destructive: true, iconType: "refresh", onClick: reload }, "Reload")));
        }
        if (!rowData) {
            return null;
        }
        return (React.createElement(TableRow, { style: style, key: rowData.i, row: rowData, rowDefinition: rowDefinition, isSelected: selectedRows.includes(rowData.i), onSelectionChange: onSelectionChange, locale: locale, canSelect: canSelect, hasIcon: hasIcon, hasImage: hasImage }));
    }, [
        canSelect,
        hasIcon,
        hasImage,
        locale,
        onSelectionChange,
        rowDefinition,
        selectedRows,
        loadedMap,
        reload,
    ]);
    const loadMoreRows = React.useCallback(async ({ startIndex, stopIndex }) => {
        const pageSize = stopIndex - startIndex + 1;
        const pageCount = Math.ceil((startIndex + 1) / pageSize);
        let after = data[startIndex - 1];
        let first = pageSize;
        if (loadedMap[startIndex - 1] !== TableRowStatus.LOADED) {
            const idx = loadedMap
                .slice(0, startIndex)
                .reverse()
                .findIndex(item => item === TableRowStatus.LOADED);
            if (idx !== -1) {
                const newStartIndex = startIndex - idx - 1;
                after = data[newStartIndex];
                first = stopIndex - newStartIndex + 1;
            }
        }
        try {
            setLoadedMap(current => {
                const clone = [...current];
                for (let i = startIndex; i <= stopIndex; i++) {
                    clone[i] = TableRowStatus.LOADING;
                }
                return clone;
            });
            const newRows = await loadMore?.({
                startIndex,
                stopIndex,
                pageCount,
                pageSize,
                after,
                first,
            });
            // No rows returned or same rows as before
            if (newRows?.length === 0 || isEqual(newRows, data)) {
                setHasAllDataLoaded(true);
                setLoadedMap(current => {
                    const clone = [...current];
                    for (let i = startIndex; i <= stopIndex; i++) {
                        clone[i] = TableRowStatus.STALE;
                    }
                    return clone;
                });
                return;
            }
            // Less rows than pageSize
            if (newRows && newRows?.length < pageSize) {
                setHasAllDataLoaded(true);
                setLoadedMap(current => {
                    const clone = [...current];
                    for (let i = startIndex + newRows.length - 1; i <= stopIndex; i++) {
                        clone[i] = TableRowStatus.STALE;
                    }
                    return clone;
                });
                return;
            }
            setLoadedMap(current => {
                const clone = [...current];
                for (let i = startIndex; i <= stopIndex; i++) {
                    if (newRows?.[i - startIndex]) {
                        clone[i] = TableRowStatus.LOADED;
                    }
                    else {
                        clone[i] = TableRowStatus.STALE;
                    }
                }
                return clone;
            });
        }
        catch (err) {
            setLoadedMap(current => {
                const clone = [...current];
                for (let i = startIndex; i <= stopIndex; i++) {
                    clone[i] = TableRowStatus.FAILED;
                }
                return clone;
            });
        }
    }, [loadMore, data, loadedMap, setHasAllDataLoaded]);
    const rowDefs = React.useMemo(() => Object.keys(rowDefinition).filter(k => k !== 'icon' && k !== 'image'), [rowDefinition]);
    const headerRenderer = React.useCallback(({ style }) => {
        const { columnWidth } = style;
        return (React.createElement("div", { className: "db-table-table-header", style: omit(style, 'columnWidth') },
            React.createElement("div", { className: "db-table-table-header-row", role: "row" },
                canSelect && (React.createElement("div", { key: "__selection", className: "db-table-table-selection-column-select-all db-table-header-cell", role: "columnheader", style: {
                        width: SELECT_COLUMN_WIDTH,
                        minWidth: SELECT_COLUMN_WIDTH,
                        maxWidth: SELECT_COLUMN_WIDTH,
                    } },
                    React.createElement(Checkbox, { "data-testid": "db-table-table-select-all", checked: selectedRows.length === data?.length, onChange: ev => onSelectAllChange(ev.target.checked) }))),
                rowDefs.map(k => (React.createElement("div", { key: k, style: {
                        width: columnWidth,
                        minWidth: columnWidth,
                        maxWidth: columnWidth,
                    }, "data-testid": "db-table-table-selection-column-header", role: "columnheader", className: rowDefinition[k]?.valueFormat === 'number'
                        ? 'db-table-header-cell db-table-table-selection-column-header-number'
                        : 'db-table-header-cell' },
                    React.createElement("span", { style: { boxSizing: 'border-box' } }, rowDefinition[k]?.title)))))));
    }, [rowDefinition, canSelect, data?.length, onSelectAllChange, selectedRows.length, rowDefs]);
    const rowGetter = React.useCallback(({ index }) => {
        return data[index];
    }, [data]);
    const rowCount = React.useMemo(() => {
        return loadedMap.filter(l => l === TableRowStatus.LOADED).length;
    }, [loadedMap]);
    const columnCount = React.useMemo(() => rowDefs.length, [rowDefs.length]);
    return (React.createElement("div", { "data-testid": "db-table-table-representation", className: "db-table-table-representation", style: { maxHeight: height || '90%', overflowY: 'hidden' } },
        React.createElement(InfiniteLoader, { ref: infiniteLoaderRef, isRowLoaded: isRowLoaded, loadMoreRows: loadMoreRows, rowCount: Number.MAX_SAFE_INTEGER, minimumBatchSize: 20, threshold: 8 }, ({ onRowsRendered, registerChild: registerInfiniteLoaderChild }) => (React.createElement(AutoSizer, { disableHeight: true, "data-testid": "db-table-table-representation", defaultHeight: height || 100 }, ({ width: autoWidth }) => {
            return (React.createElement(ColumnSizer, { columnMaxWidth: undefined, columnMinWidth: MIN_COLUMN_WIDTH, columnCount: columnCount, width: Math.max(autoWidth, rowDefs.length * MIN_COLUMN_WIDTH + (canSelect ? SELECT_COLUMN_WIDTH : 0)) }, ({ adjustedWidth, registerChild: registerColumnSizerChild, columnWidth }) => {
                const colWidth = canSelect
                    ? columnWidth - SELECT_COLUMN_WIDTH / columnCount
                    : columnWidth;
                return (React.createElement(Table, { ref: r => {
                        registerInfiniteLoaderChild(r);
                        registerColumnSizerChild(r);
                    }, columnWidth: colWidth, estimatedColumnSize: rowDefs.length, onRowsRendered: onRowsRendered, width: adjustedWidth, containerStyle: { width: '100%' }, gridStyle: { width: '100%', paddingBottom: '15px' }, height: height || 100, headerHeight: 34, rowHeight: 30, rowCount: rowCount, rowGetter: rowGetter, rowRenderer: rowRenderer, headerRowRenderer: headerRenderer, estimatedRowSize: 20, rowStyle: { columnWidth: colWidth }, headerStyle: { columnWidth: colWidth }, className: "db-table-table", containerRole: "", style: {
                        width: adjustedWidth,
                        boxSizing: 'border-box',
                    } }));
            }));
        })))));
});
TableBody.displayName = 'TableBody';

const StyledLink = styled(Link) `
  button {
    text-decoration: none;
    &:focus {
      background: transparent;
      outline: 3px solid #ffb500;
    }
  }
  span {
    color: #000;
    font-size: 12px;
    &:before {
      font-size: 12px;
    }
  }
`;
const StyledActionPopover = styled(ActionPopover) `
  display: inline-block;
`;
const TableOptionDropdown = React.memo(({ options, onSelect, selectedOption }) => {
    return (React.createElement(StyledActionPopover, { renderButton: ({ 'data-element': dataElement }) => (React.createElement(StyledLink, { onClick: noop, "data-element": dataElement, iconAlign: "right", icon: "caret_down" }, selectedOption ? options[selectedOption] : options[Object.keys(options)[0]])) }, Object.keys(options).map(k => (React.createElement(ActionPopoverItem, { key: k, onClick: () => onSelect(k) }, options[k])))));
});
TableOptionDropdown.displayName = 'TableOptionDropdown';

function usePrevious(value) {
    // The ref object is a generic container whose current property is mutable ...
    // ... and can hold any value, similar to an instance property on a class
    const ref = useRef();
    // Store current value in ref
    useEffect(() => {
        ref.current = value;
    }, [value]); // Only re-run if value changes
    // Return previous value (happens before update in useEffect above)
    return ref.current;
}

const useIsomorphicLayoutEffect = typeof window !== 'undefined' ? useLayoutEffect : useEffect;

function useEventListener(eventName, handler, element, options) {
    // Create a ref that stores handler
    const savedHandler = useRef(handler);
    useIsomorphicLayoutEffect(() => {
        savedHandler.current = handler;
    }, [handler]);
    useEffect(() => {
        // Define the listening target
        const targetElement = element?.current || window;
        if (!(targetElement && targetElement.addEventListener)) {
            return;
        }
        // Create event listener that calls handler function stored in ref
        const eventListener = event => savedHandler.current(event);
        targetElement.addEventListener(eventName, eventListener, options);
        // Remove event listener on cleanup
        return () => {
            targetElement.removeEventListener(eventName, eventListener);
        };
    }, [eventName, element, options]);
}

function useElementSize() {
    // Mutable values like 'ref.current' aren't valid dependencies
    // because mutating them doesn't re-render the component.
    // Instead, we use a state as a ref to be reactive.
    const [ref, setRef] = useState(null);
    const [size, setSize] = useState({
        width: 0,
        height: 0,
    });
    // Prevent too many rendering using useCallback
    const handleSize = useCallback(() => {
        setSize({
            width: ref?.offsetWidth || 0,
            height: ref?.offsetHeight || 0,
        });
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [ref?.offsetHeight, ref?.offsetWidth]);
    useEventListener('resize', handleSize);
    useIsomorphicLayoutEffect(() => {
        handleSize();
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [ref?.offsetHeight, ref?.offsetWidth]);
    return [setRef, size];
}

function defaultloadMoreRows() {
    return Promise.resolve([]);
}
const TableComponent = ({ callToActions, canChangeViewMode, canSelect, data, dataOptions = {}, emptyStateMessage = 'No items to display.', locale, mode, onDataOptionsChanged = noop, loadMoreRows = defaultloadMoreRows, onRepresentationModeChanged = noop, onSelectionChanged = noop, rowDefinition, selectedDataOptions = {}, stringLiterals, title, pageSize, }) => {
    const [representationMode, setRepresentationMode] = React.useState(mode);
    const [hasAllDataLoaded, setHasAllDataLoaded] = React.useState(false);
    const [selectedRows, setSelectedRows] = React.useState([]);
    const [dataOptionState, setDataOptionState] = React.useState(selectedDataOptions);
    const hasCallToActions = callToActions && Object.keys(callToActions).filter(a => !callToActions[a].isHidden).length > 0;
    const hadCallToActions = usePrevious(hasCallToActions);
    const hasData = (data ?? []).length > 0;
    const hasCardSelectAllCheckbox = hasData && canSelect && representationMode === 'card';
    const hasImage = hasData && Boolean(data?.find(d => d.image));
    const hasIcon = hasData && !hasImage && Boolean(data?.find(d => d.icon));
    const onSelectionChange = React.useCallback((i) => {
        const index = selectedRows.indexOf(i);
        const currentSelection = [...selectedRows];
        if (index === -1) {
            setSelectedRows([...selectedRows, i]);
            onSelectionChanged([...selectedRows, i]);
        }
        else {
            currentSelection.splice(index, 1);
            setSelectedRows(currentSelection);
            onSelectionChanged([...currentSelection]);
        }
    }, [onSelectionChanged, selectedRows]);
    const onDataOptionChange = React.useCallback((option, selectedOption) => {
        const newState = { ...dataOptionState, [option]: selectedOption };
        setDataOptionState(newState);
        onDataOptionsChanged({ ...newState });
    }, [dataOptionState, onDataOptionsChanged]);
    const onSelectAllChange = React.useCallback((newState) => {
        const newSelection = newState ? data?.map(d => d.i) ?? [] : [];
        setSelectedRows([...newSelection]);
        onSelectionChanged([...newSelection]);
    }, [data, onSelectionChanged]);
    const onModeChanged = React.useCallback((mode) => () => {
        if (onRepresentationModeChanged) {
            onRepresentationModeChanged(mode);
        }
        setRepresentationMode(mode);
    }, [onRepresentationModeChanged, setRepresentationMode]);
    const [headerRef, { height: headerHeight }] = useElementSize();
    const [tableComponentRef, { height: tableComponentHeight }] = useElementSize();
    const bodyHeight = React.useMemo(() => {
        const callToActionCompensation = (Number(hasCallToActions || false) - Number(hadCallToActions || false)) * 47;
        return tableComponentHeight - headerHeight - callToActionCompensation;
    }, [hadCallToActions, hasCallToActions, headerHeight, tableComponentHeight]);
    const dataLength = React.useMemo(() => data?.length || 0, [data?.length]);
    const notDivisibleByPageSize = pageSize ? dataLength % pageSize !== 0 : undefined;
    return (React.createElement("div", { className: "db-table-component", ref: tableComponentRef },
        React.createElement("div", { className: "db-table-header", ref: headerRef },
            React.createElement("div", { className: "db-table-header-title-line" },
                React.createElement("div", { className: "db-table-header-title-container db-draggable-element-indicator" },
                    React.createElement("div", { className: "db-table-header-title-wrapper" },
                        React.createElement(Typography, { variant: "h3", "data-testid": "db-table-title" }, title),
                        React.createElement("span", null,
                            React.createElement(Pill, { pillRole: "status", size: "S", fill: true, ml: "1" }, `${dataLength}${hasAllDataLoaded || dataLength === 0 || notDivisibleByPageSize ? '' : ' +'}`)))),
                canChangeViewMode && (React.createElement("div", { className: "db-table-header-representation-switch", "data-testid": "db-table-header-representation-switch" },
                    React.createElement("div", { className: representationMode === 'table'
                            ? 'db-table-header-representation-selected'
                            : undefined },
                        React.createElement(IconButton, { onAction: onModeChanged('table'), "data-testid": "db-table-header-representation-switch-table" },
                            React.createElement(Icon, { type: "csv", color: "#668592", tooltipMessage: resolveTranslationKey(stringLiterals, 'table_switch_to_table_view') }))),
                    React.createElement("div", { className: representationMode === 'card'
                            ? 'db-table-header-representation-selected'
                            : undefined },
                        React.createElement(IconButton, { onAction: onModeChanged('card'), "data-testid": "db-table-header-representation-switch-card" },
                            React.createElement(Icon, { type: "list_view", color: "#668592", tooltipMessage: resolveTranslationKey(stringLiterals, 'table_switch_to_card_view') })))))),
            (hasCardSelectAllCheckbox || Object.keys(dataOptions).length > 0) && (React.createElement("div", { className: "db-table-header-filter-line" },
                hasCardSelectAllCheckbox && (React.createElement("span", { key: "db-table-card-select-all" },
                    React.createElement("div", { className: "db-table-card-select db-table-card-select-all" },
                        React.createElement(Checkbox, { "data-testid": "db-table-card-select-all", checked: selectedRows.length === data?.length, onChange: ev => onSelectAllChange(ev.target.checked), mr: 1 }),
                        React.createElement("span", null, resolveTranslationKey(stringLiterals, 'table_select_all'))))),
                Object.keys(dataOptions).map(k => (React.createElement("span", { key: `db-table-data-option-${k}`, className: "db-table-header-filter-line-item" },
                    React.createElement(TableOptionDropdown, { onSelect: (option) => onDataOptionChange(k, option), selectedOption: dataOptionState[k], options: dataOptions[k] }))))))),
        React.createElement("div", { className: "db-table-body" },
            hasData && representationMode === 'card' && (React.createElement(CardBody, { canSelect: canSelect, data: data ?? [], hasIcon: hasIcon, hasImage: hasImage, height: bodyHeight, loadMoreRows: loadMoreRows, locale: locale, onSelectAllChange: onSelectAllChange, onSelectionChange: onSelectionChange, rowDefinition: rowDefinition, selectedRows: selectedRows, setHasAllDataLoaded: setHasAllDataLoaded })),
            hasData && representationMode === 'table' && (React.createElement(TableBody, { canSelect: canSelect, data: data ?? [], hasIcon: hasIcon, hasImage: hasImage, height: bodyHeight, loadMoreRows: loadMoreRows, locale: locale, onSelectAllChange: onSelectAllChange, onSelectionChange: onSelectionChange, rowDefinition: rowDefinition, selectedRows: selectedRows, setHasAllDataLoaded: setHasAllDataLoaded })),
            !hasData && (React.createElement("div", { className: "db-table-empty-state", "data-testid": "db-table-empty-state" },
                React.createElement(Typography, { variant: "h4", color: "blackOpacity55", margin: "auto" }, emptyStateMessage))))));
};

const Legend = ({ details, style, type }) => {
    return (React.createElement("div", { className: "db-chart-component__legend", style: style }, Object.entries(details).map(([key, item]) => {
        return (React.createElement("div", { key: key, className: "db-chart-component__legend_wrapper" },
            React.createElement("div", { style: {
                    width: 16,
                    height: 16,
                    marginRight: '0.5rem',
                    backgroundColor: item.color,
                    borderRadius: type === 'line' ? 8 : 0,
                } }),
            React.createElement("span", null, item.label)));
    })));
};

const triangleSize = 7;
const getBackgroundColor = (dark) => '#335C6D';
function TooltipRenderer(props) {
    const latestFit = useLatestWhen(props.anchor.fit, !!props.anchor.fit);
    if (!props.focusedDatum) {
        return null;
    }
    const { focusedDatum } = props;
    const { dark } = props.getOptions();
    const groupDatums = props.focusedDatum?.tooltipGroup ?? [];
    groupDatums.length;
    // Get the focused series' index
    groupDatums.findIndex(d => d === focusedDatum);
    const finalAlign = `${latestFit?.side}-${latestFit?.align}`;
    let arrowPosition;
    let triangleStyles;
    if (!arrowPosition) {
        if (finalAlign === 'left-center') {
            arrowPosition = 'right';
        }
        else if (finalAlign === 'right-center') {
            arrowPosition = 'left';
        }
        else if (finalAlign === 'top-center') {
            arrowPosition = 'bottom';
        }
        else if (finalAlign === 'bottom-center') {
            arrowPosition = 'top';
        }
        else if (finalAlign === 'right-start') {
            arrowPosition = 'bottomLeft';
        }
        else if (finalAlign === 'right-end') {
            arrowPosition = 'topLeft';
        }
        else if (finalAlign === 'left-start') {
            arrowPosition = 'bottomRight';
        }
        else if (finalAlign === 'left-end') {
            arrowPosition = 'topRight';
        }
    }
    const backgroundColor = props.backgroundColor || getBackgroundColor();
    if (arrowPosition === 'bottom') {
        triangleStyles = {
            top: '100%',
            left: '50%',
            transform: 'translate3d(-50%, 0%, 0)',
            borderLeft: `${triangleSize * 0.8}px solid transparent`,
            borderRight: `${triangleSize * 0.8}px solid transparent`,
            borderTop: `${triangleSize}px solid ${backgroundColor}`,
        };
    }
    else if (arrowPosition === 'top') {
        triangleStyles = {
            top: '0%',
            left: '50%',
            transform: 'translate3d(-50%, -100%, 0)',
            borderLeft: `${triangleSize * 0.8}px solid transparent`,
            borderRight: `${triangleSize * 0.8}px solid transparent`,
            borderBottom: `${triangleSize}px solid ${backgroundColor}`,
        };
    }
    else if (arrowPosition === 'right') {
        triangleStyles = {
            top: '50%',
            left: '100%',
            transform: 'translate3d(0%, -50%, 0)',
            borderTop: `${triangleSize * 0.8}px solid transparent`,
            borderBottom: `${triangleSize * 0.8}px solid transparent`,
            borderLeft: `${triangleSize}px solid ${backgroundColor}`,
        };
    }
    else if (arrowPosition === 'left') {
        triangleStyles = {
            top: '50%',
            left: '0%',
            transform: 'translate3d(-100%, -50%, 0)',
            borderTop: `${triangleSize * 0.8}px solid transparent`,
            borderBottom: `${triangleSize * 0.8}px solid transparent`,
            borderRight: `${triangleSize}px solid ${backgroundColor}`,
        };
    }
    else if (arrowPosition === 'topRight') {
        triangleStyles = {
            top: '0%',
            left: '100%',
            transform: 'translate3d(-50%, -50%, 0) rotate(-45deg)',
            borderTop: `${triangleSize * 0.8}px solid transparent`,
            borderBottom: `${triangleSize * 0.8}px solid transparent`,
            borderLeft: `${triangleSize * 2}px solid ${backgroundColor}`,
        };
    }
    else if (arrowPosition === 'bottomRight') {
        triangleStyles = {
            top: '100%',
            left: '100%',
            transform: 'translate3d(-50%, -50%, 0) rotate(45deg)',
            borderTop: `${triangleSize * 0.8}px solid transparent`,
            borderBottom: `${triangleSize * 0.8}px solid transparent`,
            borderLeft: `${triangleSize * 2}px solid ${backgroundColor}`,
        };
    }
    else if (arrowPosition === 'topLeft') {
        triangleStyles = {
            top: '0%',
            left: '0%',
            transform: 'translate3d(-50%, -50%, 0) rotate(45deg)',
            borderTop: `${triangleSize * 0.8}px solid transparent`,
            borderBottom: `${triangleSize * 0.8}px solid transparent`,
            borderRight: `${triangleSize * 2}px solid ${backgroundColor}`,
        };
    }
    else if (arrowPosition === 'bottomLeft') {
        triangleStyles = {
            top: '100%',
            left: '0%',
            transform: 'translate3d(-50%, -50%, 0) rotate(-45deg)',
            borderTop: `${triangleSize * 0.8}px solid transparent`,
            borderBottom: `${triangleSize * 0.8}px solid transparent`,
            borderRight: `${triangleSize * 2}px solid ${backgroundColor}`,
        };
    }
    else {
        triangleStyles = {
            opacity: 0,
        };
    }
    return (React.createElement("div", { style: {
            position: 'relative',
            // MC: custom font size
            fontSize: '14px',
            // MC: custom padding
            padding: '8px 12px',
            background: backgroundColor,
            color: dark ? 'black' : 'white',
        } },
        React.createElement("div", { style: {
                position: 'absolute',
                width: 0,
                height: 0,
                ...triangleStyles,
            } }),
        React.createElement("div", null, props.children)));
}

const DEFAULT_COLORS = [
    '#003349',
    '#0f83ab',
    '#faa43a',
    '#fd6868',
    '#53cfc9',
    '#a2d925',
    '#decf3f',
    '#734fe9',
    '#cd82ad',
    '#006d92',
    '#de7c00',
    '#f33232',
    '#3f9a80',
    '#53c200',
    '#d7af00',
    '#4c26c9',
    '#d44d99',
];
const CIRCLE_RADIUS = 5;
const ChartComponent = ({ data: inputData, options, primaryAxis: primary, primaryAxisLabel, secondaryAxes: secondary, secondaryAxisLabel, stringLiterals, }) => {
    const areAxesSwapped = options.type === 'bar' && options.areAxesSwapped;
    const areAxesStacked = options.type === 'bar' && options.areAxesStacked;
    const isHistogram = options.type === 'bar' && options.isHistogram;
    const data = React.useMemo(() => {
        return secondary.map(({ title, color, bind }) => ({
            id: bind,
            label: resolveTranslationKey(stringLiterals, title),
            /**
             * Currently this option is not taken into account (see https://github.com/TanStack/react-charts/issues/252)
             * To set the appropriate color "seriesColors" & "getSeriesStyle" are used instead.
             */
            color,
            data: inputData.map(d => ({ primary: get(d, primary.bind), secondary: get(d, bind) })),
        }));
    }, [primary, secondary, inputData, stringLiterals]);
    const primaryAxis = React.useMemo(() => ({
        position: areAxesSwapped ? 'left' : 'bottom',
        getValue: datum => datum.primary,
        showGrid: options.type === 'line',
        ...(isHistogram && { innerBandPadding: 0.005, innerSeriesBandPadding: 0.005 }),
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        scaleType: 'band',
        tickLabelRotationDeg: 0,
    }), [areAxesSwapped, isHistogram, options.type]);
    const max = React.useMemo(() => secondary
        .flatMap(({ bind }) => inputData.map(d => get(d, bind)))
        .reduce((acc, cur) => Math.max(Number(cur), acc), Number.NEGATIVE_INFINITY), [inputData, secondary]);
    const secondaryAxes = React.useMemo(() => [
        {
            position: areAxesSwapped ? 'bottom' : 'left',
            getValue: datum => datum.secondary,
            min: 0,
            max,
            showDatumElements: true,
            stacked: areAxesStacked,
            scaleType: 'linear',
            tickLabelRotationDeg: 0,
            /* eslint-disable @typescript-eslint/no-explicit-any */
            ...(options.type === 'line' && {
                elementType: 'line',
                curve: function (ctx) {
                    return {
                        areaStart: function () {
                            this._line = 0;
                        },
                        areaEnd: function () {
                            this._line = NaN;
                        },
                        lineStart: function () {
                            this._point = 0;
                        },
                        lineEnd: function () {
                            if (this._line || (this._line !== 0 && this._point === 1))
                                ctx.closePath();
                            this._line = 1 - this._line;
                        },
                        point: function (x, y) {
                            (x = +x), (y = +y);
                            switch (this._point) {
                                case 0:
                                    this._point = 1;
                                    this._line ? ctx.lineTo(x, y) : ctx.moveTo(x, y);
                                    break;
                                case 1:
                                    this._point = 2; // falls through
                                default:
                                    ctx.lineTo(x, y);
                                    break;
                            }
                        },
                    };
                },
            }),
            /* eslint-enable @typescript-eslint/no-explicit-any */
        },
    ], [areAxesSwapped, max, areAxesStacked, options.type]);
    // Dict of series by series ID (i.e. 'bind')
    const seriesMap = React.useMemo(() => {
        return secondary.reduce((acc, curr, index) => {
            acc[curr.bind] = { ...curr, color: curr.color || DEFAULT_COLORS[index] };
            return acc;
        }, {});
    }, [secondary]);
    // Dict of colors by series title
    const colors = React.useMemo(() => {
        return secondary.reduce((acc, curr, index) => {
            acc[curr.bind] = {
                color: curr.color || DEFAULT_COLORS[index],
                label: curr.title ?? curr.bind,
            };
            return acc;
        }, {});
    }, [secondary]);
    const getSeriesStyle = React.useCallback((series) => {
        return {
            fill: seriesMap[series.id].color,
            stroke: seriesMap[series.id].color,
            transition: 'none',
        };
    }, [seriesMap]);
    const tooltipRenderer = args => {
        if (!args.focusedDatum) {
            return null;
        }
        const { tooltipContent, color } = seriesMap[args.focusedDatum.seriesId];
        if (!tooltipContent) {
            return null;
        }
        const { primaryValue, secondaryValue, index } = args.focusedDatum;
        if (primaryValue === undefined || secondaryValue === undefined || index === undefined) {
            return;
        }
        return (React.createElement(TooltipRenderer, { ...args, backgroundColor: color },
            React.createElement("p", { style: { whiteSpace: 'pre-wrap' } }, tooltipContent({ primaryValue, secondaryValue, record: cloneDeep(inputData[index]) }))));
    };
    const getDatumStyle = () => ({
        // "r" is an internal property of react-charts used to determine the circle radius
        circle: { r: CIRCLE_RADIUS, strokeWidth: 2, fill: 'white' },
    });
    const onClickDatum = (args, event) => {
        const { primaryValue, secondaryValue, index, seriesId } = args || {};
        if (primaryValue === undefined ||
            secondaryValue === undefined ||
            index === undefined ||
            seriesId === undefined) {
            return;
        }
        const seriesDefinition = seriesMap[seriesId];
        if (seriesDefinition.onClick === undefined) {
            return;
        }
        const clickedPoint = { x: event.pageX, y: event.pageY };
        /**
         * Click event is always triggered on Voronoi rectangle so the following is needed to determine
         * if clicked point actually belongs to the chart.
         */
        if (Array.from(args?.element?.parentElement?.querySelectorAll(options.type === 'line' ? 'circle' : 'rect') ?? [])
            .map(rect => rect.getBoundingClientRect())
            .some(rect => {
            return (clickedPoint.x >= rect.x &&
                clickedPoint.x <= rect.x + rect.width &&
                clickedPoint.y >= rect.y &&
                clickedPoint.y <= rect.y + rect.height);
        })) {
            seriesDefinition.onClick({
                primaryValue,
                secondaryValue,
                record: cloneDeep(inputData[index]),
            });
        }
    };
    const hasLegend = secondary.length > 1;
    return (React.createElement("div", { className: "db-chart-component__y-wrapper" },
        React.createElement("div", { className: "db-chart-component", datatype: options.type, style: {
                // need to set a fixed height for the chart to work
                // subtract [ (legend's height + gap) + (primary axis height + gap) ]
                height: `calc(100% - ${hasLegend ? 17 + 32 : 0}px - ${primaryAxisLabel ? 17 + 8 : 0}px)`,
            } },
            secondaryAxisLabel && (React.createElement("div", { className: "db-chart-component__y-axis-title", style: {
                    // (x axis - chart bottom) + (primary axix label height) + (chart bottom - primary axis label)
                    marginBottom: `calc(24.8px + ${primaryAxisLabel ? 17 : 0}px + 0.5rem)`,
                } }, secondaryAxisLabel)),
            React.createElement("div", { className: "db-chart-component__x-wrapper" },
                React.createElement(Chart, { className: `db-chart-component__chart`, options: {
                        data,
                        defaultColors: DEFAULT_COLORS,
                        primaryAxis,
                        secondaryAxes,
                        tooltip: {
                            render: tooltipRenderer,
                        },
                        primaryCursor: { show: false, showLabel: false, showLine: false },
                        secondaryCursor: { show: false, showLabel: false, showLine: false },
                        getSeriesStyle,
                        getDatumStyle,
                        onClickDatum,
                        useIntersectionObserver: false,
                    } }),
                primaryAxisLabel && (React.createElement("div", { className: "db-chart-component__x-axis-title" }, primaryAxisLabel)))),
        hasLegend && (React.createElement(Legend, { details: colors, type: options.type, style: secondaryAxisLabel ? { marginLeft: '2rem' } : undefined }))));
};

const BarChartComponent = props => {
    const { areAxesSwapped = false, areAxesStacked = false, isHistogram = false } = props;
    return (React.createElement(ChartComponent, { ...props, options: { type: 'bar', areAxesSwapped, areAxesStacked, isHistogram } }));
};

const basicResolveImageUrl = (collectionId, resourceId) => `/assets/icons/${collectionId}/${resourceId}.svg`;
const VisualProcess = ({ value, onLinkClick, resolveImageUrl, }) => {
    return (React.createElement(VisualProcessEditorComponent, { canEdit: false, onLinkClick: onLinkClick || noop$1, icons: {}, name: "vp", getData: () => Promise.resolve({
            label: 'noop',
            value: 'noop',
        }), resolveImageUrl: resolveImageUrl || basicResolveImageUrl, onChange: noop$1, document: value }));
};

const LineChartComponent = props => {
    return React.createElement(ChartComponent, { ...props, options: { type: 'line' } });
};

const START_ANGLE = 90;
const END_ANGLE = -270;
const PADDING_ANGLE = 3;
const RADIAN = Math.PI / 180;
const DEFAULT_PIE_PART_COLORS = ['#00884A', '#0E74A7', '#007776', '#757575', '#C15708'];
const renderCustomizedLabel = ({ cx, cy, midAngle, innerRadius, outerRadius, percent, }) => {
    const radius = innerRadius + (outerRadius - innerRadius) * 0.5;
    const x = cx + radius * Math.cos(-midAngle * RADIAN);
    const y = cy + radius * Math.sin(-midAngle * RADIAN);
    return (React.createElement("text", { x: x, y: y, fill: "white", textAnchor: 'middle', dominantBaseline: "central" }, `${(percent * 100).toFixed(0)}%`));
};
const getCellColor = ({ piePartIndex, defaultColors, piePart, }) => {
    if (piePart.color)
        return piePart.color;
    return defaultColors[piePartIndex % defaultColors.length];
};
const PieChartWidget = ({ pieChartData, isDonut, stringLiterals }) => {
    return (React.createElement(ResponsiveContainer, { height: "100%", width: "100%" },
        React.createElement(PieChart, null,
            React.createElement(Pie, { innerRadius: isDonut ? '50%' : 0, dataKey: "value", nameKey: "title", data: pieChartData.map(d => ({
                    ...d,
                    title: d.title ? resolveTranslationKey(stringLiterals, d.title) : undefined,
                })), label: renderCustomizedLabel, labelLine: false, paddingAngle: PADDING_ANGLE, startAngle: START_ANGLE, endAngle: END_ANGLE }, pieChartData.map((piePart, piePartIndex) => (React.createElement(Cell, { key: `cell-${piePartIndex}`, fill: getCellColor({ piePartIndex, defaultColors: DEFAULT_PIE_PART_COLORS, piePart }) })))),
            React.createElement(Legend$1, { align: "center", verticalAlign: "bottom", iconType: "square" }))));
};

const EvolutionIndicator = ({ evolutionUnit, evolutionValue, stringLiterals, scale, locale, }) => {
    const classes = ['db-evolution-indicator'];
    let displayValue = '';
    let iconType = 'none';
    let iconColor = '';
    const formattedValue = formatNumber(Math.abs(evolutionValue), locale, scale);
    if (evolutionValue < 0) {
        displayValue = `- ${formattedValue} ${evolutionUnit}`;
        iconType = 'caret_down';
        iconColor = '#b1420c';
        classes.push('db-evolution-indicator-negative');
    }
    else if (evolutionValue > 0) {
        displayValue = `+ ${formattedValue} ${evolutionUnit}`;
        iconType = 'caret_up';
        iconColor = '#00884A';
        classes.push('db-evolution-indicator-positive');
    }
    else {
        displayValue = resolveTranslationKey(stringLiterals, 'no_change');
    }
    return (React.createElement("div", { className: classes.join(' ') },
        evolutionValue !== 0 && React.createElement(Icon, { type: iconType, color: iconColor }),
        React.createElement("span", null, displayValue)));
};

const renderActiveShape = props => {
    const { cx, cy, innerRadius, outerRadius, startAngle, endAngle, fill, payload } = props;
    return (React.createElement("g", null,
        React.createElement("text", { x: cx, y: cy, dy: 8, textAnchor: "middle", fill: fill, fontFamily: "Sage UI", fontSize: 42, fontWeight: 500 }, payload.name),
        React.createElement(Sector, { cx: cx, cy: cy, innerRadius: innerRadius, outerRadius: outerRadius, startAngle: startAngle, endAngle: endAngle, fill: fill })));
};
const Gauge = ({ 
// TODO: Replace with design token
color = tokens.colorsSemanticInfo500, data, evolutionUnit = '%', locale, scale = 0, stringLiterals, valueUnit = '%', }) => {
    const [containerRef, { width, height }] = useElementSize();
    const totalValue = data.totalValue || 100;
    const remainingValue = totalValue - data.value;
    const formattedValue = formatNumber(data.value, locale, scale);
    const chartData = [
        { name: 'RemainingValue', value: remainingValue, color: tokens.colorsUtilityDisabled400 },
        { name: `${formattedValue} ${valueUnit}`, value: data.value, color },
    ];
    const outerRadius = Math.min(height || 60, width || 60) / 2 - 12;
    return (React.createElement("div", { className: "db-widget-content widgetGauge" },
        data.evolutionValue != null && (React.createElement(EvolutionIndicator, { stringLiterals: stringLiterals, locale: locale, evolutionUnit: evolutionUnit, evolutionValue: data.evolutionValue, scale: scale })),
        React.createElement("div", { className: "db-gauge-container", ref: containerRef },
            React.createElement(PieChart, { width: width, height: height - 24 },
                React.createElement(Pie, { startAngle: -270, activeShape: renderActiveShape, activeIndex: 1, data: chartData, cx: "50%", cy: "50%", innerRadius: outerRadius - 30, outerRadius: outerRadius, fill: "#8884d8", paddingAngle: 2, dataKey: "value" }, chartData.map((entry, index) => (React.createElement(Cell, { key: `cell-${index}`, fill: entry.color }))))))));
};

function useParentElementSize() {
    // Mutable values like 'ref.current' aren't valid dependencies
    // because mutating them doesn't re-render the component.
    // Instead, we use a state as a ref to be reactive.
    const [ref, setRef] = useState(null);
    const [size, setSize] = useState({
        width: 0,
        height: 0,
    });
    // Prevent too many rendering using useCallback
    const handleSize = useCallback(() => {
        setSize({
            width: ref?.parentElement?.offsetWidth || 0,
            height: ref?.parentElement?.offsetHeight || 0,
        });
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [ref?.parentElement?.offsetHeight, ref?.parentElement?.offsetWidth]);
    useEventListener('resize', handleSize);
    useIsomorphicLayoutEffect(() => {
        handleSize();
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [ref?.parentElement?.offsetHeight, ref?.parentElement?.offsetWidth]);
    return [setRef, size];
}

function Preview() {
    const [containerRef, { height }] = useParentElementSize();
    return (React.createElement("div", { ref: containerRef, className: "db-preview-wrapper" },
        React.createElement(CarbonPreview, { height: `${height + 6}px` })));
}

const HtmlContainer = ({ data }) => (React.createElement("div", { dangerouslySetInnerHTML: { __html: data?.htmlContent ?? '' } }));
function RenderComponentType({ widget, locale, title, stringLiterals, }) {
    const commonProps = React.useMemo(() => ({
        stringLiterals,
        locale,
    }), [locale, stringLiterals]);
    switch (widget.type) {
        case 'bar-chart':
            return React.createElement(BarChartComponent, { ...widget.data, ...commonProps });
        case 'line-chart':
            return React.createElement(LineChartComponent, { ...widget.data, ...commonProps });
        case 'pie-chart':
            return React.createElement(PieChartWidget, { ...widget.data, ...commonProps });
        case 'basic':
            return React.createElement(HtmlContainer, { data: widget.data, ...commonProps });
        case 'line':
            return React.createElement(LineGraph, { data: widget.data, ...commonProps });
        case 'gauge':
            return React.createElement(Gauge, { ...widget.data, ...commonProps });
        case 'treemap':
            return React.createElement(TableTreeMap, { data: widget.data, ...commonProps });
        case 'preview':
            return React.createElement(Preview, null);
        case 'progress-tracker':
            return React.createElement(ProgressTracker, { ...widget.data, ...commonProps });
        case 'simple-indicator':
            return React.createElement(SimpleIndicator, { ...widget.data, ...commonProps });
        case 'multiple-indicators':
            return React.createElement(MultipleIndicators, { ...widget.data, ...commonProps });
        case 'simple-list':
            return React.createElement(SimpleList, { ...widget.data, ...commonProps });
        case 'horizontal-bar':
            return React.createElement(HorizontalBar, { ...widget.data, ...commonProps });
        case 'status-report':
            return React.createElement(StatusReport, { ...widget.data, ...commonProps });
        case 'tile-indicator':
            return React.createElement(TileIndicator, { ...widget.data, ...commonProps });
        case 'table':
            return (React.createElement(TableComponent, { ...widget.data, ...commonProps, title: title, callToActions: widget.callToActions }));
        case 'visual-process':
            return React.createElement(VisualProcess, { ...widget.data, ...commonProps });
        default:
            return React.createElement("div", null,
                "Unknown widget type: ",
                widget.type);
    }
}

const WidgetWrapper = React.memo(({ widget, locale, stringLiterals }) => {
    return (React.createElement("div", { "data-testid": `${widget.type}-${widget.i}`, className: `db-widget-skeleton-content db-widget-type-${widget.type}` },
        React.createElement(RenderComponentType, { widget: widget, locale: locale, title: widget.title, stringLiterals: stringLiterals })));
});
WidgetWrapper.displayName = 'WidgetWrapper';

class HttpService {
    source;
    instance;
    constructor() {
        this.instance = axios.create();
        this.source = axios.CancelToken.source();
    }
    get axiosInstance() {
        return this.instance;
    }
    resetCancelTokens() {
        this.source.cancel();
        this.source = axios.CancelToken.source();
    }
    get(url, config) {
        return this.instance.get(url, config);
    }
    delete(url, config) {
        return this.instance.delete(url, config);
    }
    post(url, data, config) {
        return this.instance.post(url, data, config);
    }
    put(url, data, config) {
        return this.instance.put(url, data, config);
    }
    patch(url, data, config) {
        return this.instance.patch(url, data, config);
    }
}
var httpService = new HttpService();

class DashboardHttpService {
    set requestInterceptor(requestInterceptor) {
        httpService.axiosInstance.interceptors.request.use(requestInterceptor?.onFulfilled, requestInterceptor?.onRejected);
    }
    set responseInterceptor(responseInterceptor) {
        httpService.axiosInstance.interceptors.response.use(responseInterceptor?.onFulfilled, responseInterceptor?.onRejected);
    }
}
var dashboard_http_service = new DashboardHttpService();

const getWidgetTestId = (widget) => {
    if (widget.title) {
        return `db-widget-container-${camelCase(widget.title)}`;
    }
    if (widget.type === 'simple-indicator') {
        const testIdTitle = widget.data.mainIndicator.subtitle ||
            widget.data.mainIndicator.title;
        if (testIdTitle) {
            return `db-widget-container-${camelCase(testIdTitle)}`;
        }
    }
    return undefined;
};
/** ## Widget re-usable skeleton container
 *
 * This is a UI-normalized container to be used as a widget with a header, a title, and some options.
 * This skeleton model is meant to be used inside the drag'n'drop dashboard layout/canvas.
 *
 * Props from **ToggleActionsMenu** are inherited.
 * **props.align** is not used: since the position of the menu is set to right, and then dropdown aligns from right.
 * The content (through **props.children**) is meant to be filled with pure rendered KPI.
 *
 * props.isSeparator enables an alternative simplified version of the widget, to be used as a separator.
 * So, it just has a title.
 * Same stylistic result can be achieved if the regular widget template is used 'under' a container with class name "db-section-separator".
 * In that second case, widget will be built as a regular one, but unnecessary parts will be hidden by CSS.
 *
 * @todo  Build footer area.
 * @todo  Build case when widget content is empty.
 * @todo  Build case when widget is inside of available-widgets list (+ set category).
 */
function WidgetSkeleton({ locale, stringLiterals, widget, isEditMode, onWidgetRemoved, }) {
    const [isLoadingTranslations, setLoadingTranslations] = React.useState(!!widget.url);
    const [widgetData, setWidgetData] = React.useState({});
    // In the BMS product a URL may be provided for each URL to fetch additional strings which can be used within the scope of the widget
    React.useEffect(() => {
        if (widget.url) {
            httpService
                .get(widget.url)
                .then(result => {
                const successToJSON = JSON.parse(result.data.value.json);
                setWidgetData({ data: translateObject(successToJSON, stringLiterals) });
            })
                .finally(() => {
                setLoadingTranslations(false);
            });
        }
    }, [widget.url]);
    // While the translations are getting fetched, we display the placeholder widget kind
    const widgetType = isLoadingTranslations ? 'preview' : widget?.type;
    const isSeparator = widgetType === 'separator';
    const testId = getWidgetTestId(widget);
    const actionDropdownTestId = widget.title
        ? `db-widget-container-dropdown-actions-${camelCase(widget.title)}`
        : undefined;
    const hasCallToActions = widget.callToActions &&
        Object.keys(widget.callToActions).filter(a => !widget.callToActions?.[a].isHidden).length > 0;
    const triggerOnClick = (item) => () => {
        if (item.action === 'closeWidget') {
            onWidgetRemoved(widget.i);
        }
        if (item.onClick) {
            item.onClick();
        }
    };
    const classNames = ['db-widget-skeleton'];
    classNames.push(isSeparator ? 'db-section-separator' : 'sectionWidget');
    if (hasCallToActions) {
        classNames.push('db-widget-skeleton-with-call-to-actions');
    }
    const widgetStringLiterals = {
        ...stringLiterals,
    };
    const renderDropdown = () => (React.createElement("div", { className: "db-header-right" }, isSeparator ? (isEditMode && React.createElement("a", { href: "#", onClick: () => onWidgetRemoved(widget.i), className: "close" })) : (React.createElement(ActionPopover, { "data-testid": actionDropdownTestId }, widget.dropdown?.items?.map(a => {
        if (a.isDivider) {
            return React.createElement(ActionPopoverDivider, { key: a.i });
        }
        return (React.createElement(ActionPopoverItem, { key: a.i, icon: a.icon, "data-testid": `db-widget-action-item-${camelCase(resolveTranslationKey(widgetStringLiterals, a.label))}`, onClick: triggerOnClick(a) }, resolveTranslationKey(widgetStringLiterals, a.label)));
    })))));
    const renderCallsToAction = () => (React.createElement("div", { className: "db-widget-call-to-actions-container" }, Object.keys(widget.callToActions ?? {}).map(a => {
        const action = widget.callToActions?.[a];
        if (action?.isHidden) {
            return null;
        }
        const buttonProps = {
            buttonType: 'tertiary',
            disabled: action?.isDisabled,
        };
        if (typeof action?.onClick === 'string') {
            buttonProps.href = action.onClick;
            buttonProps.target = action.target;
        }
        else {
            buttonProps.onClick = action?.onClick;
        }
        return (React.createElement(Button, { key: a, ...buttonProps, "data-testid": `db-widget-call-to-actions-${camelCase(action?.title)}` }, resolveTranslationKey(widgetStringLiterals, action?.title)));
    })));
    return (React.createElement("div", { className: classNames.join(' '), id: widget.i, "data-testid": testId },
        widget.title && widget.type !== 'table' && (React.createElement("div", { className: "db-widget-skeletonHeader" },
            React.createElement("div", { className: "skeletonHeader" },
                React.createElement("div", { className: "db-header-left db-draggable-element-indicator" },
                    React.createElement(Typography, { variant: isSeparator ? 'h2' : 'h3', fontWeight: isSeparator ? '700' : undefined }, resolveTranslationKey(widgetStringLiterals, widget.title)))))),
        widget.dropdown && renderDropdown(),
        !isSeparator && (React.createElement(React.Fragment, null,
            React.createElement(WidgetWrapper, { widget: { ...widget, ...widgetData }, locale: locale, stringLiterals: widgetStringLiterals }),
            hasCallToActions && renderCallsToAction()))));
}

const defaultLiterals = {
    action_edit: 'Edit',
    action_close: 'Close',
    action_delete: 'Delete',
    action_clone: 'Clone',
    action_share: 'Share',
    general_welcome_page: 'Welcome Page',
    general_invalid_json: 'Json not valid for current widget type',
    general_finish_editing: 'Finish Editing',
    dashboard_settings: 'Dashboard settings',
    table_select_all: 'Select All',
    table_variant_card: 'Card View',
    table_variant_table: 'Table View',
    table_variant_area: 'Area View',
    table_switch_to_table_view: 'Switch to table view',
    table_switch_to_card_view: 'Switch to card view',
    table_compare_with_previous_period: 'Compare with previous period',
    no_change: 'No change',
};

function useAnchors(initialStateAnchors, store, stringLiterals) {
    const initialAnchors = useRef(translateObject(initialStateAnchors, stringLiterals));
    const [myAnchors, setMyAnchors] = useState(initialAnchors.current);
    const anchorDistanceLightUp = 100;
    useEffect(() => {
        if (initialAnchors.current.some(anchorItem => !document.getElementById(anchorItem.id))) {
            return;
        }
        const newAnchorList = initialAnchors.current
            .filter(anchorItem => document.getElementById(anchorItem.id) !== null)
            .map(anchorItem => {
            const element = document.getElementById(anchorItem.id);
            return { ...anchorItem, currentPosition: element.getBoundingClientRect().top };
        });
        let selectAnchorLight;
        for (let i = 0; i < newAnchorList.length; i++) {
            if (newAnchorList[i].currentPosition <= anchorDistanceLightUp)
                selectAnchorLight = newAnchorList[i].id;
        }
        const setAnchorLight = newAnchorList.map(anchorItem => {
            let isVisible = false;
            anchorItem.id === selectAnchorLight ? (isVisible = true) : (isVisible = false);
            return { ...anchorItem, visible: isVisible };
        });
        setMyAnchors(setAnchorLight);
    }, [store.renderOnChange]);
    return myAnchors;
}

const getWidgetAnchors = (storeStateWidgets, displayAnchors) => {
    const widgetList = storeStateWidgets
        .filter(widget => widget.type == 'separator' && displayAnchors)
        .map(widget => ({ title: widget.title, id: widget.i, y: widget.y, visible: false }));
    return widgetList.sort((a, b) => (a.y > b.y ? 1 : b.y > a.y ? -1 : 0));
};

const NEW_WIDGET_ID = '__NEW_WIDGET';
const PENDING_WIDGET_ID = '__PENDING_ITEM';

const BackgroundPlaceholders = (layout) => {
    const colHorizontalMargin = 24;
    const colVerticalMargin = 24;
    const { width, cols, rowHeight } = layout;
    const dropAreaWidth = width - 48;
    const tileWidth = (dropAreaWidth - colHorizontalMargin * (cols - 1)) / cols;
    const tileHeight = rowHeight;
    const patternWidth = tileWidth + colHorizontalMargin;
    const patternHeight = tileHeight + colVerticalMargin;
    return (React.createElement("div", { className: "placeholders" },
        React.createElement("svg", { width: "100%", height: "100%" },
            React.createElement("pattern", { id: "pattern-placeholder", x: "0", y: "0", width: patternWidth, height: patternHeight, patternUnits: "userSpaceOnUse" },
                React.createElement("rect", { className: "placeholderTile", width: tileWidth, x: "0", y: "0", height: tileHeight, fill: tokens.colorsUtilityMajor040 })),
            React.createElement("rect", { x: "0", y: "0", width: "100%", height: "100%", fill: "url(#pattern-placeholder)" }))));
};
function DragDrop({ isEditMode, widgets, layout, locale, stringLiterals, onNewItemAdded, onWidgetRemoved, onWidgetLayoutChanged, }) {
    const [ref, size] = useElementSize();
    return (React.createElement("div", { className: "dragDropContainer", ref: ref }, size?.width && (React.createElement(React.Fragment, null,
        isEditMode ? React.createElement(BackgroundPlaceholders, { ...layout, ...size }) : null,
        React.createElement(ReactGridLayout, { className: "db-draggable-area", onLayoutChange: onWidgetLayoutChanged, layout: widgets, cols: layout.cols, rowHeight: layout.rowHeight, isDraggable: isEditMode, isDroppable: isEditMode, isResizable: isEditMode, droppingItem: { i: PENDING_WIDGET_ID, h: 2, w: 2 }, draggableCancel: '', onDrop: (_widgets, newItem, event) => {
                if (onNewItemAdded === undefined) {
                    return;
                }
                const dataTransfer = event.dataTransfer;
                const eventData = dataTransfer.getData('application/json');
                const parsedData = eventData ? JSON.parse(eventData) : {};
                onNewItemAdded({ ...newItem, i: NEW_WIDGET_ID, ...parsedData });
            }, width: size.width, draggableHandle: '.db-draggable-element-indicator', margin: [24, 24] }, widgets.map(widget => (React.createElement("div", { key: widget.i, className: "db-widget-container" },
            React.createElement(WidgetSkeleton, { widget: widget, locale: locale, stringLiterals: stringLiterals, isEditMode: isEditMode, onWidgetRemoved: onWidgetRemoved })))))))));
}

const ActionPopoverWrapper = styled.div `
  float: right;
`;
/** ## Flexible Masonry drag'n'drop canvas
 *
 * This component uses [React Grid Layout](https://github.com/react-grid-layout/react-grid-layout) as an alternative to [Gridstack](https://gridstackjs.com/).
 *
 * This component is meant to be used as full-available width.
 * It uses a responsive/elastic grid of 12 virtual columns.
 *
 * Each child container of the RGL component is initially positionable (x, y), has initial size (w, h), and other get/set properties (see [React Grid Layout Github repo](https://github.com/react-grid-layout/react-grid-layout) for more API details such as drag'n'drop, resize, etc...).
 * Use **WidgetSleketon** inside child containers of the RGL component.
 * Rendered graphs and KPI are the "children" of **WidgetSkeleton**, so we should virtually be able to display anything as a content.
 *
 * A widget can be dragged by its title.
 * Widget that are resizable can be resized by using bottom-right corner (cursor will change on hover).
 *
 * props.layout is the dataset to handle shapes, positions, and other properties.
 *
 * props.layout.i is different from props.layout.id .
 * props.layout.i refers to the "key" attribute used by React Grid Layout to identify each container. This is mandatory, and each key must be unique.
 * props.layout.id refers to the ID attribute of an HTML tag. Here it is used to define a target for the anchors links. This is optional, and each id must be unique.
 *
 * props.editMode is the starter point for the template edit mode for the user. Actually work in progress through CSS, but can be improved with specific state. FIXME PIFAL: to be completed.
 *
 * props.displayAnchors split the screen in 2 columns: main content on the left (12 columns grid), auto-generated anchors sticky on the right.
 *
 * props.highlightAnchorsFullSection defines the behavior of the anchors highlights: false (default) = once at a time | true = multiple anchors can be highlighted if they appear on screen.
 *
 * @todo  Bind dynamic properties, combine layout dataset with building calls of **WidgetSkeleton**.
 * @todo  Check if **WidgetSkeleton** children can be filled through layout dataset, or through the RGL component props.
 * @todo  Build case when widgets are just a simple list of available boxes to drag and drop inside the canvas.
 * @todo  FIXME: overlap of options menu depending of z-index of RGL elements.
 * @todo  Refine styles + check z-index of options menu + resize handler.
 * @todo  Define minimal sizes of widgets.
 * @todo  Check naming rules.
 * @todo  Optimize and split this long component into smaller parts.
 * @todo  Add smooth scroll animations when clicking on anchor link.
 */
function DragDropCanvas(props) {
    const stringLiterals = merge({}, defaultLiterals, props.stringLiterals);
    const [displayAnchors, setDisplayAnchors] = React.useState(props.areAnchorsDisplayed || false);
    React.useEffect(() => {
        setDisplayAnchors(props.areAnchorsDisplayed || false);
    }, [props.areAnchorsDisplayed]);
    const [widgets, setWidgets] = React.useState(props.widgets || []);
    React.useEffect(() => {
        setWidgets(props.widgets || []);
    }, [props.widgets]);
    const [isEditMode, setEditMode] = React.useState(props.isEditMode || false);
    React.useEffect(() => {
        setEditMode(props.isEditMode || false);
    }, [props.isEditMode]);
    const availableAnchors = getWidgetAnchors(widgets, displayAnchors);
    const anchorState = useAnchors(availableAnchors, props, stringLiterals);
    const popoverMenuItems = [
        {
            id: '0',
            action: 'editLayout',
            icon: 'edit',
            label: resolveTranslationKey(stringLiterals, 'action_edit'),
            onClick: () => (props.onEditAction ? props.onEditAction() : setEditMode(true)),
        },
        {
            id: '1',
            action: 'cloneLayout',
            icon: 'duplicate',
            label: resolveTranslationKey(stringLiterals, 'action_clone'),
        },
        {
            id: '2',
            action: 'shareLayout',
            icon: 'share',
            label: resolveTranslationKey(stringLiterals, 'action_share'),
        },
        {
            id: '3',
            action: 'deleteLayout',
            icon: 'delete',
            label: resolveTranslationKey(stringLiterals, 'action_delete'),
        },
    ];
    async function saveLayout() {
        setEditMode(false);
        const layout = {
            ...props.layout,
            isDraggable: false,
            isEditMode: false,
        };
        props.onSavedData?.({ layout, widgets });
        if (props.layout.url) {
            const params = {
                Json: {
                    ...props,
                    layout,
                    widgets,
                },
            };
            await httpService.put(props.layout.url, params, {
                headers: {
                    'content-type': 'text/plain',
                },
            });
        }
    }
    const onWidgetRemoved = React.useCallback((i) => {
        const updatedWidgets = [...widgets];
        const indexToRemove = widgets.findIndex(w => w.i === i);
        updatedWidgets.splice(indexToRemove, 0);
        setWidgets(updatedWidgets);
    }, [widgets]);
    const onNewItemAdded = React.useCallback((newItem) => {
        const newWidgets = [
            ...widgets,
            {
                ...newItem,
                ...props.newItem,
                type: newItem.type || 'basic',
                i: newItem.i,
                x: newItem.x,
                y: newItem.y,
                w: newItem.w,
                h: newItem.h,
            },
        ];
        setWidgets(newWidgets);
        if (props.onDataChange) {
            props.onDataChange({ layout: props.layout, widgets: newWidgets });
        }
    }, [widgets, props.onDataChange]);
    const onWidgetLayoutChanged = React.useCallback((newLayout) => {
        // First we check if the layout was changed with properties we care about
        const hasLayoutChanged = !!newLayout.find(l => {
            const existingWidget = widgets.find(w => w.i === l.i);
            if (!existingWidget) {
                return true;
            }
            return (existingWidget.x !== l.x ||
                existingWidget.y !== l.y ||
                existingWidget.h !== l.h ||
                existingWidget.w !== l.w);
        });
        // If it hasn't we stopped processing
        if (!hasLayoutChanged) {
            return;
        }
        const newWidgets = widgets
            .filter(widget => newLayout.find(l => widget.i === l.i))
            .map(widget => {
            const newLayoutProps = newLayout.find(l => widget.i === l.i);
            return {
                ...widget,
                x: newLayoutProps.x,
                y: newLayoutProps.y,
                w: newLayoutProps.w,
                h: newLayoutProps.h,
            };
        });
        setWidgets(newWidgets);
        if (props.onDataChange) {
            const filteredWidgets = newWidgets.filter(i => i.i !== PENDING_WIDGET_ID);
            if (!isEqual(widgets, filteredWidgets)) {
                props.onDataChange({ layout: props.layout, widgets: filteredWidgets });
            }
        }
    }, [widgets, props.onDataChange]);
    return (React.createElement("div", { id: "dashboardWrapper", className: isEditMode ? 'dashboardEditMode' : '' },
        !props.isHeaderHidden && (React.createElement("div", { id: "dashboardHeader" },
            React.createElement("div", { className: "actionsBar" }, isEditMode ? (React.createElement("button", { onClick: saveLayout, className: "exitEditMode" },
                React.createElement("span", null, resolveTranslationKey(stringLiterals, 'general_finish_editing')),
                "\u00A0",
                React.createElement("i", { className: "icon bmsIcon-tick" }))) : (React.createElement(React.Fragment, null,
                React.createElement("button", { className: "tabButton" }, resolveTranslationKey(stringLiterals, 'general_welcome_page')),
                React.createElement(ActionPopoverWrapper, null,
                    React.createElement(ActionPopover, { renderButton: ({ tabIndex, 'data-element': dataElement, ariaAttributes }) => (React.createElement("div", null,
                            React.createElement(ActionPopoverMenuButton, { buttonType: "tertiary", ariaAttributes: ariaAttributes, iconPosition: "after", iconType: "dropdown", size: "small", tabIndex: tabIndex, "data-element": dataElement }, resolveTranslationKey(stringLiterals, 'dashboard_settings')))) }, popoverMenuItems.map(a => (React.createElement(ActionPopoverItem, { key: a.id, icon: a.icon, onClick: a.onClick }, a.label)))))))))),
        !isEditMode ? (props.areAnchorsDisplayed ? (React.createElement("div", { id: "dashboard", className: (props.areAnchorsDisplayed ? 'withAnchors ' : '') +
                (isEditMode ? 'dashboardEditMode ' : '') },
            React.createElement("div", { className: "canvasFull" },
                React.createElement("div", { className: "canvasLeft" },
                    React.createElement(DragDrop, { layout: props.layout, widgets: widgets, isEditMode: isEditMode, locale: props.locale, stringLiterals: stringLiterals, onWidgetRemoved: onWidgetRemoved, onWidgetLayoutChanged: onWidgetLayoutChanged })),
                React.createElement("div", { className: "canvasRight" },
                    React.createElement("div", null,
                        React.createElement(Tabs, { align: "right", position: "left", selectedTabId: anchorState.find(t => t.visible)?.id, onTabChange: selectedId => (window.location.hash = selectedId) }, anchorState.map(a => (React.createElement(Tab, { key: a.id, tabId: a.id, title: a.title }))))))))) : (React.createElement(DragDrop, { layout: props.layout, widgets: widgets, isEditMode: isEditMode, locale: props.locale, stringLiterals: stringLiterals, onWidgetRemoved: onWidgetRemoved, onWidgetLayoutChanged: onWidgetLayoutChanged }))) : (React.createElement(DragDrop, { isEditMode: isEditMode, layout: props.layout, locale: props.locale, onNewItemAdded: onNewItemAdded, onWidgetLayoutChanged: onWidgetLayoutChanged, onWidgetRemoved: onWidgetRemoved, stringLiterals: stringLiterals, widgets: widgets }))));
}

function DragDropCanvasWrapper(props) {
    return React.createElement(DragDropCanvas, { ...props.json });
}

export { BarChartComponent as BarChart, DragDropCanvas, DragDropCanvasWrapper, Gauge, HorizontalBar, LineChartComponent as LineChart, LineGraph, MultipleIndicators, NEW_WIDGET_ID, PENDING_WIDGET_ID, PieChartWidget as PieChart, ProgressTracker, SimpleIndicator, SimpleList, StatusReport, TableComponent as Table, TableTreeMap, TileIndicator, VisualProcess, WidgetSkeleton, dashboard_http_service as dashboardHttpService, DragDropCanvasWrapper as default };
//# sourceMappingURL=index.js.map
